import mongoose from "mongoose";
import Users from "../model/users.model.js";
import agentTracking from "../model/agentTracking.model.js";

const ObjectId = mongoose.Types.ObjectId;


// Add New Tracking Point
export const addRoute = async (req, res) => {
    try {
        const { agentId, date, latitude, longitude, time } = req.body;

        // Check if a document with the same date and agentId exists
        const existingTracking = await agentTracking.findOne({ agentId, date });

        if (existingTracking) {
            // Check if a location with the same time exists
            const existingLocation = existingTracking.location.find(loc => loc.time === time);

            if (existingLocation) {
                // Location with the same time already exists, don't save again
                return res.status(200).json({
                    isSuccess: false,
                    message: "Location with the same time already exists"
                });
            } else {
                // Document exists, but location with the same time does not, so update the location array
                existingTracking.location.push({ latitude, longitude, time });
                await existingTracking.save();

                return res.status(200).json({
                    isSuccess: true,
                    message: "Location added to existing tracking data",
                    data: existingTracking
                });
            }
        } else {
            // Document does not exist, create a new one
            const newPoint = await agentTracking.create({
                agentId,
                date,
                location: [{ latitude, longitude, time }],
            });

            return res.status(200).json({
                isSuccess: true,
                message: "New tracking data created",
                data: newPoint
            });
        }
    } catch (error) {
        console.error("Error: ", error);
        return res.status(500).json({
            isSuccess: false,
            message: "Something went wrong.",
            error: error.message
        });
    }
};


// Get all Tracking Point of an Agent
export const getAllRouteByAgent = async (req, res) => {
    try {
        const allTrackingPoints = await agentTracking.find({ agentId: new ObjectId(req.params.agentId), date: req.params.date });
        res.send({
            isSuccess: true,
            data: allTrackingPoints
        })
    } catch (error) {
        console.error("Error: ", error);
        return res.status(500).send({
            isSuccess: false,
            message: "Something went wrong.",
            error: error.message
        });
    }
}
