import mongoose from "mongoose";
import Orders from "../model/order.model.js";
import Items from "../model/items.model.js";
import Orderitems from "../model/orderitem.model.js";
import Address from '../model/address.model.js';
import Customers from "../model/customer.model.js";
import Users from "../model/users.model.js";
const ObjectId = mongoose.Types.ObjectId;
import moment from 'moment';
import coinhists from "../model/coinhist.model.js";






// Total no. of customer
export const activeCust = async (req, res) => {

    const today = new Date();
    const sevenDaysAgo = new Date(today);
    sevenDaysAgo.setDate(sevenDaysAgo.getDate() - 7);

    const totalCustomersLast7Days = await Customers.countDocuments({
        isActive: true,
        createdAt: { $gte: sevenDaysAgo, $lt: today },
    });

    console.log("Total number of customers in the last 7 days:", totalCustomersLast7Days);

    const totalCustomersInDB = await Customers.countDocuments({ isActive: true });

    console.log("Total number of customers in the entire database:", totalCustomersInDB);


    if (totalCustomersLast7Days && totalCustomersInDB) {
        res.json({
            isSuccess: true,
            message: "Customer count.",
            count: totalCustomersInDB,
            count7: totalCustomersLast7Days
        });
    } else {
        res.json({
            isSuccess: false,
            message: "Something went wrong.",
            count: totalCustomersInDB,
            count7: totalCustomersLast7Days
        });
    }
}




// Total no. of orders
export const activeOrder = async (req, res) => {

    // const order = await Orders.find({}).count()
    const today = new Date();
    const sevenDaysAgo = new Date(today);
    sevenDaysAgo.setDate(today.getDate() - 7);

    const ordersLast7Days = await Orders.countDocuments({
        createdAt: { $gte: sevenDaysAgo, $lte: today }
    });

    const totalOrders = await Orders.countDocuments();

    console.log("Orders in the last 7 days:", ordersLast7Days);
    console.log("Total orders in the database:", totalOrders);


    if (ordersLast7Days && totalOrders) {
        res.json({
            isSuccess: true,
            message: "Order History count.",
            count: totalOrders,
            count7: ordersLast7Days,
        });
    } else {
        res.json({
            isSuccess: false,
            message: "Something went wrong.",
            count: totalOrders,
            count7: ordersLast7Days,
        });
    }
}




// Today's Order - Order Count and Estimated Amount
export const activeEst = async (req, res) => {


    const startDate = moment().subtract(7, 'days').startOf('day').toDate();
    const endDate = moment().endOf('day').toDate();

    const estPrice = await Orders.aggregate([
        {
            $match: {
                createdAt: {
                    $gte: startDate,
                    $lte: endDate
                }
            }
        },
        {
            $group: {
                _id: null,
                totalEstPrice: { $sum: "$est_price" }
            }
        }
    ]);

    const totalEstPriceLast7Days = estPrice.length > 0 ? estPrice[0].totalEstPrice : 0;
    console.log("Total Est Price in the Last 7 Days:", totalEstPriceLast7Days);

    const totalEstPriceAllOrders = await Orders.aggregate([
        {
            $group: {
                _id: null,
                totalEstPrice: { $sum: "$est_price" }
            }
        }
    ]);

    const totalEstPriceAllOrdersValue = totalEstPriceAllOrders.length > 0 ? totalEstPriceAllOrders[0].totalEstPrice : 0;
    console.log("Total Est Price of All Orders in the Database:", totalEstPriceAllOrdersValue);


    //   console.log("Total Est Price:", totalEstPrice.toFixed(2));


    if (totalEstPriceLast7Days && totalEstPriceAllOrdersValue) {
        res.json({
            isSuccess: true,
            message: "Total Estimate Price.",
            count: totalEstPriceAllOrdersValue.toFixed(2),
            count7: totalEstPriceLast7Days.toFixed(2)
        });
    } else {
        res.json({
            isSuccess: false,
            message: "Something went wrong.",
            count: totalEstPriceAllOrdersValue.toFixed(2),
            count7: totalEstPriceLast7Days.toFixed(2)
        });
    }
}




// Total Payment
export const activePayment = async (req, res) => {
    const statuses = ["processed", "Cash", "Payment successful"];
    const today = new Date();
    const sevenDaysAgo = new Date(today);
    sevenDaysAgo.setDate(today.getDate() - 7);

    const pipeline = [
        {
            $match: {
                status: { $in: statuses },
                createdAt: { $gte: sevenDaysAgo, $lte: today }
            }
        },
        {
            $group: {
                _id: null,
                totalEstPriceLast7Days: { $sum: "$est_price" }
            }
        },
        {
            $project: {
                _id: 0,
                totalEstPriceLast7Days: { $round: ["$totalEstPriceLast7Days", 2] }
            }
        },
        {
            $unionWith: {
                coll: "orders", // Replace "orders" with your actual collection name
                pipeline: [
                    {
                        $match: {
                            status: { $in: statuses }
                        }
                    },
                    {
                        $group: {
                            _id: null,
                            totalEstPriceAllOrders: { $sum: "$est_price" }
                        }
                    },
                    {
                        $project: {
                            _id: 0,
                            totalEstPriceAllOrders: { $round: ["$totalEstPriceAllOrders", 2] }
                        }
                    }
                ]
            }
        }
    ];

    const result = await Orders.aggregate(pipeline);
    //console.log("result", result, result.length, result[0].totalEstPriceAllOrders);
    var totalEstPriceLast7Days = 0
    var totalEstPriceAllOrders = 0
    if (result.length == 2) {
        totalEstPriceLast7Days = result.length > 0 ? result[0].totalEstPriceLast7Days : 0;
        totalEstPriceAllOrders = result.length > 1 ? result[1].totalEstPriceAllOrders : 0;
    } else if (result.length == 1) {
        totalEstPriceAllOrders = result[0].totalEstPriceAllOrders;
    } else {
        totalEstPriceLast7Days = 0
        totalEstPriceAllOrders = 0
    }


    console.log("Total Est Price (Last 7 Days):", totalEstPriceLast7Days);
    console.log("Total Est Price (All Orders):", totalEstPriceAllOrders);


    // console.log("Total Est Price for specified statuses:", totalEstPrice.toFixed(2));

    if (totalEstPriceLast7Days && totalEstPriceAllOrders) {
        res.json({
            isSuccess: true,
            message: "Total Payment.",
            count: totalEstPriceAllOrders,
            count7: totalEstPriceLast7Days,
        });
    } else {
        res.json({
            isSuccess: false,
            message: "Something went wrong.",
            count: totalEstPriceAllOrders,
            count7: totalEstPriceLast7Days,
        });
    }
}





// Monthly Avg. price
export const monthAvg = async (req, res) => {
    console.log(req.params.year);
    const pipeline = [
        {
            $addFields: {
                schedule_date_iso: {
                    $dateFromString: {
                        dateString: "$schedule_date",
                        format: "%d-%m-%Y"
                    }
                }
            }
        },
        {
            $match: {
                $and: [
                    {
                        $expr: {
                            $eq: [
                                { $year: "$schedule_date_iso" },
                                parseInt(req.params.year)
                            ]
                        }
                    },
                    {
                        $or: [
                            { status: "processed" },
                            { status: "cash" },
                            { status: "Payment successful" }
                        ]
                    }
                ]
            }
        },
        {
            $group: {
                _id: {
                    month: { $month: "$schedule_date_iso" },
                    year: { $year: "$schedule_date_iso" }
                },
                totalEstPrice: { $sum: "$est_price" }
            }
        },
        {
            $addFields: {
                totalEstPrice: { $round: ["$totalEstPrice", 2] }
            }
        },
        {
            $sort: {
                "_id.year": 1,
                "_id.month": 1
            }
        }
    ];



    const result = await Orders.aggregate(pipeline);

    if (result) {
        res.json({
            isSuccess: true,
            message: "Monthly Estimate amount.",
            result: result,
        });
    } else {
        res.json({
            isSuccess: false,
            message: "Something went wrong.",
        });
    }
}





// daily Avg. price
export const dailyAvg = async (req, res) => {

    const pipeline = [
        {
            $addFields: {
                schedule_date_iso: {
                    $dateFromString: {
                        dateString: "$schedule_date",
                        format: "%d-%m-%Y"
                    }
                }
            }
        },
        {
            $match: {
                $and: [
                    {
                        $expr: {
                            $eq: [
                                { $month: "$schedule_date_iso" },
                                parseInt(req.query.month)
                            ]
                        }
                    },
                    {
                        $expr: {
                            $eq: [
                                { $year: "$schedule_date_iso" },
                                parseInt(req.query.year)
                            ]
                        }
                    },
                    {
                        $or: [
                            { status: "processed" },
                            { status: "cash" },
                            { status: "Payment successful" }
                        ]
                    }
                ]
            }
        },
        {
            $group: {
                _id: {
                    schedule_date_iso: "$schedule_date_iso",
                    // You can add more fields based on your requirements
                },
                totalEstPrice: { $sum: "$est_price" }
            }
        },
        {
            $addFields: {
                totalEstPrice: { $round: ["$totalEstPrice", 2] }
            }
        },
        {
            $sort: {
                "_id.schedule_date_iso": 1
            }
        }
    ];






    const result = await Orders.aggregate(pipeline);


    if (result) {
        res.json({
            isSuccess: true,
            message: "Daily Estimate amount.",
            result: result,
        });
    } else {
        res.json({
            isSuccess: false,
            message: "Something went wrong.",
        });
    }


}





// Monthly last 6 months Avg. price
export const monthAvg6Month = async (req, res) => {
    console.log(req.params.year);
    const pipeline = [
        {
            $addFields: {
                schedule_date_iso: {
                    $dateFromString: {
                        dateString: "$schedule_date",
                        format: "%d-%m-%Y"
                    }
                }
            }
        },
        {
            $match: {
                $and: [
                    {
                        $expr: {
                            $gte: [
                                "$schedule_date_iso",
                                {
                                    $dateFromParts: {
                                        year: parseInt(req.params.year),
                                        month: { $subtract: [{ $month: new Date() }, 6] },
                                        day: 1
                                    }
                                }
                            ]
                        }
                    },
                    {
                        $or: [
                            { status: "processed" },
                            { status: "cash" },
                            { status: "Payment successful" }
                        ]
                    }
                ]
            }
        },
        {
            $group: {
                _id: {
                    month: { $month: "$schedule_date_iso" },
                    year: { $year: "$schedule_date_iso" }
                },
                totalEstPrice: { $sum: "$est_price" }
            }
        },
        {
            $addFields: {
                totalEstPrice: { $round: ["$totalEstPrice", 2] }
            }
        },
        {
            $sort: {
                "_id.year": 1,
                "_id.month": 1
            }
        }
    ];


    const result = await Orders.aggregate(pipeline);

    if (result) {
        res.json({
            isSuccess: true,
            message: "Last 6 month Estimate amount.",
            result: result,
        });
    } else {
        res.json({
            isSuccess: false,
            message: "Something went wrong.",
        });
    }
}





// Avg amount according to payment mode
export const payAvg = async (req, res) => {
    console.log(req.params.mode);
    if (req.params.mode == 'all') {
        const pipeline = [
            {
                $match: {
                    $or: [
                        { status: "processed" },
                        { status: "cash" },
                        { status: "Payment successful" }
                    ]
                }
            },
            {
                $group: {
                    _id: {
                        mode: "$mode" // Use $mode as the field to group by
                    },
                    totalEstPrice: { $sum: "$est_price" }
                }
            },
            {
                $addFields: {
                    totalEstPrice: { $round: ["$totalEstPrice", 2] }
                }
            },
            {
                $sort: {
                    "_id.mode": 1
                }
            }
        ];

        const result = await Orders.aggregate(pipeline);

        if (result) {
            res.json({
                isSuccess: true,
                message: "Avg amount of each payment mode.",
                result: result,
            });
        } else {
            res.json({
                isSuccess: false,
                message: "Something went wrong.",
            });
        }
    } else if (req.params.mode != 'all') {
        const mode = req.params.mode;

        const pipeline = [
            {
                $match: {
                    $and: [
                        {
                            $or: [
                                { status: "processed" },
                                { status: "cash" },
                                { status: "Payment successful" }
                            ]
                        },
                        { "mode": req.params.mode } // Add a condition for the specific mode
                    ]
                }
            },
            {
                $group: {
                    _id: {
                        mode: "$mode", // Dynamically group by the "mode" field
                    },
                    totalEstPrice: { $sum: "$est_price" }
                }
            },
            {
                $addFields: {
                    totalEstPrice: { $round: ["$totalEstPrice", 2] }
                }
            },
            {
                $sort: {
                    "_id.mode": 1
                }
            }
        ];




        const result = await Orders.aggregate(pipeline);

        if (result) {
            res.json({
                isSuccess: true,
                message: `Avg amount of ${mode} payment mode.`,
                result: result,
            });
        } else {
            res.json({
                isSuccess: false,
                message: "Something went wrong.",
            });
        }


    }

}





// Avg amount according to item-category
export const categoryAvg = async (req, res) => {
    const pipeline = [
        {
            $lookup: {
                from: "customers",
                localField: "customer_id",
                foreignField: "_id",
                as: "customerDetails"
            }
        },
        {
            $unwind: "$item"
        },
        {
            $lookup: {
                from: "items",
                localField: "item.itemid",
                foreignField: "_id",
                as: "itemDetails"
            }
        },
        {
            $unwind: "$itemDetails"
        },
        {
            $group: {
                _id: "$itemDetails._id", // Group by itemDetails _id
                totalWeight: { $sum: "$item.weight" }, // Assuming the weight field in the "item" array
                items: {
                    $push: {
                        item: "$item.itemid",
                        createdAt: "$createdAt"
                    }
                }
            }
        },
        {
            $sort: {
                createdAt: -1
            }
        }
    ]


    const result = await Orders.aggregate(pipeline);

    if (result) {
        res.json({
            isSuccess: true,
            message: "Last 6 month Estimate amount.",
            result: result,
        });
    } else {
        res.json({
            isSuccess: false,
            message: "Something went wrong.",
        });
    }
}







// Total estimate weight and price for each item-catagory 
export const categoryAvgItem = async (req, res) => {
    const pipeline = [
        {
            $match: {
                $or: [
                    { status: "processed" },
                    { status: "cash" }
                ]
            }
        },
        {
            $unwind: "$item"
        },
        {
            $lookup: {
                from: "items",
                localField: "item.itemid",
                foreignField: "_id",
                as: "orderedItems"
            }
        },
        {
            $unwind: "$orderedItems"
        },
        {
            $group: {
                _id: "$orderedItems.item_name",
                totalAmount: {
                    $sum: {
                        $multiply: [
                            { $toDouble: "$item.price" },
                            { $toDouble: "$item.weight" }
                        ]
                    }
                },
                totalWeight: { $sum: { $toDouble: "$item.weight" } },
                orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
            }
        },
        {
            $group: {
                _id: "$orderedItems.category", // Group by the category field within orderedItems
                items: {
                    $push: {
                        item_name: "$_id",
                        totalWeight: "$totalWeight",
                        totalAmount: "$totalAmount",
                        unit: "$orderedItems.unit"
                    }
                },
                totalAmount: { $sum: "$totalAmount" }, // Calculate the sum of totalAmount for each category
                totalWeight: { $sum: "$totalWeight" }  // Calculate the sum of totalWeight for each category
            }
        },
        {
            $lookup: {
                from: "itemcategories", // Assuming the collection name is itemcategories
                localField: "_id",
                foreignField: "_id",
                as: "categoryDetails"
            }
        },
        {
            $project: {
                _id: 0,
                category: "$_id",
                items: 1,
                categoryDetails: { $arrayElemAt: ["$categoryDetails.item_name", 0] }, // Include the joined category details
                totalAmount: 1, // Include the sum of totalAmount for each category
                totalWeight: 1  // Include the sum of totalWeight for each category
            }
        },
        {
            $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
        }
    ];



    const result = await Orders.aggregate(pipeline);

    if (result) {
        res.json({
            isSuccess: true,
            message: "Total estimate weight and price for each item-catagory.",
            result: result,
        });
    } else {
        res.json({
            isSuccess: false,
            message: "Something went wrong.",
        });
    }
}









// const pipeline = [
//     // {
//     //     $match: {
//     //         _id: new mongoose.Types.ObjectId(req.params.id)
//     //     }
//     // },
//     {
//         $lookup: {
//             from: "items",
//             localField: "item.itemid",
//             foreignField: "_id",
//             as: "itemDetails"
//         }
//     },
//     {
//         $group: {
//             _id: "$_id",
//             item: { $push: "$item" },
//             itemDetails: { $push: "$itemDetails" },
//         }
//     },
//     {
//         $sort: {
//             createdAt: -1 // 1 for ascending order, -1 for descending order
//         }
//     }
// ];

// const result = await Orders.aggregate(pipeline);


//agent or partner dashboard api for showing analytics data for current date(chandan)

export const agentAnalytics = async (req, res) => {
    const agentId = req.params.id;
    if (!agentId) {
      return res.json({ isSuccess: false, message: "id is required" });
    }
  
    const currentDate = new Date();
    const formattedDate = `${String(currentDate.getDate()).padStart(2, '0')}-${String(currentDate.getMonth() + 1).padStart(2, '0')}-${currentDate.getFullYear()}`;
    console.log("formatted date is", formattedDate);
  
    try {
      const result = await Orders.aggregate([
        {
          $match: {
            schedule_date: formattedDate,
            agent_id: new ObjectId(agentId)
          }
        },
        {
          $facet: {
            statusCounts: [
              {
                $group: {
                  _id: {
                    $cond: {
                      if: { $in: ["$status", ["Pickup Completed", "processed"]] },
                      then: "Completed",
                      else: "Other"
                    }
                  },
                  count: { $sum: 1 }
                }
              }
            ],
            itemDetails: [
              {
                $match: {
                  status: { $in: ["Pickup Completed", "processed","processing"] }
                }
              },
              {
                $unwind: { path: "$item" }
              },
              {
                $lookup: {
                  from: "items",
                  localField: "item.itemid",
                  foreignField: "_id",
                  as: "itemDetails"
                }
              },
              {
                $unwind: { path: "$itemDetails" }
              },
              {
                $group: {
                  _id: "$itemDetails.unit",
                  count: { $sum: 1 },
                  totalPc: {
                    $sum: {
                      $cond: [{ $eq: ["$itemDetails.unit", "Pc"] }, "$item.weight", 0]
                    }
                  },
                  totalKg: {
                    $sum: {
                      $cond: [{ $eq: ["$itemDetails.unit", "Kg"] }, "$item.weight", 0]
                    }
                  }
                }
              },
              {
                $project: {
                  _id: 0,
                  Name: "$_id",
                  count: 1,
                  totalPc: {
                    $cond: [{ $eq: ["$_id", "Pc"] }, "$totalPc", 0]
                  },
                  totalKg: {
                    $cond: [{ $eq: ["$_id", "Kg"] }, "$totalKg", 0]
                  }
                }
              }
            ]
          }
        }
      ]);
  
      // Process status counts
      let totalOrders = 0;
      let completedOrders = 0;
      let pendingOrders = 0;
  
      if (result[0] && result[0].statusCounts) {
        result[0].statusCounts.forEach(item => {
          if (item._id === "Completed") {
            completedOrders += item.count;
          } else if (item._id === "Other") {
            pendingOrders += item.count;
          }
          totalOrders += item.count;
        });
      }
  
      // Initialize totalPc and totalKg with default values
      let totalPc = 0;
      let totalKg = 0;
  
      // Process itemDetails if available
      if (result[0] && result[0].itemDetails) {
        result[0].itemDetails.forEach(item => {
          if (item.Name === "Pc") {
            totalPc = item.totalPc;
          } else if (item.Name === "Kg") {
            totalKg = item.totalKg;
          }
        });
      }
  
      // Ensure totalPc and totalKg are always included with default values
      res.json({
        isSuccess: true,
        message: "Analytics data fetched successfully.",
        TodayOrders: {
          totalOrders,
          completedOrders,
          pendingOrders
        },
        CollectedScrap: {
          totalPc,
          totalKg
        }
      });
    } catch (err) {
      console.error(err);
      res.json({
        isSuccess: false,
        message: "Something went wrong.",
      });
    }
  };

export const customerAnalyticsData = async (req, res) => {
    const type=req.query.type
    const customerId=req.params.customerId
    if(!type || !customerId){
        return res.json({ isSuccess: false, message: "type and customerId is required" });
    }
    let typeMatchCriteria = {};
    const currentDate = new Date();
    if (type === "lm") {
        // Last month
        const firstDayLastMonth = new Date(currentDate.getFullYear(), currentDate.getMonth() - 1, 1);
        const lastDayLastMonth = new Date(currentDate.getFullYear(), currentDate.getMonth(), 0);
        typeMatchCriteria = {
            $gte: firstDayLastMonth, // Using Date object directly
            $lte: lastDayLastMonth    // Using Date object directly
        };
    } else if (type === "cy") {
        // Current year
        typeMatchCriteria = {
            $gte: new Date(currentDate.getFullYear(), 0, 1), // Start of the year
            $lte: new Date(currentDate.getFullYear(), 11, 31) // End of the year
        };
    } else if (type !== "all") {
        return res.status(400).json({
            isSuccess: false,
            message: "Invalid type"
        });
    }
    console.log(typeMatchCriteria);
    try{
            const pipeline=[
                {
                  $match:
                    {
                      customer_id:new mongoose.Types.ObjectId(customerId),
                      status: {
                        $in: [
                          "Pickup Completed",
                          "processed",
                          "processing"
                        ]
                      }
                    }
                },
                {
                  $addFields: {
                    formatDate: {
                      $dateFromString: {
                        dateString: {
                          $concat: [
                            {
                              $arrayElemAt: [
                                {
                                  $split: [
                                    "$schedule_date",
                                    "-"
                                  ]
                                },
                                2
                              ]
                            },
                            // Year
                            "-",
                            {
                              $arrayElemAt: [
                                {
                                  $split: [
                                    "$schedule_date",
                                    "-"
                                  ]
                                },
                                1
                              ]
                            },
                            // Month
                            "-",
                            {
                              $arrayElemAt: [
                                {
                                  $split: [
                                    "$schedule_date",
                                    "-"
                                  ]
                                },
                                0
                              ]
                            }
                          ]
                        }
                      }
                    }
                  }
                },
                ...(type !== "all" ? [{
                    $match: {
                        formatDate: typeMatchCriteria
                    }
                }] : []),
                {
                  $unwind:
                    {
                      path: "$item"
                    }
                },
                {
                  $lookup:
                    {
                      from: "items",
                      localField: "item.itemid",
                      foreignField: "_id",
                      as: "itemDetails"
                    }
                },
                {
                  $unwind:
                    {
                      path: "$itemDetails"
                    }
                },
                {
                  $lookup:
                    {
                      from: "itemcategories",
                      localField: "itemDetails.category",
                      foreignField: "_id",
                      as: "categoryDetails"
                    }
                },
                {
                  $unwind: "$categoryDetails"
                },
                {
                  $addFields:
                    {
                      emmisionRate: "$categoryDetails.emission",
                      emmisionValue: {
                        $multiply: [
                          "$categoryDetails.emission",
                          "$item.weight"
                        ]
                      }
                    }
                },
                {
                  $group:
                    {
                      _id: "$_id",
                      emmesionValue: {
                        $sum: "$emmisionValue"
                      },
                      emmesionUnit: {
                        $sum: "$item.weight"
                      },
                      totalPrice: {
                        $first: "$est_price"
                      }
                    }
                },
                {
                  $group:
                    /**
                     * _id: The id of the group.
                     * fieldN: The first field name.
                     */
                    {
                      _id: null,
                      orderCount: {
                        $sum: 1
                      },
                      emmesionValue: {
                        $sum: "$emmesionValue"
                      },
                      emmesionUnit: {
                        $sum: "$emmesionUnit"
                      },
                      totalPrice: {
                        $sum: "$totalPrice"
                      }
                    }
                },
                {
                  $addFields:
                    {
                      treeSaved: {
                        $divide: ["$emmesionValue", 4]
                      },
                      fuelSaved: {
                        $divide: ["$emmesionValue", 2.68]
                      }
                    }
                },
                {
                  $project:
                    {
                      _id: 0
                    }
                }
              ]
              console.log("Pipleine is print", pipeline)
              const result = await Orders.aggregate(pipeline);
              const CoinResult=await coinhists.aggregate(
                [
                    {
                      $match: {
                        customer_id:new mongoose.Types.ObjectId(customerId) ,
                        purpose: "Referal credit from Recycle-Pay."
                      }
                    },
                    {
                      $group:
                        /**
                         * _id: The id of the group.
                         * fieldN: The first field name.
                         */
                        {
                          _id: null,
                          EarnAmount: {
                            $sum: "$earn"
                          }
                        }
                    },
                    {
                      $addFields: {
                        EarnAmount: {
                          $divide: ["$EarnAmount", 10]
                        } // Divide the sum by 10
                      }
                    },
                    {
                      $project:
                        /**
                         * specifications: The fields to
                         *   include or exclude.
                         */
                        {
                          _id: 0,
                          EarnAmount: 1
                        }
                    }
                  ]
              );
              console.log("Result is print", result)
              const initialResponse = {
                orderCount: 0,
                emmesionValue: 0,
                TotalUnit: 0,
                totalPrice: 0,
                treeSaved: 0,
                fuelSaved: 0
            };
            if (result.length > 0) {
                initialResponse.orderCount = result[0].orderCount;
                initialResponse.emmesionValue = parseFloat(result[0].emmesionValue).toFixed(1);
                initialResponse.TotalUnit = parseFloat(result[0].emmesionUnit).toFixed(1);
                if( CoinResult.length>0){
                    initialResponse.totalPrice = parseFloat(CoinResult[0].EarnAmount+result[0].totalPrice).toFixed(1);
                }
                else{
                    initialResponse.totalPrice = parseFloat(result[0].totalPrice).toFixed(1);
                }
                
                initialResponse.treeSaved = parseFloat(result[0].treeSaved).toFixed(0);
                initialResponse.fuelSaved = parseFloat(result[0].fuelSaved).toFixed(0);
            }
            res.json({
                isSuccess: true,
                data: initialResponse
            });
    }
    catch(error){
        console.log(error)
        res.status(500).json({
            isSuccess: false,
            message: error
        })
    }
}