import mongoose from "mongoose";
import Itemcategorys from "../model/itemcategory.model.js";
import Items from "../model/items.model.js";
import Location from "../model/location.model.js";

import multer from 'multer';
const ObjectId = mongoose.Types.ObjectId;

const Storage = multer.diskStorage({
    destination: 'uploads',
    filename: (req, res, cb) => {
        cb(null, Date.now() + "_" + (res.originalname).trim());
    }
});
const upload = multer({
    storage: Storage,
}).single('image');


// Add Items Category
export const addItems = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
            return res.send({ "uploaded": false });
        }

        const { item_name } = req.body;
        // Check if an item with the same name already exists
        const existingItem = await Itemcategorys.findOne({ item_name: { $regex: new RegExp(`^${item_name}$`, 'i') } });

        if (existingItem) {
            return res.send({ "uploaded": false, "message": "Item with the same name already exists." });
        }

        const newItems = new Itemcategorys({
            item_name: req.body.item_name,
            // price: req.body.price,
            // dealer_price: req.body.dealer_price,
            // unit: req.body.unit,
            image: {
                contentType: 'image',
                data: req.file.filename
            },
            imageURL: 'uploads/' + req.file.filename,
            description: req.body.description,
            itemCatLang: JSON.parse(req.body.itemCatLang),
            //here we will add the emission rate of the product
            emission:req.body.emission
            

        });
               //console.log("The emission rate is------------",emission);
        try {
            await newItems.save();
            return res.send({
                isSuccess: true,
                message: "Create Item catagory successfully.",
            });
        } catch (error) {
            console.error(error);
            return res.send({
                isSuccess: false,
                message: "Something went wrong.",
            });
        }
    });
};



// Update Items Category
export const updateItems = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
            return res.send({
                error: true,
                message: "File upload failed"
            });
        }

        console.log(req.body);
        const updateData = {
            $set: { "item_name": req.body.item_name, "description": req.body.description,"emission":req.body.emission, "itemCatLang": JSON.parse(req.body.itemCatLang) },
        };

        if (req.file) {
            updateData.$set.image = {
                contentType: 'image',
                data: req.file.filename
            };
            updateData.$set.imageURL = 'uploads/' + req.file.filename;
        }
        
        try {
            const updateItems = await Itemcategorys.findOneAndUpdate({ _id: new ObjectId(req.params.id) }, updateData);

            if (updateItems) {
                console.log(updateItems)
                res.status(200).json({ message: "Update Item catagory successfully." });
            } else {
                res.status(200).json({ message: "Something went wrong." });
            }
        } catch (error) {
            console.error(error);
            return res.send({
                error: true,
                message: "Internal server error"
            });
        }

    });
};


// Get individual Item Category
export const getItem = async (req, res) => {
    await Itemcategorys.findOne({ _id: req.params.id }).then((x) => {
        console.log(x);
        if (x) {
            return res.send({
                isSuccess: true,
                message: "Get individual item.",
                item: x
            });
        } else {
            return res.send({
                isSuccess: false,
                message: "Something went wrong.",
            });
        }
    })
}


// Get Item-category with related items from item table
export const getItemCatwithItem = async (req, res) => {
    const pipeline = [
        {
            $lookup: {
                from: "itemcategories",
                localField: "category",
                foreignField: "_id",
                as: "item"
            }
        },
        {
            $unwind: "$item"
        },
        {
            $addFields: {
                image: {
                    $cond: {
                        if: { $isArray: "$image" },
                        then: "$image",
                        else: [
                            {
                                contentType: "image",
                                data: "$image.data"
                            }
                        ]
                    }
                },
                imageURL: {
                    $cond: {
                        if: { $isArray: "$imageURL" },
                        then: "$imageURL",
                        else: ["$imageURL"]
                    }
                }
            }
        },
        {
            $group: {
                _id: "$item._id",
                category: { $first: "$item.item_name" },
                imageurl: { $first: "$item.imageURL" },
                image: { $first: "$item.image" },
                description: { $first: "$item.description" },
                items: { $push: "$$ROOT" }
            }
        },
        {
            $sort: {
                category: 1 // 1 for ascending order, -1 for descending order
            }
        },
        {
            $project: {
                _id: 0,
                category: 1,
                imageurl: 1,
                image: 1,
                description: 1,
                items: 1
            }
        },
    ];

    try {
        const items = await Items.aggregate(pipeline);
        res.send({
            isSuccess: true,
            msg: "Filter Future items.",
            item: items
        });
    } catch (error) {
        console.error(error);
        res.send({
            isSuccess: false,
            msg: "Something went wrong.",
        });
    }
};


// Get All Category
export const getItemCat = async (req, res) => {
    await Itemcategorys.find({}).then((y) => {
        if (y) {
            return res.send({
                isSuccess: true,
                message: "Get All item Categories.",
                item: y
            });
        } else {
            return res.send({
                isSuccess: false,
                message: "Something went wrong.",
                item: []
            });
        }
    })
}


// Get All Category By PIN
export const getItemCatByPIN = async (req, res) => {
    const location = await Location.findOne({ PIN: req.params.PIN, isActive: true, isDelete:false });
    console.log(location);
    const cityid = new ObjectId(location.cityid);
    try {
        // Find item categories for the specified cityid
        const items = await Items.find({ cityid: cityid, isActive: true });
        console.log(cityid);
        console.log(items);

        // Extract unique categories from the items
        const categoryIds = [...new Set(items.map(item => item.category.toString()))];

        // Fetch category details from the itemcategory table using the unique category IDs
        const categories = await Itemcategorys.find({ _id: { $in: categoryIds } });

        if (categories.length > 0) {
            return res.send({
                isSuccess: true,
                msg: "Get All item Categories.",
                item: categories
            });
        } else {
            return res.send({
                isSuccess: true,
                msg: "Something went wrong.",
                item: []
            });
        }
    } catch (error) {
        console.error(error);
        return res.send({
            isSuccess: false,
            msg: "Something went wrong.",
            item: []
        });
    }
}



export const getItemCatByPINLang = async (req, res) => {
    const location = await Location.findOne({ PIN: req.params.PIN, isActive: true, isDelete:false });
    console.log(location);
    const cityid = new ObjectId(location.cityid);
    try {
        // Find item categories for the specified cityid
        const items = await Items.find({ cityid: cityid, isActive: true });
        console.log(cityid);
        console.log(items);

        // Extract unique categories from the items
        const categoryIds = [...new Set(items.map(item => new ObjectId(item.category)))];

        // console.log('categoryIds', categoryIds);

        const categories = await Itemcategorys.aggregate([
            {
              $match: {
                _id: { $in: categoryIds }
              }
            },
            {
              $addFields: {
                categoryLang: {
                  $filter: {
                    input: "$itemCatLang",
                    as: "lang",
                    cond: { $eq: ["$$lang.language", req.params.language] }
                  }
                }
              }
            },
            {
              $project: {
                _id: 1,
                item_name: 1,
                imageURL: 1,
                description: 1,
                isActive: 1,
                isDelete: 1,
                createdAt: 1,
                updatedAt: 1,
                __v: 1,
                categoryLang: 1
              }
            }
          ]);
          

        if (categories.length > 0) {
            return res.send({
                isSuccess: true,
                msg: "Get All item Categories.",
                item: categories
            });
        } else {
            return res.send({
                isSuccess: true,
                msg: "Something went wrong.",
                item: []
            });
        }
    } catch (error) {
        console.error(error);
        return res.send({
            isSuccess: false,
            msg: "Something went wrong.",
            item: []
        });
    }
}



// Get All Items by category and PIN
export const getItemsByCatAndCityid = async (req, res) => {
    try {
        const category_id = req.params.category;
        const addressPIN = req.params.PIN;

        const locationExists = await Location.findOne({ PIN: addressPIN });

        if (!locationExists) {
            return res.send({
                isSuccess: false,
                message: "City not exists"
            });
        }

        const cityid = locationExists.cityid;

        const items = await Items.find({ category: new ObjectId(category_id), cityid: cityid, isActive: true });

        if (!items || items.length === 0) {
            return res.send({
                isSuccess: false,
                message: "No items found for the provided category and city.",
                result: []
            });
        }

        const formattedItems = items.map(item => {
            let itemimageURL = '';
            let itemimages = [];
        
            if (Array.isArray(item.imageURL) && item.imageURL.length > 0) {
                itemimageURL = item.imageURL[0];
                itemimages = item.imageURL;
            } else if (typeof item.imageURL === 'string') {
                itemimageURL = item.imageURL;
                itemimages = [item.imageURL];
            }
        
            // Destructure and skip unwanted fields
            const { imageURL, image, ...rest } = item._doc;

            return {
                _id: { item_name: item.item_name, location: item.cityid },
                totalAmount: 0,
                totalWeight: 0,
                orderedItems: {
                    imageURL: itemimageURL,
                    itemimages: itemimages,
                    ...rest
                },
                location: item.cityid
            };
        });

        return res.send({
            isSuccess: true,
            message: "All items of the provided category and city.",
            result: formattedItems
        });

    } catch (error) {
        console.error("Error: ", error);
        return res.status(500).send({
            isSuccess: false,
            message: "Something went wrong.",
            error: error.message
        });
    }
};





export const getItemsByCatAndCityidLang = async (req, res) => {
    try {
        const category_id = req.params.category;
        const addressPIN = req.params.PIN;

        const locationExists = await Location.findOne({ PIN: addressPIN });

        if (!locationExists) {
            return res.send({
                isSuccess: false,
                message: "City not exists"
            });
        }

        const cityid = locationExists.cityid;

        // const items = await Items.find({ category: new ObjectId(category_id), cityid: cityid, isActive: true });
        const items = await Items.aggregate([
            {
              $match: {
                category: new ObjectId(category_id),
                cityid: cityid,
                isActive: true
              }
            },
            {
                $addFields: {
                    orderedItemsitemLang: [{
                        $arrayElemAt: [
                            {
                                $filter: {
                                    input: "$itemLang",
                                    cond: { $eq: ["$$this.language", req.params.language] }
                                }
                            },
                            0 
                        ]
                    }]
                }
            },
              {
                $match: {
                  "orderedItemsLang": { $ne: req.params.language } // Exclude documents where itemLang is empty
                }
              },
              {
                $addFields: {
                  customSort: {
                    $cond: [
                      {
                        $in: [
                            { $arrayElemAt: ["$orderedItemsitemLang.item_name", 0] }, // Access the first item_name in the array
                            ["କାର୍ଡବୋର୍ଡ", "Cardboard", "गत्ता"] // Languages you want to prioritize
                        ]
                      },
                      0, // These items should come first
                      1  // All other items should follow
                    ]
                  }
                }
              },
              
            {
                $sort: {
                    customSort: 1,  // Sort by customSort first
                    "_id.item_name": 1  // Then sort alphabetically by item name
                  }
            }
          ]);
          
          

        if (!items || items.length === 0) {
            return res.send({
                isSuccess: false,
                message: "No items found for the provided category and city.",
                result: []
            });
        }

        const formattedItems = items.map(item => {
            let itemimageURL = '';
            let itemimages = [];
        
            if (Array.isArray(item.imageURL) && item.imageURL.length > 0) {
                itemimageURL = item.imageURL[0];
                itemimages = item.imageURL;
            } else if (typeof item.imageURL === 'string') {
                itemimageURL = item.imageURL;
                itemimages = [item.imageURL];
            }
        
            const { imageURL: tempImageURL, image: tempImage, ...rest } = item;
        
            return {
                _id: { item_name: rest.item_name, location: item.cityid },
                totalAmount: 0, // Set to 0 since this API doesn't calculate total amount
                totalWeight: 0, // Set to 0 since this API doesn't calculate total weight
                orderedItems: {
                    imageURL: itemimageURL,
                    itemimages: itemimages,
                    ...rest
                },
                location: item.cityid
            };
        });        

        return res.send({
            isSuccess: true,
            message: "All items of the provided category and city.",
            result: formattedItems
        });

    } catch (error) {
        console.error("Error: ", error);
        return res.status(500).send({
            isSuccess: false,
            message: "Something went wrong.",
            error: error.message
        });
    }
};


