import mongoose from "mongoose";
import Items from "../model/items.model.js";
import Orders from "../model/order.model.js";
import Itemcategorys from "../model/itemcategory.model.js";
import CityPercent from "../model/city_percent.model.js";
import Location from "../model/location.model.js";
import SibApiV3Sdk from 'sib-api-v3-sdk';
import dotenv from 'dotenv';
dotenv.config();

import multer from 'multer';
import Address from "../model/address.model.js";
import bodyParser from "body-parser";

const { json } = bodyParser;
const ObjectId = mongoose.Types.ObjectId;

const Storage = multer.diskStorage({
    destination: 'uploads',
    filename: (req, res, cb) => {
        cb(null, Date.now() + "_" + (res.originalname).trim());
    }
});
const upload = multer({
    storage: Storage,
}).array('image');

// Add Items
export const addItems = async (req, res) => {   
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
            return res.send({ "uploaded": false });
        }

        const { item_name, price, dealer_price, unit, city, cityid, category, description } = req.body;
        console.log("the selected city is", cityid);
        const parsedCityid = JSON.parse(`[${cityid}]`);
        
        console.log("the array is",parsedCityid);
        
         for ( let i=0;i<parsedCityid.length;i++){
            const cityid=parsedCityid[i].cityId;
            const cityName=parsedCityid[i].cityName;
            
        // // Check if an item with the same name & city already exists
        const existingItem = await Items.findOne({ item_name: { $regex: new RegExp(`^${item_name}$`, 'i') }, cityid: cityid });
        
        console.log("the existing item is",existingItem);
    

        if (existingItem) {
            return res.send({ "uploaded": false, "message": "Item with the same name & cityid already exists." });
        }
    }
    
        const newImages = [];
        const newImageURLs = [];

        req.files.forEach((file) => {
            newImages.push({
                contentType: 'image',
                data: file.filename
            });
            newImageURLs.push('uploads/' + file.filename);
        });
        const newItems=[];
        for(let i=0;i<parsedCityid.length;i++){
            const cityid=parsedCityid[i].cityId;
            const cityName=parsedCityid[i].cityName;

         newItems.push({
            item_name: item_name,
            price: price,
            dealer_price: dealer_price,
            unit: unit,
            city: cityName,
            cityid: cityid,
            category: category,
            image: newImages,
            imageURL: newImageURLs,
            description: description,
            itemLang: JSON.parse(req.body.itemLang)
        });
    }

        try {
            await Items.insertMany(newItems);
            res.send({ "uploaded": true, data: newItems });
        } catch (error) {
          console.error("Error sending notifications:", error);
        }
    });
};


// Get all Items for Admin View
export const getAllItems = async (req, res) => {
    const items = await Items.find({ isDelete: false })
        .populate('category', 'item_name') // Populate the category details
        .sort({ createdAt: -1 }).exec();
    res.send(items)
}


// Get all item according to purchase
// export const getAllItemsPur = async (req, res) => {
//     // const pipeline = [
//     //     {
//     //       $match: {
//     //         $or: [
//     //           { status: "processed" },
//     //           { status: "cash" }
//     //         ]
//     //       }
//     //     },
//     //     {
//     //       $unwind: "$item"
//     //     },
//     //     {
//     //       $lookup: {
//     //         from: "items",
//     //         localField: "item.itemid",
//     //         foreignField: "_id",
//     //         as: "orderedItems"
//     //       }
//     //     },
//     //     {
//     //       $unwind: "$orderedItems"
//     //     },
//     //     {
//     //       $group: {
//     //         _id: "$orderedItems.item_name",
//     //         totalAmount: {
//     //           $sum: {
//     //             $multiply: [
//     //               { $toDouble: "$item.price" },
//     //               { $toDouble: "$item.weight" }
//     //             ]
//     //           }
//     //         },
//     //         totalWeight: { $sum: { $toDouble: "$item.weight" } },
//     //         orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //       }
//     //     },
//     //     {
//     //       $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     //   ];

//     // const pipeline = [
//     //     {
//     //         $unwind: "$items"
//     //     },
//     //     {
//     //         $lookup: {
//     //             from: "orders",
//     //             let: { orderId: "$_id", itemId: "$items.itemid" },
//     //             pipeline: [
//     //                 {
//     //                     $match: {
//     //                         $expr: {
//     //                             $and: [
//     //                                 { $in: ["$$itemId", "$items.itemid"] },
//     //                                 { $in: ["$$orderId", "$_id"] },
//     //                                 { $in: ["processed", "$status", "$cash"] }
//     //                             ]
//     //                         }
//     //                     }
//     //                 },
//     //                 {
//     //                     $unwind: "$items"
//     //                 },
//     //                 {
//     //                     $lookup: {
//     //                         from: "items",
//     //                         localField: "items.itemid",
//     //                         foreignField: "_id",
//     //                         as: "orderedItems"
//     //                     }
//     //                 },
//     //                 {
//     //                     $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //                 },
//     //                 {
//     //                     $group: {
//     //                         _id: "$orderedItems.item_name",
//     //                         totalAmount: {
//     //                             $sum: {
//     //                                 $multiply: [
//     //                                     { $toDouble: "$items.price" },
//     //                                     { $toDouble: "$items.weight" }
//     //                                 ]
//     //                             }
//     //                         },
//     //                         totalWeight: { $sum: { $toDouble: "$items.weight" } },
//     //                         orderedItems: { $first: "$orderedItems" }
//     //                     }
//     //                 }
//     //             ],
//     //             as: "joinedOrders"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$joinedOrders", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: "$joinedOrders._id",
//     //             totalAmount: { $sum: "$joinedOrders.totalAmount" },
//     //             totalWeight: { $sum: "$joinedOrders.totalWeight" },
//     //             orderedItems: { $first: "$joinedOrders.orderedItems" }
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 }
//     //     }
//     // ];

//     // const pipeline = [
//     //     {
//     //         $lookup: {
//     //             from: "items",
//     //             let: { itemId: "$_id" },
//     //             pipeline: [
//     //                 {
//     //                     $match: {
//     //                         $expr: {
//     //                             $eq: ["$_id", "$$itemId"]
//     //                         }
//     //                     }
//     //                 },
//     //                 // Add a lookup stage for the "Orders" collection
//     //                 {
//     //                     $lookup: {
//     //                         from: "orders",
//     //                         localField: "itemid",  // Assuming the field in "items" that corresponds to "item.itemid" in "Orders"
//     //                         foreignField: "item.itemid",
//     //                         as: "orders"
//     //                     }
//     //                 }
//     //             ],
//     //             as: "orderedItems"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: "$orderedItems.item_name",
//     //             totalAmount: {
//     //                 $sum: {
//     //                     $multiply: [
//     //                         { $toDouble: "$item.price" },
//     //                         { $toDouble: "$item.weight" }
//     //                     ]
//     //                 }
//     //             },
//     //             totalWeight: {
//     //                 $sum: {
//     //                     $cond: [
//     //                         { $eq: ["$orderedItems", []] },  // Check if the "orderedItems" array is empty (no match)
//     //                         0,  // If no match, set totalWeight to 0
//     //                         { $toDouble: "$item.weight" }  // Otherwise, use the weight value
//     //                     ]
//     //                 }
//     //             },
//     //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     // ];


//     const pipeline = [
//         {
//             $match: {
//                 $or: [
//                     { status: "processed" },
//                     { status: "cash" }
//                 ]
//             }
//         },
//         {
//             $unwind: "$item"
//         },
//         {
//             $lookup: {
//                 from: "items",
//                 localField: "item.itemid",
//                 foreignField: "_id",
//                 as: "orderedItems"
//             }
//         },
//         {
//             $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//         },
//         {
//             $group: {
//                 _id: "$orderedItems.item_name",
//                 totalAmount: {
//                     $sum: {
//                         $cond: [
//                             { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
//                             0, // If missing, set totalAmount to 0
//                             {
//                                 $multiply: [
//                                     { $toDouble: "$item.price" },
//                                     { $toDouble: "$item.weight" }
//                                 ]
//                             }
//                         ]
//                     }
//                 },
//                 totalWeight: {
//                     $sum: {
//                         $cond: [
//                             { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
//                             0, // If missing, set totalWeight to 0
//                             { $toDouble: "$item.weight" } // Otherwise, use the weight value
//                         ]
//                     }
//                 },
//                 orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//             }
//         },
//         {
//             $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//         }
//     ];

//     const result = await Orders.aggregate(pipeline);


//     // const pipeline = [
//     //     {
//     //         $match: {
//     //             $or: [
//     //                 { status: "processed" },
//     //                 { status: "cash" }
//     //             ]
//     //         }
//     //     },
//     //     {
//     //         $unwind: "$item"
//     //     },
//     //     {
//     //         $lookup: {
//     //             from: "items",
//     //             localField: "item.itemid",
//     //             foreignField: "_id",
//     //             as: "orderedItems"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: {
//     //                 $ifNull: [
//     //                     "$orderedItems.item_name",
//     //                     "$item.item_name"
//     //                 ]
//     //             },
//     //             totalAmount: {
//     //                 $sum: {
//     //                     $multiply: [
//     //                         { $toDouble: "$item.price" },
//     //                         { $toDouble: "$item.weight" }
//     //                     ]
//     //                 }
//     //             },
//     //             totalWeight: {
//     //                 $sum: {
//     //                     $cond: [
//     //                         { $eq: ["$orderedItems", []] }, // Check if "orderedItems" array is empty (no match)
//     //                         { $toDouble: "$item.weight" }, // If no match, use the weight value
//     //                         0 // Otherwise, set totalWeight to 0
//     //                     ]
//     //                 }
//     //             },
//     //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     // ];


//     const existingItemNames = result.map((item) => item._id);

//     const missingItemsPipeline = [
//         {
//             $match: {
//                 item_name: { $nin: existingItemNames }
//             }
//         },
//         {
//             $group: {
//                 _id: "$item_name",
//                 totalAmount: { $sum: 0 }, 
//                 totalWeight: { $sum: 0 }, 
//                 orderedItems: { $first: "$$ROOT" } 
//             }
//         },
//         {
//             $sort: {
//                 _id: 1 
//             }
//         }
//     ];

//     const missingItems = await Items.aggregate(missingItemsPipeline);

//     // Concatenate the results
//     const finalResult = result.concat(missingItems);

//     // Sort the final result by totalAmount in descending order
//     const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);

//     // Use sortedResult for further processing or display
//     console.log(sortedResult);




//     if (result) {
//         res.json({
//             isSuccess: true,
//             message: "Items sort by estimate purchase.",
//             result: sortedResult,
//         });
//     } else {
//         res.json({
//             isSuccess: false,
//             message: "Something went wrong.",
//         });
//     }
// }


export const getAllItemsPur = async (req, res) => {
    // const pipeline = [
    //     {
    //       $match: {
    //         $or: [
    //           { status: "processed" },
    //           { status: "cash" }
    //         ]
    //       }
    //     },
    //     {
    //       $unwind: "$item"
    //     },
    //     {
    //       $lookup: {
    //         from: "items",
    //         localField: "item.itemid",
    //         foreignField: "_id",
    //         as: "orderedItems"
    //       }
    //     },
    //     {
    //       $unwind: "$orderedItems"
    //     },
    //     {
    //       $group: {
    //         _id: "$orderedItems.item_name",
    //         totalAmount: {
    //           $sum: {
    //             $multiply: [
    //               { $toDouble: "$item.price" },
    //               { $toDouble: "$item.weight" }
    //             ]
    //           }
    //         },
    //         totalWeight: { $sum: { $toDouble: "$item.weight" } },
    //         orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //       }
    //     },
    //     {
    //       $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    //   ];

    // const pipeline = [
    //     {
    //         $unwind: "$items"
    //     },
    //     {
    //         $lookup: {
    //             from: "orders",
    //             let: { orderId: "$_id", itemId: "$items.itemid" },
    //             pipeline: [
    //                 {
    //                     $match: {
    //                         $expr: {
    //                             $and: [
    //                                 { $in: ["$$itemId", "$items.itemid"] },
    //                                 { $in: ["$$orderId", "$_id"] },
    //                                 { $in: ["processed", "$status", "$cash"] }
    //                             ]
    //                         }
    //                     }
    //                 },
    //                 {
    //                     $unwind: "$items"
    //                 },
    //                 {
    //                     $lookup: {
    //                         from: "items",
    //                         localField: "items.itemid",
    //                         foreignField: "_id",
    //                         as: "orderedItems"
    //                     }
    //                 },
    //                 {
    //                     $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //                 },
    //                 {
    //                     $group: {
    //                         _id: "$orderedItems.item_name",
    //                         totalAmount: {
    //                             $sum: {
    //                                 $multiply: [
    //                                     { $toDouble: "$items.price" },
    //                                     { $toDouble: "$items.weight" }
    //                                 ]
    //                             }
    //                         },
    //                         totalWeight: { $sum: { $toDouble: "$items.weight" } },
    //                         orderedItems: { $first: "$orderedItems" }
    //                     }
    //                 }
    //             ],
    //             as: "joinedOrders"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$joinedOrders", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: "$joinedOrders._id",
    //             totalAmount: { $sum: "$joinedOrders.totalAmount" },
    //             totalWeight: { $sum: "$joinedOrders.totalWeight" },
    //             orderedItems: { $first: "$joinedOrders.orderedItems" }
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 }
    //     }
    // ];

    // const pipeline = [
    //     {
    //         $lookup: {
    //             from: "items",
    //             let: { itemId: "$_id" },
    //             pipeline: [
    //                 {
    //                     $match: {
    //                         $expr: {
    //                             $eq: ["$_id", "$$itemId"]
    //                         }
    //                     }
    //                 },
    //                 // Add a lookup stage for the "Orders" collection
    //                 {
    //                     $lookup: {
    //                         from: "orders",
    //                         localField: "itemid",  // Assuming the field in "items" that corresponds to "item.itemid" in "Orders"
    //                         foreignField: "item.itemid",
    //                         as: "orders"
    //                     }
    //                 }
    //             ],
    //             as: "orderedItems"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: "$orderedItems.item_name",
    //             totalAmount: {
    //                 $sum: {
    //                     $multiply: [
    //                         { $toDouble: "$item.price" },
    //                         { $toDouble: "$item.weight" }
    //                     ]
    //                 }
    //             },
    //             totalWeight: {
    //                 $sum: {
    //                     $cond: [
    //                         { $eq: ["$orderedItems", []] },  // Check if the "orderedItems" array is empty (no match)
    //                         0,  // If no match, set totalWeight to 0
    //                         { $toDouble: "$item.weight" }  // Otherwise, use the weight value
    //                     ]
    //                 }
    //             },
    //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    // ];


    const pipeline = [
        {
            $match: {
                $and: [
                    // { cityid: new ObjectId(req.params.cityid) },
                    {$or: [
                        { status: "processed" },
                        { status: "cash" }
                    ]}
                ]
            }
        },
        {
            $unwind: "$item"
        },
        {
            $lookup: {
                from: "items",
                localField: "item.itemid",
                foreignField: "_id",
                as: "orderedItems"
            }
        },
        {
            $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
        },
        {
            $match: {
                "orderedItems.isActive": true
            }
        },
        {
            $lookup: {
                from: "locations",
                localField: "orderedItems.cityid",
                foreignField: "cityid",
                as: "locationItems"
            }
        },
        {
            $unwind: { path: "$locationItems", preserveNullAndEmptyArrays: true }
        },
        {
            $match: {
                "locationItems.PIN": '751013'
            }
        },
        {
            $group: {
                _id: {
                    item_name: "$orderedItems.item_name",
                    // location: "$locationItems.PIN"
                    location: "$locationItems.PIN",
                    cityid: "$orderedItems.cityid" 
                },
                totalAmount: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalAmount to 0
                            {
                                $multiply: [
                                    { $toDouble: "$item.price" },
                                    { $toDouble: "$item.weight" }
                                ]
                            }
                        ]
                    }
                },
                totalWeight: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalWeight to 0
                            { $toDouble: "$item.weight" } // Otherwise, use the weight value
                        ]
                    }
                },
                orderedItems: { $first: "$orderedItems" }, // Select the first orderedItems document
                // item_name: "$orderedItems.item_name",
                cityid: { $first: "$orderedItems.cityid" }
            }
        },
        {
            $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
        }
    ];

    const result = await Orders.aggregate(pipeline);


    // const pipeline = [
    //     {
    //         $match: {
    //             $or: [
    //                 { status: "processed" },
    //                 { status: "cash" }
    //             ]
    //         }
    //     },
    //     {
    //         $unwind: "$item"
    //     },
    //     {
    //         $lookup: {
    //             from: "items",
    //             localField: "item.itemid",
    //             foreignField: "_id",
    //             as: "orderedItems"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: {
    //                 $ifNull: [
    //                     "$orderedItems.item_name",
    //                     "$item.item_name"
    //                 ]
    //             },
    //             totalAmount: {
    //                 $sum: {
    //                     $multiply: [
    //                         { $toDouble: "$item.price" },
    //                         { $toDouble: "$item.weight" }
    //                     ]
    //                 }
    //             },
    //             totalWeight: {
    //                 $sum: {
    //                     $cond: [
    //                         { $eq: ["$orderedItems", []] }, // Check if "orderedItems" array is empty (no match)
    //                         { $toDouble: "$item.weight" }, // If no match, use the weight value
    //                         0 // Otherwise, set totalWeight to 0
    //                     ]
    //                 }
    //             },
    //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    // ];

    // const existingItemNames = result.map((item) => item._id);

    var x = []
    // const existingItemNames = result.map((item) => {
    //     console.log(x.push(item.cityid));
    // });

    // console.log(existingItemNames);

    const existingItemNames = result.map((item) => item._id);
    console.log(existingItemNames.length == 0);

    // console.log(existingItemNames);

    if (existingItemNames.length != 0) {
        const missingItemsPipeline = [
            {
                $match: {
                    $and: [
                        { item_name: { $nin: existingItemNames.map(item => item.item_name) } },
                        { cityid: result[0].cityid },
                        {
                            isActive: true
                        }
                    ]
                }
            },
            {
                $group: {
                    _id: {
                        item_name: "$item_name",
                        cityid: result[0].location // Use result[0].location directly, assuming it's a valid cityid
                    },
                    totalAmount: { $sum: 0 },
                    totalWeight: { $sum: 0 },
                    orderedItems: { $first: "$$ROOT" },
                    city: { $first: "$$ROOT.cityid" }
                }
            },
            {
                $sort: {
                    "_id.item_name": 1
                }
            }
        ];
        
    
        const missingItems = await Items.aggregate(missingItemsPipeline);
    
        // Concatenate the results
        const finalResult = result.concat(missingItems);
    
        // Sort the final result by totalAmount in descending order
        const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);
    
        // Use sortedResult for further processing or display
        console.log('finalResult', finalResult);
        console.log('missingItems', missingItems);
    
    
    
    
        if (result) {
            res.json({
                isSuccess: true,
                message: "Items sort by estimate purchase.",
                result: sortedResult,
            });
        } else {
            res.json({
                isSuccess: false,
                message: "Something went wrong.",
            });
        }
    } else {

        await Location.findOne({ PIN: req.params.cityid }).then(async (x) => {
            console.log("1234567890x", x);
            const missingItemsPipeline = [
                {
                    $match: {
                        $and: [
                            { item_name: { $nin: existingItemNames.map(item => item.item_name) } },
                            { cityid: new ObjectId(x.cityid) }
                        ]
                    }
                },
                {
                    $group: {
                        _id: {
                            item_name: "$item_name",
                            cityid: x.cityid // Use result[0].location directly, assuming it's a valid cityid
                        },
                        totalAmount: { $sum: 0 },
                        totalWeight: { $sum: 0 },
                        orderedItems: { $first: "$$ROOT" },
                        city: { $first: "$$ROOT.cityid" }
                    }
                },
                {
                    $sort: {
                        "_id.item_name": 1
                    }
                }
            ];
            
        
            const missingItems = await Items.aggregate(missingItemsPipeline);
        
            // Concatenate the results
            const finalResult = result.concat(missingItems);
        
            // Sort the final result by totalAmount in descending order
            const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);   
        
        
        
            if (result) {
                res.json({
                    isSuccess: true,
                    message: "Items sort by estimate purchase.",
                    result: sortedResult,
                });
            } else {
                res.json({
                    isSuccess: false,
                    message: "Something went wrong.",
                });
            }
        } )

        
    }
    // const missingItemsPipeline = [
    //     {
    //         $match: {
    //             // item_name: existingItemNames[0].item_name,
    //             // cityid: x[0]
    //             // cityid: req.params.cityid,
    //             // cityid: existingItemNames[0].cityid,
    //             item_name: { $nin: existingItemNames }
    //         }
    //     },
    //     {
    //         $group: {
    //             _id: {
    //                 // item_name: "$item_name",
    //                 cityid: "$cityid"
    //             },
    //             totalAmount: { $sum: 0 }, 
    //             totalWeight: { $sum: 0 }, 
    //             orderedItems: { $first: "$$ROOT" },
    //             city: { $first: "$$ROOT.cityid" }
    //         }
    //     },
    //     {
    //         $sort: {
    //             _id: 1 
    //         }
    //     }
    // ];

    // const missingItems = await Items.aggregate(missingItemsPipeline);

    // // Concatenate the results
    // const finalResult = result.concat(missingItems);

    // // Sort the final result by totalAmount in descending order
    // const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);

    // // Use sortedResult for further processing or display
    // console.log(finalResult);




    // if (result) {
    //     res.json({
    //         isSuccess: true,
    //         message: "Items sort by estimate purchase.",
    //         result: sortedResult,
    //     });
    // } else {
    //     res.json({
    //         isSuccess: false,
    //         message: "Something went wrong.",
    //     });
    // }
}

// export const getAllItemsPurbyCityid = async (req, res) => {
//     // const pipeline = [
//     //     {
//     //       $match: {
//     //         $or: [
//     //           { status: "processed" },
//     //           { status: "cash" }
//     //         ]
//     //       }
//     //     },
//     //     {
//     //       $unwind: "$item"
//     //     },
//     //     {
//     //       $lookup: {
//     //         from: "items",
//     //         localField: "item.itemid",
//     //         foreignField: "_id",
//     //         as: "orderedItems"
//     //       }
//     //     },
//     //     {
//     //       $unwind: "$orderedItems"
//     //     },
//     //     {
//     //       $group: {
//     //         _id: "$orderedItems.item_name",
//     //         totalAmount: {
//     //           $sum: {
//     //             $multiply: [
//     //               { $toDouble: "$item.price" },
//     //               { $toDouble: "$item.weight" }
//     //             ]
//     //           }
//     //         },
//     //         totalWeight: { $sum: { $toDouble: "$item.weight" } },
//     //         orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //       }
//     //     },
//     //     {
//     //       $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     //   ];

//     // const pipeline = [
//     //     {
//     //         $unwind: "$items"
//     //     },
//     //     {
//     //         $lookup: {
//     //             from: "orders",
//     //             let: { orderId: "$_id", itemId: "$items.itemid" },
//     //             pipeline: [
//     //                 {
//     //                     $match: {
//     //                         $expr: {
//     //                             $and: [
//     //                                 { $in: ["$$itemId", "$items.itemid"] },
//     //                                 { $in: ["$$orderId", "$_id"] },
//     //                                 { $in: ["processed", "$status", "$cash"] }
//     //                             ]
//     //                         }
//     //                     }
//     //                 },
//     //                 {
//     //                     $unwind: "$items"
//     //                 },
//     //                 {
//     //                     $lookup: {
//     //                         from: "items",
//     //                         localField: "items.itemid",
//     //                         foreignField: "_id",
//     //                         as: "orderedItems"
//     //                     }
//     //                 },
//     //                 {
//     //                     $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //                 },
//     //                 {
//     //                     $group: {
//     //                         _id: "$orderedItems.item_name",
//     //                         totalAmount: {
//     //                             $sum: {
//     //                                 $multiply: [
//     //                                     { $toDouble: "$items.price" },
//     //                                     { $toDouble: "$items.weight" }
//     //                                 ]
//     //                             }
//     //                         },
//     //                         totalWeight: { $sum: { $toDouble: "$items.weight" } },
//     //                         orderedItems: { $first: "$orderedItems" }
//     //                     }
//     //                 }
//     //             ],
//     //             as: "joinedOrders"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$joinedOrders", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: "$joinedOrders._id",
//     //             totalAmount: { $sum: "$joinedOrders.totalAmount" },
//     //             totalWeight: { $sum: "$joinedOrders.totalWeight" },
//     //             orderedItems: { $first: "$joinedOrders.orderedItems" }
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 }
//     //     }
//     // ];

//     // const pipeline = [
//     //     {
//     //         $lookup: {
//     //             from: "items",
//     //             let: { itemId: "$_id" },
//     //             pipeline: [
//     //                 {
//     //                     $match: {
//     //                         $expr: {
//     //                             $eq: ["$_id", "$$itemId"]
//     //                         }
//     //                     }
//     //                 },
//     //                 // Add a lookup stage for the "Orders" collection
//     //                 {
//     //                     $lookup: {
//     //                         from: "orders",
//     //                         localField: "itemid",  // Assuming the field in "items" that corresponds to "item.itemid" in "Orders"
//     //                         foreignField: "item.itemid",
//     //                         as: "orders"
//     //                     }
//     //                 }
//     //             ],
//     //             as: "orderedItems"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: "$orderedItems.item_name",
//     //             totalAmount: {
//     //                 $sum: {
//     //                     $multiply: [
//     //                         { $toDouble: "$item.price" },
//     //                         { $toDouble: "$item.weight" }
//     //                     ]
//     //                 }
//     //             },
//     //             totalWeight: {
//     //                 $sum: {
//     //                     $cond: [
//     //                         { $eq: ["$orderedItems", []] },  // Check if the "orderedItems" array is empty (no match)
//     //                         0,  // If no match, set totalWeight to 0
//     //                         { $toDouble: "$item.weight" }  // Otherwise, use the weight value
//     //                     ]
//     //                 }
//     //             },
//     //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     // ];

//     // console.log(req.params.cityid);

//     const pipeline = [
//         {
//             $match: {
//                 $and: [
//                     // { cityid: new ObjectId(req.params.cityid) },
//                     {$or: [
//                         { status: "processed" },
//                         { status: "cash" }
//                     ]}
//                 ]
//             }
//         },
//         {
//             $unwind: "$item"
//         },
//         {
//             $lookup: {
//                 from: "items",
//                 localField: "item.itemid",
//                 foreignField: "_id",
//                 as: "orderedItems"
//             }
//         },
//         {
//             $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//         },
//         {
//             $lookup: {
//                 from: "locations",
//                 localField: "orderedItems.cityid",
//                 foreignField: "cityid",
//                 as: "locationItems"
//             }
//         },
//         {
//             $unwind: { path: "$locationItems", preserveNullAndEmptyArrays: true }
//         },
//         {
//             $match: {
//                 "locationItems.PIN": req.params.cityid
//             }
//         },
//         {
//             $group: {
//                 _id: {
//                     item_name: "$orderedItems.item_name",
//                     // location: "$locationItems.PIN"
//                     location: "$locationItems.PIN",
//                 },
//                 totalAmount: {
//                     $sum: {
//                         $cond: [
//                             { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
//                             0, // If missing, set totalAmount to 0
//                             {
//                                 $multiply: [
//                                     { $toDouble: "$item.price" },
//                                     { $toDouble: "$item.weight" }
//                                 ]
//                             }
//                         ]
//                     }
//                 },
//                 totalWeight: {
//                     $sum: {
//                         $cond: [
//                             { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
//                             0, // If missing, set totalWeight to 0
//                             { $toDouble: "$item.weight" } // Otherwise, use the weight value
//                         ]
//                     }
//                 },
//                 orderedItems: { $first: "$orderedItems" }, // Select the first orderedItems document
//                 location: { $first: "$orderedItems.cityid" }
//             }
//         },
//         {
//             $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//         }
//     ];

//     const result = await Orders.aggregate(pipeline);


//     // const pipeline = [
//     //     {
//     //         $match: {
//     //             $or: [
//     //                 { status: "processed" },
//     //                 { status: "cash" }
//     //             ]
//     //         }
//     //     },
//     //     {
//     //         $unwind: "$item"
//     //     },
//     //     {
//     //         $lookup: {
//     //             from: "items",
//     //             localField: "item.itemid",
//     //             foreignField: "_id",
//     //             as: "orderedItems"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: {
//     //                 $ifNull: [
//     //                     "$orderedItems.item_name",
//     //                     "$item.item_name"
//     //                 ]
//     //             },
//     //             totalAmount: {
//     //                 $sum: {
//     //                     $multiply: [
//     //                         { $toDouble: "$item.price" },
//     //                         { $toDouble: "$item.weight" }
//     //                     ]
//     //                 }
//     //             },
//     //             totalWeight: {
//     //                 $sum: {
//     //                     $cond: [
//     //                         { $eq: ["$orderedItems", []] }, // Check if "orderedItems" array is empty (no match)
//     //                         { $toDouble: "$item.weight" }, // If no match, use the weight value
//     //                         0 // Otherwise, set totalWeight to 0
//     //                     ]
//     //                 }
//     //             },
//     //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     // ];


//     const existingItemNames = result.map((item) => item._id);

//     const missingItemsPipeline = [
//         {
//             $match: {
//                 // cityid: new ObjectId(req.params.cityid) ,
//                 item_name: { $nin: existingItemNames },
//                 cityid: new ObjectId(existingItemNames.cityid)
//             }
//         },
//         {
//             $group: {
//                 _id: {
//                     item_name: "$item_name",
//                     cityid: "$cityid"
//                 },
//                 totalAmount: { $sum: 0 }, 
//                 totalWeight: { $sum: 0 }, 
//                 orderedItems: { $first: "$$ROOT" },
//                 city: { $first: "$$ROOT.cityid" }
//             }
//         },
//         {
//             $sort: {
//                 _id: 1 
//             }
//         }
//     ];

//     const missingItems = await Items.aggregate(missingItemsPipeline);

//     // Concatenate the results
//     const finalResult = result.concat(missingItems);

//     // Sort the final result by totalAmount in descending order
//     const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);

//     // Use sortedResult for further processing or display
//     console.log(sortedResult);




//     if (result) {
//         res.json({
//             isSuccess: true,
//             message: "Items sort by estimate purchase.",
//             result: sortedResult,
//         });
//     } else {
//         res.json({
//             isSuccess: false,
//             message: "Something went wrong.",
//         });
//     }
// }


// export const getAllItemsPurbyCityid = async (req, res) => {
//     // const pipeline = [
//     //     {
//     //       $match: {
//     //         $or: [
//     //           { status: "processed" },
//     //           { status: "cash" }
//     //         ]
//     //       }
//     //     },
//     //     {
//     //       $unwind: "$item"
//     //     },
//     //     {
//     //       $lookup: {
//     //         from: "items",
//     //         localField: "item.itemid",
//     //         foreignField: "_id",
//     //         as: "orderedItems"
//     //       }
//     //     },
//     //     {
//     //       $unwind: "$orderedItems"
//     //     },
//     //     {
//     //       $group: {
//     //         _id: "$orderedItems.item_name",
//     //         totalAmount: {
//     //           $sum: {
//     //             $multiply: [
//     //               { $toDouble: "$item.price" },
//     //               { $toDouble: "$item.weight" }
//     //             ]
//     //           }
//     //         },
//     //         totalWeight: { $sum: { $toDouble: "$item.weight" } },
//     //         orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //       }
//     //     },
//     //     {
//     //       $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     //   ];

//     // const pipeline = [
//     //     {
//     //         $unwind: "$items"
//     //     },
//     //     {
//     //         $lookup: {
//     //             from: "orders",
//     //             let: { orderId: "$_id", itemId: "$items.itemid" },
//     //             pipeline: [
//     //                 {
//     //                     $match: {
//     //                         $expr: {
//     //                             $and: [
//     //                                 { $in: ["$$itemId", "$items.itemid"] },
//     //                                 { $in: ["$$orderId", "$_id"] },
//     //                                 { $in: ["processed", "$status", "$cash"] }
//     //                             ]
//     //                         }
//     //                     }
//     //                 },
//     //                 {
//     //                     $unwind: "$items"
//     //                 },
//     //                 {
//     //                     $lookup: {
//     //                         from: "items",
//     //                         localField: "items.itemid",
//     //                         foreignField: "_id",
//     //                         as: "orderedItems"
//     //                     }
//     //                 },
//     //                 {
//     //                     $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //                 },
//     //                 {
//     //                     $group: {
//     //                         _id: "$orderedItems.item_name",
//     //                         totalAmount: {
//     //                             $sum: {
//     //                                 $multiply: [
//     //                                     { $toDouble: "$items.price" },
//     //                                     { $toDouble: "$items.weight" }
//     //                                 ]
//     //                             }
//     //                         },
//     //                         totalWeight: { $sum: { $toDouble: "$items.weight" } },
//     //                         orderedItems: { $first: "$orderedItems" }
//     //                     }
//     //                 }
//     //             ],
//     //             as: "joinedOrders"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$joinedOrders", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: "$joinedOrders._id",
//     //             totalAmount: { $sum: "$joinedOrders.totalAmount" },
//     //             totalWeight: { $sum: "$joinedOrders.totalWeight" },
//     //             orderedItems: { $first: "$joinedOrders.orderedItems" }
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 }
//     //     }
//     // ];

//     // const pipeline = [
//     //     {
//     //         $lookup: {
//     //             from: "items",
//     //             let: { itemId: "$_id" },
//     //             pipeline: [
//     //                 {
//     //                     $match: {
//     //                         $expr: {
//     //                             $eq: ["$_id", "$$itemId"]
//     //                         }
//     //                     }
//     //                 },
//     //                 // Add a lookup stage for the "Orders" collection
//     //                 {
//     //                     $lookup: {
//     //                         from: "orders",
//     //                         localField: "itemid",  // Assuming the field in "items" that corresponds to "item.itemid" in "Orders"
//     //                         foreignField: "item.itemid",
//     //                         as: "orders"
//     //                     }
//     //                 }
//     //             ],
//     //             as: "orderedItems"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: "$orderedItems.item_name",
//     //             totalAmount: {
//     //                 $sum: {
//     //                     $multiply: [
//     //                         { $toDouble: "$item.price" },
//     //                         { $toDouble: "$item.weight" }
//     //                     ]
//     //                 }
//     //             },
//     //             totalWeight: {
//     //                 $sum: {
//     //                     $cond: [
//     //                         { $eq: ["$orderedItems", []] },  // Check if the "orderedItems" array is empty (no match)
//     //                         0,  // If no match, set totalWeight to 0
//     //                         { $toDouble: "$item.weight" }  // Otherwise, use the weight value
//     //                     ]
//     //                 }
//     //             },
//     //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     // ];

//     // console.log(req.params.cityid);

//     const pipeline = [
//         {
//             $match: {
//                 $and: [
//                     // { cityid: new ObjectId(req.params.cityid) },
//                     {$or: [
//                         { status: "processed" },
//                         { status: "cash" }
//                     ]}
//                 ]
//             }
//         },
//         {
//             $unwind: "$item"
//         },
//         {
//             $lookup: {
//                 from: "items",
//                 localField: "item.itemid",
//                 foreignField: "_id",
//                 as: "orderedItems"
//             }
//         },
//         {
//             $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//         },
//         {
//             $lookup: {
//                 from: "locations",
//                 localField: "orderedItems.cityid",
//                 foreignField: "cityid",
//                 as: "locationItems"
//             }
//         },
//         {
//             $unwind: { path: "$locationItems", preserveNullAndEmptyArrays: true }
//         },
//         {
//             $match: {
//                 "locationItems.PIN": req.params.cityid
//             }
//         },
//         {
//             $group: {
//                 _id: {
//                     item_name: "$orderedItems.item_name",
//                     // location: "$locationItems.PIN"
//                     location: "$locationItems.PIN",
//                     cityid: "$orderedItems.cityid" 
//                 },
//                 totalAmount: {
//                     $sum: {
//                         $cond: [
//                             { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
//                             0, // If missing, set totalAmount to 0
//                             {
//                                 $multiply: [
//                                     { $toDouble: "$item.price" },
//                                     { $toDouble: "$item.weight" }
//                                 ]
//                             }
//                         ]
//                     }
//                 },
//                 totalWeight: {
//                     $sum: {
//                         $cond: [
//                             { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
//                             0, // If missing, set totalWeight to 0
//                             { $toDouble: "$item.weight" } // Otherwise, use the weight value
//                         ]
//                     }
//                 },
//                 orderedItems: { $first: "$orderedItems" }, // Select the first orderedItems document
//                 // item_name: "$orderedItems.item_name",
//                 cityid: { $first: "$orderedItems.cityid" }
//             }
//         },
//         {
//             $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//         }
//     ];

//     const result = await Orders.aggregate(pipeline);


//     // const pipeline = [
//     //     {
//     //         $match: {
//     //             $or: [
//     //                 { status: "processed" },
//     //                 { status: "cash" }
//     //             ]
//     //         }
//     //     },
//     //     {
//     //         $unwind: "$item"
//     //     },
//     //     {
//     //         $lookup: {
//     //             from: "items",
//     //             localField: "item.itemid",
//     //             foreignField: "_id",
//     //             as: "orderedItems"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: {
//     //                 $ifNull: [
//     //                     "$orderedItems.item_name",
//     //                     "$item.item_name"
//     //                 ]
//     //             },
//     //             totalAmount: {
//     //                 $sum: {
//     //                     $multiply: [
//     //                         { $toDouble: "$item.price" },
//     //                         { $toDouble: "$item.weight" }
//     //                     ]
//     //                 }
//     //             },
//     //             totalWeight: {
//     //                 $sum: {
//     //                     $cond: [
//     //                         { $eq: ["$orderedItems", []] }, // Check if "orderedItems" array is empty (no match)
//     //                         { $toDouble: "$item.weight" }, // If no match, use the weight value
//     //                         0 // Otherwise, set totalWeight to 0
//     //                     ]
//     //                 }
//     //             },
//     //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     // ];

//     const existingItemNames = result.map((item) => item._id);

//     var x = []
//     // const existingItemNames = result.map((item) => {
//     //     console.log(x.push(item.cityid));
//     // });

//     console.log(existingItemNames);

//     const missingItemsPipeline = [
//         {
//             $match: {
//                 // item_name: existingItemNames[0].item_name,
//                 // cityid: x[0]
//                 // cityid: req.params.cityid,
//                 // cityid: existingItemNames[0].cityid,
//                 item_name: { $nin: existingItemNames }
//             }
//         },
//         {
//             $group: {
//                 _id: {
//                     // item_name: "$item_name",
//                     cityid: "$cityid"
//                 },
//                 totalAmount: { $sum: 0 }, 
//                 totalWeight: { $sum: 0 }, 
//                 orderedItems: { $first: "$$ROOT" },
//                 city: { $first: "$$ROOT.cityid" }
//             }
//         },
//         {
//             $sort: {
//                 _id: 1 
//             }
//         }
//     ];

//     const missingItems = await Items.aggregate(missingItemsPipeline);

//     // Concatenate the results
//     const finalResult = result.concat(missingItems);

//     // Sort the final result by totalAmount in descending order
//     const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);

//     // Use sortedResult for further processing or display
//     console.log(finalResult);




//     if (result) {
//         res.json({
//             isSuccess: true,
//             message: "Items sort by estimate purchase.",
//             result: sortedResult,
//         });
//     } else {
//         res.json({
//             isSuccess: false,
//             message: "Something went wrong.",
//         });
//     }
// }


// export const getAllItemsPurbyCityid = async (req, res) => {
//     // const pipeline = [
//     //     {
//     //       $match: {
//     //         $or: [
//     //           { status: "processed" },
//     //           { status: "cash" }
//     //         ]
//     //       }
//     //     },
//     //     {
//     //       $unwind: "$item"
//     //     },
//     //     {
//     //       $lookup: {
//     //         from: "items",
//     //         localField: "item.itemid",
//     //         foreignField: "_id",
//     //         as: "orderedItems"
//     //       }
//     //     },
//     //     {
//     //       $unwind: "$orderedItems"
//     //     },
//     //     {
//     //       $group: {
//     //         _id: "$orderedItems.item_name",
//     //         totalAmount: {
//     //           $sum: {
//     //             $multiply: [
//     //               { $toDouble: "$item.price" },
//     //               { $toDouble: "$item.weight" }
//     //             ]
//     //           }
//     //         },
//     //         totalWeight: { $sum: { $toDouble: "$item.weight" } },
//     //         orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //       }
//     //     },
//     //     {
//     //       $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     //   ];

//     // const pipeline = [
//     //     {
//     //         $unwind: "$items"
//     //     },
//     //     {
//     //         $lookup: {
//     //             from: "orders",
//     //             let: { orderId: "$_id", itemId: "$items.itemid" },
//     //             pipeline: [
//     //                 {
//     //                     $match: {
//     //                         $expr: {
//     //                             $and: [
//     //                                 { $in: ["$$itemId", "$items.itemid"] },
//     //                                 { $in: ["$$orderId", "$_id"] },
//     //                                 { $in: ["processed", "$status", "$cash"] }
//     //                             ]
//     //                         }
//     //                     }
//     //                 },
//     //                 {
//     //                     $unwind: "$items"
//     //                 },
//     //                 {
//     //                     $lookup: {
//     //                         from: "items",
//     //                         localField: "items.itemid",
//     //                         foreignField: "_id",
//     //                         as: "orderedItems"
//     //                     }
//     //                 },
//     //                 {
//     //                     $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //                 },
//     //                 {
//     //                     $group: {
//     //                         _id: "$orderedItems.item_name",
//     //                         totalAmount: {
//     //                             $sum: {
//     //                                 $multiply: [
//     //                                     { $toDouble: "$items.price" },
//     //                                     { $toDouble: "$items.weight" }
//     //                                 ]
//     //                             }
//     //                         },
//     //                         totalWeight: { $sum: { $toDouble: "$items.weight" } },
//     //                         orderedItems: { $first: "$orderedItems" }
//     //                     }
//     //                 }
//     //             ],
//     //             as: "joinedOrders"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$joinedOrders", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: "$joinedOrders._id",
//     //             totalAmount: { $sum: "$joinedOrders.totalAmount" },
//     //             totalWeight: { $sum: "$joinedOrders.totalWeight" },
//     //             orderedItems: { $first: "$joinedOrders.orderedItems" }
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 }
//     //     }
//     // ];

//     // const pipeline = [
//     //     {
//     //         $lookup: {
//     //             from: "items",
//     //             let: { itemId: "$_id" },
//     //             pipeline: [
//     //                 {
//     //                     $match: {
//     //                         $expr: {
//     //                             $eq: ["$_id", "$$itemId"]
//     //                         }
//     //                     }
//     //                 },
//     //                 // Add a lookup stage for the "Orders" collection
//     //                 {
//     //                     $lookup: {
//     //                         from: "orders",
//     //                         localField: "itemid",  // Assuming the field in "items" that corresponds to "item.itemid" in "Orders"
//     //                         foreignField: "item.itemid",
//     //                         as: "orders"
//     //                     }
//     //                 }
//     //             ],
//     //             as: "orderedItems"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: "$orderedItems.item_name",
//     //             totalAmount: {
//     //                 $sum: {
//     //                     $multiply: [
//     //                         { $toDouble: "$item.price" },
//     //                         { $toDouble: "$item.weight" }
//     //                     ]
//     //                 }
//     //             },
//     //             totalWeight: {
//     //                 $sum: {
//     //                     $cond: [
//     //                         { $eq: ["$orderedItems", []] },  // Check if the "orderedItems" array is empty (no match)
//     //                         0,  // If no match, set totalWeight to 0
//     //                         { $toDouble: "$item.weight" }  // Otherwise, use the weight value
//     //                     ]
//     //                 }
//     //             },
//     //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     // ];

//     // console.log(req.params.cityid);

//     const pipeline = [
//         {
//             $match: {
//                 $and: [
//                     // { cityid: new ObjectId(req.params.cityid) },
//                     {$or: [
//                         { status: "processed" },
//                         { status: "cash" }
//                     ]}
//                 ]
//             }
//         },
//         {
//             $unwind: "$item"
//         },
//         {
//             $lookup: {
//                 from: "items",
//                 localField: "item.itemid",
//                 foreignField: "_id",
//                 as: "orderedItems"
//             }
//         },
//         {
//             $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//         },
//         {
//             $lookup: {
//                 from: "locations",
//                 localField: "orderedItems.cityid",
//                 foreignField: "cityid",
//                 as: "locationItems"
//             }
//         },
//         {
//             $unwind: { path: "$locationItems", preserveNullAndEmptyArrays: true }
//         },
//         {
//             $match: {
//                 "locationItems.PIN": req.params.cityid
//             }
//         },
//         {
//             $group: {
//                 _id: {
//                     item_name: "$orderedItems.item_name",
//                     // location: "$locationItems.PIN"
//                     location: "$locationItems.PIN",
//                 },
//                 totalAmount: {
//                     $sum: {
//                         $cond: [
//                             { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
//                             0, // If missing, set totalAmount to 0
//                             {
//                                 $multiply: [
//                                     { $toDouble: "$item.price" },
//                                     { $toDouble: "$item.weight" }
//                                 ]
//                             }
//                         ]
//                     }
//                 },
//                 totalWeight: {
//                     $sum: {
//                         $cond: [
//                             { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
//                             0, // If missing, set totalWeight to 0
//                             { $toDouble: "$item.weight" } // Otherwise, use the weight value
//                         ]
//                     }
//                 },
//                 orderedItems: { $first: "$orderedItems" }, // Select the first orderedItems document
//                 location: { $first: "$orderedItems.cityid" }
//             }
//         },
//         {
//             $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//         }
//     ];

//     const result = await Orders.aggregate(pipeline);


//     // const pipeline = [
//     //     {
//     //         $match: {
//     //             $or: [
//     //                 { status: "processed" },
//     //                 { status: "cash" }
//     //             ]
//     //         }
//     //     },
//     //     {
//     //         $unwind: "$item"
//     //     },
//     //     {
//     //         $lookup: {
//     //             from: "items",
//     //             localField: "item.itemid",
//     //             foreignField: "_id",
//     //             as: "orderedItems"
//     //         }
//     //     },
//     //     {
//     //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
//     //     },
//     //     {
//     //         $group: {
//     //             _id: {
//     //                 $ifNull: [
//     //                     "$orderedItems.item_name",
//     //                     "$item.item_name"
//     //                 ]
//     //             },
//     //             totalAmount: {
//     //                 $sum: {
//     //                     $multiply: [
//     //                         { $toDouble: "$item.price" },
//     //                         { $toDouble: "$item.weight" }
//     //                     ]
//     //                 }
//     //             },
//     //             totalWeight: {
//     //                 $sum: {
//     //                     $cond: [
//     //                         { $eq: ["$orderedItems", []] }, // Check if "orderedItems" array is empty (no match)
//     //                         { $toDouble: "$item.weight" }, // If no match, use the weight value
//     //                         0 // Otherwise, set totalWeight to 0
//     //                     ]
//     //                 }
//     //             },
//     //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
//     //         }
//     //     },
//     //     {
//     //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
//     //     }
//     // ];


//     const existingItemNames = result.map((item) => item._id);

//     const missingItemsPipeline = [
//         {
//             $match: {
//                 // cityid: new ObjectId(req.params.cityid) ,
//                 item_name: { $nin: existingItemNames },
//                 cityid: new ObjectId(existingItemNames.cityid)
//             }
//         },
//         {
//             $group: {
//                 _id: {
//                     item_name: "$item_name",
//                     cityid: "$cityid"
//                 },
//                 totalAmount: { $sum: 0 }, 
//                 totalWeight: { $sum: 0 }, 
//                 orderedItems: { $first: "$$ROOT" },
//                 city: { $first: "$$ROOT.cityid" }
//             }
//         },
//         {
//             $sort: {
//                 _id: 1 
//             }
//         }
//     ];

//     const missingItems = await Items.aggregate(missingItemsPipeline);

//     // Concatenate the results
//     const finalResult = result.concat(missingItems);

//     // Sort the final result by totalAmount in descending order
//     const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);

//     // Use sortedResult for further processing or display
//     console.log(sortedResult);




//     if (result) {
//         res.json({
//             isSuccess: true,
//             message: "Items sort by estimate purchase.",
//             result: sortedResult,
//         });
//     } else {
//         res.json({
//             isSuccess: false,
//             message: "Something went wrong.",
//         });
//     }
// }


// Get all Items for Customer View
export const getActiveItems = async (req, res) => {
    const items = await Items.find({ isActive: true, isDelete: false })
        .collation({ locale: 'en', strength: 2 })                   //  for case-insensitive sorting
        .populate('category', 'item_name') // Populate the category details
        .sort({ item_name: 1 }).exec();
    res.send(items)
}


// Get particular Item by ID
export const getItemsById = async (req, res) => {
    const items = await Items.find({ _id: new ObjectId(req.params.id), isDelete: false })
        .populate('category', 'item_name');      // Populate the category details
    res.send(items)
}


// Update an Item
export const updateItems = async (req, res) => {



    // console.log(process.env.MAILAPIKEY)
    // console.log(process.env.APIKEY)




    upload(req, res, async (err) => {
        const item = await Items.findById(req.body._id);
        console.log("item details are", item);

        

        if (err) {
            console.log(err);
            return res.send({
                error: true,
                message: "File upload failed"
            });
        }

        const updateData = {
            $set: { "item_name": req.body.item_name, "price": req.body.price, "dealer_price": req.body.dealer_price, "category": req.body.category, "unit": req.body.unit, "description": req.body.description, "itemLang": JSON.parse(req.body.itemLang) },
        };




        const images = [];
        const imageURLs = [];

        if (req.files && req.files.length > 0) {
            req.files.forEach((file) => {
                images.push({
                    contentType: 'image',
                    data: file.filename
                });
                imageURLs.push('uploads/' + file.filename);
            });
        }
        try{
            const item = await Items.findById(req.body._id);
            var previousPrice = item.price;
            previousPrice = parseFloat(previousPrice).toFixed(2); 
            console.log("the previous price was", previousPrice);
            //to implement the logic in which city the updates hasbeen made
           

        }catch(error){
           console.log(error);
        }


        try {
            // Update images only if new images are uploaded
            if (images.length > 0) {
                const item = await Items.findById(req.body._id);
               

                if (!item) {
                    return res.status(404).json({ message: "Item not found" });
                }

                item.image = item.image.concat(images);
                item.imageURL = item.imageURL.concat(imageURLs);

                await item.save();

            }
             
            // Update item data
            const updateItems = await Items.updateOne({ _id: new ObjectId(req.body._id) }, updateData);


            if (updateItems) {
                console.log(updateItems)
                // let previousPrice=Items.price;
                // console.log("the previous price was",previousPrice);
                let updatedPrice = req.body.price;
                console.log("the updated price is", updatedPrice)
                var cityy=item.city;
                console.log("the city is",cityy);
                await sendEmailToAdmin( req.body.item_name, previousPrice, updatedPrice,cityy)

                res.status(200).json({ message: "update success" });
                //here we will write the logic fi any update then send the email notification



            } else {
                res.status(200).json({ message: "update failed" });
            }
        } catch (error) {
            console.error(error);
            return res.send({
                error: true,
                message: "Internal server error"
            });
        }

    });
};
//function for send the email to the admin
async function sendEmailToAdmin( itemName, previousPrice, updatedPrice,cityy) {
    var defaultClient = SibApiV3Sdk.ApiClient.instance;
    // Configure API key authorization: api-key
    var apiKey = defaultClient.authentications['api-key'];
    console.log("the api key is", apiKey)
    apiKey.apiKey = process.env.MAILAPIKEY;
    // console.log("the api key after this is", apiKey)
    // Uncomment below two lines to configure authorization using: partner-key
    // var partnerKey = defaultClient.authentications['partner-key'];
    // partnerKey.apiKey = 'YOUR API KEY';
    var apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
    var adminMail = new SibApiV3Sdk.SendSmtpEmail();
    const getIndianTime = () => {
        return new Date().toLocaleTimeString('en-IN', { 
            timeZone: 'Asia/Kolkata', 
            hour: '2-digit', 
            minute: '2-digit', 
            second: '2-digit', 
            hour12: false 
        });
    };

    try {

        adminMail = {
            to: [
                
                   { email: 'baniprasad.mangaraj@ajatus.co.in'},
                   { email: 'neelima@ajatus.co.in'},
                   { email: 'omprakash.sadangi@ajatus.co.in'},
                    
                
            ],
            from: {
                email: 'support@ceibagreen.com'
            },
            templateId: 27,
            params: {
                parameter: 'recycle_pay',
                subject: `Item updated ${itemName}`,
                body: `Item updated ${itemName} from ${previousPrice} to ${updatedPrice}`,
                date: new Date().toLocaleDateString(),
                time: getIndianTime(),
                item_name: itemName,
                previous_price: `₹${previousPrice}`,
                updated_price: `₹${updatedPrice}`,
                city:cityy

            },
            subject: `Item updated ${itemName}`,
            headers: {
                'X-Mailin-custom': 'custom_header_1:custom_value_1|custom_header_2:custom_value_2'
            },
        }
        const data = await apiInstance.sendTransacEmail(adminMail);
        console.log("mail send successfully", data.messageId);
        console.log(itemName)
    } catch (error) {
        console.log(error);
    }
}


// Activate an item
export const activateItemById = async (req, res) => {
    try {
        const items = await Items.findByIdAndUpdate({ _id: new ObjectId(req.params.id) }, { isActive: true }, { new: true });
        res.send(items);
    } catch (error) {
        console.error(error);
        res.send({ error: 'Internal Server Error' });
    }
};

// Deactivate an item
export const deactivateItemById = async (req, res) => {
    try {
        const items = await Items.findByIdAndUpdate({ _id: new ObjectId(req.params.id) }, { isActive: false }, { new: true });
        res.send(items);
    } catch (error) {
        console.error(error);
        res.send({ error: 'Internal Server Error' });
    }
};


// Delete Items by ID
export const deleteItemsById = async (req, res) => {
    const items = await Items.updateOne({ _id: new ObjectId(req.params.id) }, { $set: { "isActive": false, "isDelete": true } });
    if (items) {
        res.send("deleted successfully")
    } else {
        res.send("deletion failed")
    }
}


// Search an item by Name
export const searchItem = async (req, res) => {
    // try {
    //     const item_name = req.query.item_name;
    //     const searchItem = item_name.trim();

    //     const projection = {
    //         _id: 0,
    //         item_name: 1,
    //     };

    //     // Check if an item with the same name already exists
    //     // const existingItem = await Items.find({$text:{$search:searchItem}});
    //     const existingItem = await Items.find({ item_name: { $regex: new RegExp('^' + searchItem, 'i') } });

    //     if (existingItem.length !== 0) {
    //         return res.send({ error: false, existingItem, message: "Item found" });
    //     } else {
    //         return res.send({ error: true, message: "Item not found" });
    //     }
    // } catch (error) {
    //     console.error(error);
    //     res.send({ error: 'Internal Server Error' });
    // }

    const item_name = req.query.item_name;
    const searchItem = item_name.trim();
    const pipeline = [
        {
            $match: {
                $or: [
                    { status: "processed" },
                    { status: "cash" }
                ]
            }
        },
        {
            $unwind: "$item"
        },
        {
            $lookup: {
                from: "items",
                localField: "item.itemid",
                foreignField: "_id",
                as: "orderedItems"
            }
        },
        {
            $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
        },
        {
            $group: {
                _id: "$orderedItems.item_name",
                totalAmount: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalAmount to 0
                            {
                                $multiply: [
                                    { $toDouble: "$item.price" },
                                    { $toDouble: "$item.weight" }
                                ]
                            }
                        ]
                    }
                },
                totalWeight: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalWeight to 0
                            { $toDouble: "$item.weight" } // Otherwise, use the weight value
                        ]
                    }
                },
                orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
            }
        },
        {
            $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
        }
    ];

    const result = await Orders.aggregate(pipeline);

    const existingItemNames = result.map((item) => item._id);

    // Find the missing items from the "items" collection
    const missingItemsPipeline = [
        {
            $match: {
                $and: [
                    { item_name: { $nin: existingItemNames } },
                    { item_name: { $regex: new RegExp('^' + searchItem, 'i') } } // Match item_name with regex
                ]
            }
        },
        {
            $group: {
                _id: "$item_name",
                totalAmount: { $sum: 0 }, // Set totalAmount to 0 for missing items
                totalWeight: { $sum: 0 }, // Set totalWeight to 0 for missing items
                orderedItems: { $first: "$$ROOT" } // Select the first document for missing items
            }
        }
    ];

    const missingItems = await Items.aggregate(missingItemsPipeline);

    const finalResult = result.concat(missingItems);

    const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);

    const filteredResult = sortedResult.filter(item => {
        const itemId = item._id;
        return itemId && itemId.match(new RegExp('^' + searchItem, 'i'));
      });
      
      console.log(filteredResult);
      
    if (result) {
        res.json({
            isSuccess: true,
            message: "Item found.",
            result: filteredResult,
        });
    } else {
        res.json({
            isSuccess: false,
            result: [],
            message: "Item not found.",
        });
    }
}


// Add City & percentage
export const addCityPercent = async (req, res) => {
    const { city, percentage } = req.body;

    // Check if an item with the same name & city already exists
    const existingCity = await CityPercent.findOne({ city: { $regex: new RegExp(`^${city}$`, 'i') } });

    if (existingCity) {
        return res.send({ "uploaded": false, "message": "City already exists." });
    }

    const newCity = new CityPercent({
        city: city,
        percentage: percentage
    });

    try {
        await newCity.save();
        res.send({ "uploaded": true });
    } catch (error) {
        console.error(error);
        res.send({ "uploaded": false });
    }
};


// Get all City & percentage
export const getAllCityPercent = async (req, res) => {
    try {
        const cities = await CityPercent.find({ isDelete: false })
        .collation({ locale: 'en', strength: 2 }) // Case insensitive sorting
        .sort({ city: 1 }) // Sort alphabetically by city name
        .exec();
        res.send({cities});
    } catch (error) {
        console.error("Error occurred while fetching cities:", error);
        res.status(500).json({ error: 'An error occurred while fetching cities' });
    }
};






// ItemCategory find by cityid
export const itemCatInd = async (req, res) => {
    console.log(new ObjectId(req.params.cityid));
    await CityPercent.findOne({ _id: new ObjectId(req.params.cityid), isDelete: false }).then((ind) => {
        if (ind) {
            return res.send({
                isSuccess: true,
                message: "Get Individual city percentage.",
                item: ind
            });
        } else {
            return res.send({
                isSuccess: false,
                message: "Something went wrong.",
                item: []
            });
        }
    })
}


// add multiple images
export const addMultipleImages = async (req, res) => {
    upload(req, res, async (err) => {
        try {
            if (!req.files || req.files.length === 0) {
                return res.status(400).json({ message: "No files uploaded" });
            }

            const item = await Items.findById(req.params.itemId);

            if (!item) {
                return res.status(404).json({ message: "Item not found" });
            }

            const newImages = [];
            const newImageURLs = [];

            req.files.forEach((file) => {
                const imageObject = {
                    contentType: 'image',
                    data: file.filename
                };
                newImages.push(imageObject);
                newImageURLs.push('uploads/' + file.filename);
            });

            // Concatenate new images to the existing item's images array
            item.image = item.image.concat(newImages);
            item.imageURL = item.imageURL.concat(newImageURLs);

            await item.save();

            res.status(200).json({ message: "All images added successfully" });
        } catch (error) {
            console.error(error);
            res.status(500).json({ error: true, message: "Internal server error" });
        }
    });
};


// API to delete item images one by one
export const deleteItemImage = async (req, res) => {
    try {
        const itemId = req.params.itemId;
        const { index } = req.params;

        const item = await Items.findById(itemId);
        console.log("item ", item);

        if (!item) {
            return res.status(404).json({ message: "Item not found" });
        }

        // Check if the index is within the range of the images array
        if (index < 0 || index >= item.image.length) {
            return res.status(400).json({ message: "Invalid image index" });
        }

        // Create a copy of the item.image array
        const updatedImages = [...item.image];
        const updatedImageURL = [...item.imageURL];

        console.log("updatedImaeURL", updatedImageURL);

        // Remove the specified image from the copy of the array
        updatedImages.splice(index, 1);
        updatedImageURL.splice(index, 1);

        // Update the item with the modified image array
        item.image = updatedImages;
        item.imageURL = updatedImageURL;

        await item.save();

        res.status(200).json({ message: `Image at index ${index} deleted successfully` });
    } catch (error) {
        console.error(error);
        res.status(500).json({ error: true, message: "Internal server error" });
    }
};


// Update city-category percentage, item collection's price
export const updatePrice = async (req, res) => {
  const dealerPriceMultiplier = req.body.percentage / 100;

  try {
    const city = await CityPercent.findByIdAndUpdate(
      { _id: new ObjectId(req.body.cityid) },
      { percentage: req.body.percentage }
    );

    if (!city) {
      return res.send({
        isSuccess: false,
        message: 'City not found.',
        item: []
      });
    }

    await Items.updateMany(
      { cityid: new ObjectId(city._id) },
      [
        {
          $set: {
            price: {
                $round: [
                    {
                      $multiply: ['$dealer_price', { $subtract: [1, dealerPriceMultiplier] }]
                    },
                    2
                  ]
            },
            itemLang: {
              $map: {
                input: '$itemLang',
                as: 'item',
                in: {
                  $mergeObjects: [
                    '$$item',
                    {
                      price: {
                        $round: [
                            {
                              $multiply: ['$dealer_price', { $subtract: [1, dealerPriceMultiplier] }]
                            },
                            2
                          ]
                      }
                    }
                  ]
                }
              }
            }
          }
        }
      ]
    );

    return res.send({
      isSuccess: true,
      message: 'Prices updated successfully.'
    });
  } catch (error) {
    console.log(error);
    return res.send({
      isSuccess: false,
      message: 'Error updating prices.',
      error
    });
  }
};





export const getAllItemsPurbyCityid = async (req, res) => {
    // const pipeline = [
    //     {
    //       $match: {
    //         $or: [
    //           { status: "processed" },
    //           { status: "cash" }
    //         ]
    //       }
    //     },
    //     {
    //       $unwind: "$item"
    //     },
    //     {
    //       $lookup: {
    //         from: "items",
    //         localField: "item.itemid",
    //         foreignField: "_id",
    //         as: "orderedItems"
    //       }
    //     },
    //     {
    //       $unwind: "$orderedItems"
    //     },
    //     {
    //       $group: {
    //         _id: "$orderedItems.item_name",
    //         totalAmount: {
    //           $sum: {
    //             $multiply: [
    //               { $toDouble: "$item.price" },
    //               { $toDouble: "$item.weight" }
    //             ]
    //           }
    //         },
    //         totalWeight: { $sum: { $toDouble: "$item.weight" } },
    //         orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //       }
    //     },
    //     {
    //       $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    //   ];

    // const pipeline = [
    //     {
    //         $unwind: "$items"
    //     },
    //     {
    //         $lookup: {
    //             from: "orders",
    //             let: { orderId: "$_id", itemId: "$items.itemid" },
    //             pipeline: [
    //                 {
    //                     $match: {
    //                         $expr: {
    //                             $and: [
    //                                 { $in: ["$$itemId", "$items.itemid"] },
    //                                 { $in: ["$$orderId", "$_id"] },
    //                                 { $in: ["processed", "$status", "$cash"] }
    //                             ]
    //                         }
    //                     }
    //                 },
    //                 {
    //                     $unwind: "$items"
    //                 },
    //                 {
    //                     $lookup: {
    //                         from: "items",
    //                         localField: "items.itemid",
    //                         foreignField: "_id",
    //                         as: "orderedItems"
    //                     }
    //                 },
    //                 {
    //                     $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //                 },
    //                 {
    //                     $group: {
    //                         _id: "$orderedItems.item_name",
    //                         totalAmount: {
    //                             $sum: {
    //                                 $multiply: [
    //                                     { $toDouble: "$items.price" },
    //                                     { $toDouble: "$items.weight" }
    //                                 ]
    //                             }
    //                         },
    //                         totalWeight: { $sum: { $toDouble: "$items.weight" } },
    //                         orderedItems: { $first: "$orderedItems" }
    //                     }
    //                 }
    //             ],
    //             as: "joinedOrders"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$joinedOrders", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: "$joinedOrders._id",
    //             totalAmount: { $sum: "$joinedOrders.totalAmount" },
    //             totalWeight: { $sum: "$joinedOrders.totalWeight" },
    //             orderedItems: { $first: "$joinedOrders.orderedItems" }
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 }
    //     }
    // ];

    // const pipeline = [
    //     {
    //         $lookup: {
    //             from: "items",
    //             let: { itemId: "$_id" },
    //             pipeline: [
    //                 {
    //                     $match: {
    //                         $expr: {
    //                             $eq: ["$_id", "$$itemId"]
    //                         }
    //                     }
    //                 },
    //                 // Add a lookup stage for the "Orders" collection
    //                 {
    //                     $lookup: {
    //                         from: "orders",
    //                         localField: "itemid",  // Assuming the field in "items" that corresponds to "item.itemid" in "Orders"
    //                         foreignField: "item.itemid",
    //                         as: "orders"
    //                     }
    //                 }
    //             ],
    //             as: "orderedItems"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: "$orderedItems.item_name",
    //             totalAmount: {
    //                 $sum: {
    //                     $multiply: [
    //                         { $toDouble: "$item.price" },
    //                         { $toDouble: "$item.weight" }
    //                     ]
    //                 }
    //             },
    //             totalWeight: {
    //                 $sum: {
    //                     $cond: [
    //                         { $eq: ["$orderedItems", []] },  // Check if the "orderedItems" array is empty (no match)
    //                         0,  // If no match, set totalWeight to 0
    //                         { $toDouble: "$item.weight" }  // Otherwise, use the weight value
    //                     ]
    //                 }
    //             },
    //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    // ];

    // console.log(req.params.cityid);

    const pipeline = [
        {
            $match: {
                $and: [
                    {$or: [
                        { status: "processed" },
                        { status: "cash" }
                    ]}
                ]
            }
        },
        {
            $unwind: "$item"
        },
        {
            $lookup: {
                from: "items",
                localField: "item.itemid",
                foreignField: "_id",
                as: "orderedItems"
            }
        },
        {
            $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
        },
        {
            $match: {
                "orderedItems.isActive": true
            }
        },
        {
            $lookup: {
                from: "locations",
                localField: "orderedItems.cityid",
                foreignField: "cityid",
                as: "locationItems"
            }
        },
        {
            $unwind: { path: "$locationItems", preserveNullAndEmptyArrays: true }
        },
        {
            $match: {
                "locationItems.PIN": req.params.cityid,
            }
        },
        {
            $group: {
                _id: {
                    item_name: "$orderedItems.item_name",
                    location: "$locationItems.PIN",
                    cityid: "$orderedItems.cityid" 
                },
                totalAmount: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalAmount to 0
                            {
                                $multiply: [
                                    { $toDouble: "$item.price" },
                                    { $toDouble: "$item.weight" }
                                ]
                            }
                        ]
                    }
                },
                totalWeight: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalWeight to 0
                            { $toDouble: "$item.weight" } // Otherwise, use the weight value
                        ]
                    }
                },orderedItems: {
                    $first: {
                        _id: "$orderedItems._id",
                        item_name: "$orderedItems.item_name",
                        price: "$orderedItems.price",
                        dealer_price: "$orderedItems.dealer_price",
                        description: "$orderedItems.description",
                        isActive: "$orderedItems.isActive",
                        isDelete: "$orderedItems.isDelete",
                        createdAt: "$orderedItems.createdAt",
                        updatedAt: "$orderedItems.updatedAt",
                        __v: "$orderedItems.__v",
                        unit: "$orderedItems.unit",
                        category: "$orderedItems.category",
                        city: "$orderedItems.city",
                        cityid: "$orderedItems.cityid",
                        imageURL: {
                            $cond: [
                                { $isArray: "$orderedItems.imageURL" },
                                { $arrayElemAt: ["$orderedItems.imageURL", 0] },
                                "$orderedItems.imageURL"
                            ]
                        },
                        itemimages: {
                            $cond: [
                                { $isArray: "$orderedItems.imageURL" },
                                "$orderedItems.imageURL",
                                ["$orderedItems.imageURL"]
                            ]
                        }
                    }
                },
                cityid: { $first: "$orderedItems.cityid" }
            }
        },
        {
            $sort: { totalAmount: -1 }
        }
    ];
    
    const result = await Orders.aggregate(pipeline);
    
    console.log('result', result);    
    // const pipeline = [
    //     {
    //         $match: {
    //             $or: [
    //                 { status: "processed" },
    //                 { status: "cash" }
    //             ]
    //         }
    //     },
    //     {
    //         $unwind: "$item"
    //     },
    //     {
    //         $lookup: {
    //             from: "items",
    //             localField: "item.itemid",
    //             foreignField: "_id",
    //             as: "orderedItems"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: {
    //                 $ifNull: [
    //                     "$orderedItems.item_name",
    //                     "$item.item_name"
    //                 ]
    //             },
    //             totalAmount: {
    //                 $sum: {
    //                     $multiply: [
    //                         { $toDouble: "$item.price" },
    //                         { $toDouble: "$item.weight" }
    //                     ]
    //                 }
    //             },
    //             totalWeight: {
    //                 $sum: {
    //                     $cond: [
    //                         { $eq: ["$orderedItems", []] }, // Check if "orderedItems" array is empty (no match)
    //                         { $toDouble: "$item.weight" }, // If no match, use the weight value
    //                         0 // Otherwise, set totalWeight to 0
    //                     ]
    //                 }
    //             },
    //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    // ];


    const existingItemNames = result.map((item) => item._id);
    console.log(existingItemNames);

    if (existingItemNames.length != 0) {
        const missingItemsPipeline = [
            {
                $match: {
                    $and: [
                        {
                            item_name: {
                                $nin: existingItemNames.map(item => item.item_name)
                            }
                        },
                        { cityid: result[0].cityid },
                        { isActive: true }
                    ]
                }
            },
            {
                $lookup: {
                    from: "items",
                    localField: "item_name",
                    foreignField: "item_name",
                    as: "orderedItems"
                }
            },
            {
                $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
            },
            {
                $group: {
                    _id: {
                        item_name: "$item_name",
                        cityid: result[0]._id.cityid
                    },
                    totalAmount: { $sum: 0 },
                    totalWeight: { $sum: 0 },
                    orderedItems: {
                        $first: {
                            _id: "$orderedItems._id",
                            item_name: "$orderedItems.item_name",
                            price: "$orderedItems.price",
                            dealer_price: "$orderedItems.dealer_price",
                            description: "$orderedItems.description",
                            isActive: "$orderedItems.isActive",
                            isDelete: "$orderedItems.isDelete",
                            createdAt: "$orderedItems.createdAt",
                            updatedAt: "$orderedItems.updatedAt",
                            __v: "$orderedItems.__v",
                            unit: "$orderedItems.unit",
                            category: "$orderedItems.category",
                            city: "$orderedItems.city",
                            cityid: "$orderedItems.cityid",
                            imageURL: {
                                $cond: [
                                    { $isArray: "$orderedItems.imageURL" },
                                    { $arrayElemAt: ["$orderedItems.imageURL", 0] },
                                    "$orderedItems.imageURL"
                                ]
                            },
                            itemimages: {
                                $cond: [
                                    { $isArray: "$orderedItems.imageURL" },
                                    "$orderedItems.imageURL",
                                    ["$orderedItems.imageURL"]
                                ]
                            }
                        }
                    },
                    city: { $first: "$$ROOT.cityid" }
                }
            },
            {
                $sort: {
                    "_id.item_name": 1
                }
            }
        ];
    
        const missingItems = await Items.aggregate(missingItemsPipeline);
    
        // Concatenate the results
        const finalResult = result.concat(missingItems);
    
        // Sort the final result by totalAmount in descending order
        const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);
    
        if (result) {
            res.json({
                isSuccess: true,
                message: "Items sort by estimate purchase.",
                result: sortedResult,
            });
        } else {
            res.json({
                isSuccess: false,
                message: "Something went wrong.",
            });
        }
    } else {
        await Location.findOne({ PIN: req.params.cityid }).then(async (x) => {
            console.log("1234567890x", x);
            const missingItemsPipeline = [
                {
                    $match: {
                        $and: [
                            { item_name: { $nin: existingItemNames.map(item => item.item_name) } },
                            { cityid: x.cityid }
                        ]
                    }
                },
                {
                    $group: {
                        _id: {
                            item_name: "$item_name",
                            cityid: x.cityid // Use result[0].location directly, assuming it's a valid cityid
                        },
                        totalAmount: { $sum: 0 },
                        totalWeight: { $sum: 0 },
                        orderedItems: {
                            $first: {
                                _id: "$$ROOT._id",
                                item_name: "$$ROOT.item_name",
                                price: "$$ROOT.price",
                                dealer_price: "$$ROOT.dealer_price",
                                description: "$$ROOT.description",
                                isActive: "$$ROOT.isActive",
                                isDelete: "$$ROOT.isDelete",
                                createdAt: "$$ROOT.createdAt",
                                updatedAt: "$$ROOT.updatedAt",
                                __v: "$$ROOT.__v",
                                unit: "$$ROOT.unit",
                                category: "$$ROOT.category",
                                city: "$$ROOT.city",
                                cityid: "$$ROOT.cityid",
                                imageURL: {
                                    $cond: [
                                        { $isArray: "$$ROOT.imageURL" },
                                        { $arrayElemAt: ["$$ROOT.imageURL", 0] },
                                        "$$ROOT.imageURL"
                                    ]
                                },
                                itemimages: {
                                    $cond: [
                                        { $isArray: "$$ROOT.imageURL" },
                                        "$$ROOT.imageURL",
                                        ["$$ROOT.imageURL"]
                                    ]
                                }
                            }
                        },
                        city: { $first: "$$ROOT.cityid" }
                    }
                },
                {
                    $sort: {
                        "_id.item_name": 1
                    }
                }
            ];
    
            const missingItems = await Items.aggregate(missingItemsPipeline);
    
            // Concatenate the results
            const finalResult = result.concat(missingItems);
        
            // Sort the final result by totalAmount in descending order
            const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);      
               
            if (result) {
                res.json({
                    isSuccess: true,
                    message: "Items sort by estimate purchase.",
                    result: sortedResult,
                });
            } else {
                res.json({
                    isSuccess: false,
                    message: "Something went wrong.",
                });
            }
        } )

        
    }
    
}




///comment old name for data faster getAllItemsPurbyCityidByLang to OldgetAllItemsPurbyCityidByLang

export const OldgetAllItemsPurbyCityidByLang = async (req, res) => {
    // const pipeline = [
    //     {
    //       $match: {
    //         $or: [
    //           { status: "processed" },
    //           { status: "cash" }
    //         ]
    //       }
    //     },
    //     {
    //       $unwind: "$item"
    //     },
    //     {
    //       $lookup: {
    //         from: "items",
    //         localField: "item.itemid",
    //         foreignField: "_id",
    //         as: "orderedItems"
    //       }
    //     },
    //     {
    //       $unwind: "$orderedItems"
    //     },
    //     {
    //       $group: {
    //         _id: "$orderedItems.item_name",
    //         totalAmount: {
    //           $sum: {
    //             $multiply: [
    //               { $toDouble: "$item.price" },
    //               { $toDouble: "$item.weight" }
    //             ]
    //           }
    //         },
    //         totalWeight: { $sum: { $toDouble: "$item.weight" } },
    //         orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //       }
    //     },
    //     {
    //       $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    //   ];

    // const pipeline = [
    //     {
    //         $unwind: "$items"
    //     },
    //     {
    //         $lookup: {
    //             from: "orders",
    //             let: { orderId: "$_id", itemId: "$items.itemid" },
    //             pipeline: [
    //                 {
    //                     $match: {
    //                         $expr: {
    //                             $and: [
    //                                 { $in: ["$$itemId", "$items.itemid"] },
    //                                 { $in: ["$$orderId", "$_id"] },
    //                                 { $in: ["processed", "$status", "$cash"] }
    //                             ]
    //                         }
    //                     }
    //                 },
    //                 {
    //                     $unwind: "$items"
    //                 },
    //                 {
    //                     $lookup: {
    //                         from: "items",
    //                         localField: "items.itemid",
    //                         foreignField: "_id",
    //                         as: "orderedItems"
    //                     }
    //                 },
    //                 {
    //                     $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //                 },
    //                 {
    //                     $group: {
    //                         _id: "$orderedItems.item_name",
    //                         totalAmount: {
    //                             $sum: {
    //                                 $multiply: [
    //                                     { $toDouble: "$items.price" },
    //                                     { $toDouble: "$items.weight" }
    //                                 ]
    //                             }
    //                         },
    //                         totalWeight: { $sum: { $toDouble: "$items.weight" } },
    //                         orderedItems: { $first: "$orderedItems" }
    //                     }
    //                 }
    //             ],
    //             as: "joinedOrders"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$joinedOrders", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: "$joinedOrders._id",
    //             totalAmount: { $sum: "$joinedOrders.totalAmount" },
    //             totalWeight: { $sum: "$joinedOrders.totalWeight" },
    //             orderedItems: { $first: "$joinedOrders.orderedItems" }
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 }
    //     }
    // ];

    // const pipeline = [
    //     {
    //         $lookup: {
    //             from: "items",
    //             let: { itemId: "$_id" },
    //             pipeline: [
    //                 {
    //                     $match: {
    //                         $expr: {
    //                             $eq: ["$_id", "$$itemId"]
    //                         }
    //                     }
    //                 },
    //                 // Add a lookup stage for the "Orders" collection
    //                 {
    //                     $lookup: {
    //                         from: "orders",
    //                         localField: "itemid",  // Assuming the field in "items" that corresponds to "item.itemid" in "Orders"
    //                         foreignField: "item.itemid",
    //                         as: "orders"
    //                     }
    //                 }
    //             ],
    //             as: "orderedItems"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: "$orderedItems.item_name",
    //             totalAmount: {
    //                 $sum: {
    //                     $multiply: [
    //                         { $toDouble: "$item.price" },
    //                         { $toDouble: "$item.weight" }
    //                     ]
    //                 }
    //             },
    //             totalWeight: {
    //                 $sum: {
    //                     $cond: [
    //                         { $eq: ["$orderedItems", []] },  // Check if the "orderedItems" array is empty (no match)
    //                         0,  // If no match, set totalWeight to 0
    //                         { $toDouble: "$item.weight" }  // Otherwise, use the weight value
    //                     ]
    //                 }
    //             },
    //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    // ];

    // console.log(req.params.cityid);


    const pipeline = [
        {
            $match: {
                $and: [
                    {$or: [
                        { status: "processed" },
                        { status: "cash" }
                    ]}
                ]
            }
        },
        {
            $unwind: "$item"
        },
        {
            $lookup: {
                from: "items",
                localField: "item.itemid",
                foreignField: "_id",
                as: "orderedItems"
            }
        },
        {
            $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
        },
        {
            $match: {
                "orderedItems.isActive": true
            }
        },
        {
            $lookup: {
                from: "locations",
                localField: "orderedItems.cityid",
                foreignField: "cityid",
                as: "locationItems"
            }
        },
        {
            $unwind: { path: "$locationItems", preserveNullAndEmptyArrays: true }
        },
        {
            $match: {
                "locationItems.PIN": req.params.cityid,
            }
        },
        {
            $addFields: {
                "orderedItems.itemLang": {
                    $filter: {
                        input: "$orderedItems.itemLang",
                        as: "item",
                        cond: { $eq: ["$$item.language", req.params.language] }
                    }
                }
            }
        },
        {
            $group: {
                _id: {
                    item_name: "$orderedItems.item_name",
                    location: "$locationItems.PIN",
                    cityid: "$orderedItems.cityid" 
                },
                totalAmount: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalAmount to 0
                            {
                                $multiply: [
                                    { $toDouble: "$item.price" },
                                    { $toDouble: "$item.weight" }
                                ]
                            }
                        ]
                    }
                },
                totalWeight: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalWeight to 0
                            { $toDouble: "$item.weight" } // Otherwise, use the weight value
                        ]
                    }
                },
                orderedItems: {
                    $first: {
                        _id: "$orderedItems._id",
                        item_name: "$orderedItems.item_name",
                        price: "$orderedItems.price",
                        dealer_price: "$orderedItems.dealer_price",
                        description: "$orderedItems.description",
                        isActive: "$orderedItems.isActive",
                        isDelete: "$orderedItems.isDelete",
                        createdAt: "$orderedItems.createdAt",
                        updatedAt: "$orderedItems.updatedAt",
                        __v: "$orderedItems.__v",
                        unit: "$orderedItems.unit",
                        category: "$orderedItems.category",
                        city: "$orderedItems.city",
                        cityid: "$orderedItems.cityid",
                        imageURL: {
                            $cond: [
                                { $isArray: "$orderedItems.imageURL" },
                                { $arrayElemAt: ["$orderedItems.imageURL", 0] },
                                "$orderedItems.imageURL"
                            ]
                        },
                        itemimages: {
                            $cond: [
                                { $isArray: "$orderedItems.imageURL" },
                                "$orderedItems.imageURL",
                                ["$orderedItems.imageURL"]
                            ]
                        },
                        itemLang: "$orderedItems.itemLang", // Include the itemLang field
                        orderedItemsitemLang: [{ $first: "$orderedItems.itemLang" }],
                    }
                }, // Select the first orderedItems document
                orderedItemsitemLang: { $first: "$orderedItems.itemLang" }, // Select the first orderedItems document
                cityid: { $first: "$orderedItems.cityid" }
            }
        },
        {
            $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
        }
    ];

    
    
    
    

    const result = await Orders.aggregate(pipeline);

    console.log('result', result);
    // const pipeline = [
    //     {
    //         $match: {
    //             $or: [
    //                 { status: "processed" },
    //                 { status: "cash" }
    //             ]
    //         }
    //     },
    //     {
    //         $unwind: "$item"
    //     },
    //     {
    //         $lookup: {
    //             from: "items",
    //             localField: "item.itemid",
    //             foreignField: "_id",
    //             as: "orderedItems"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: {
    //                 $ifNull: [
    //                     "$orderedItems.item_name",
    //                     "$item.item_name"
    //                 ]
    //             },
    //             totalAmount: {
    //                 $sum: {
    //                     $multiply: [
    //                         { $toDouble: "$item.price" },
    //                         { $toDouble: "$item.weight" }
    //                     ]
    //                 }
    //             },
    //             totalWeight: {
    //                 $sum: {
    //                     $cond: [
    //                         { $eq: ["$orderedItems", []] }, // Check if "orderedItems" array is empty (no match)
    //                         { $toDouble: "$item.weight" }, // If no match, use the weight value
    //                         0 // Otherwise, set totalWeight to 0
    //                     ]
    //                 }
    //             },
    //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    // ];


    const existingItemNames = result.map((item) => item._id);
    console.log(existingItemNames);

    if (existingItemNames.length != 0) {
        const missingItemsPipeline = [
            {
                $match: {
                    $and: [
                        { item_name: { $nin: existingItemNames.map(item => item.item_name) } },
                        { cityid: result[0].cityid },
                        { isActive: true }
                    ]
                }
            },
            {
                $addFields: {
                    orderedItemsitemLang: {
                        $cond: {
                            if: { $isArray: "$itemLang" },
                            then: {
                                $filter: {
                                    input: "$itemLang",
                                    as: "item",
                                    cond: { $eq: ["$$item.language", req.params.language] }
                                }
                            },
                            else: "$$REMOVE"
                        }
                    }
                }
            },
            {
                $match: { orderedItemsitemLang: { $exists: true, $ne: [] } }
            },
            {
                $group: {
                    _id: {
                        item_name: "$item_name",
                        cityid: result[0].cityid
                    },
                    totalAmount: { $sum: 0 },
                    totalWeight: { $sum: 0 },
                    orderedItems: {
                        $first: {
                            _id: "$_id",
                            item_name: "$item_name",
                            price: "$price",
                            dealer_price: "$dealer_price",
                            description: "$description",
                            isActive: "$isActive",
                            isDelete: "$isDelete",
                            createdAt: "$createdAt",
                            updatedAt: "$updatedAt",
                            __v: "$__v",
                            unit: "$unit",
                            category: "$category",
                            city: "$city",
                            cityid: "$cityid",
                            imageURL: {
                                $cond: [
                                    { $isArray: "$imageURL" },
                                    { $arrayElemAt: ["$imageURL", 0] },
                                    "$imageURL"
                                ]
                            },
                            itemimages: {
                                $cond: [
                                    { $isArray: "$imageURL" },
                                    "$imageURL",
                                    ["$imageURL"]
                                ],
                            },
                            itemLang: "$$ROOT.itemLang",
                            orderedItemsitemLang: [{ $first: "$orderedItemsitemLang" }],
                        }
                    },
                    city: { $first: "$$ROOT.cityid" }
                }
            },
            {
                $sort: {
                    "_id.item_name": 1
                }
            }
        ];        
    
        const missingItems = await Items.aggregate(missingItemsPipeline);
    
        // Concatenate the results
        const finalResult = result.concat(missingItems);
    
        // Sort the final result by totalAmount in descending order
        const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);
    
        // Use sortedResult for further processing or display
        console.log('missingItems', missingItems);
    
        if (result) {
            res.json({
                isSuccess: true,
                message: "Items sort by estimate purchase.",
                result: sortedResult,
            });
        } else {
            res.json({
                isSuccess: false,
                message: "Something went wrong.",
            });
        }
    } 
    else {

        await Location.findOne({ PIN: req.params.cityid }).then(async (x) => {
            console.log("1234567890x", x);
            const missingItemsPipeline = [
                {
                    $match: {
                        $and: [
                            { item_name: { $nin: existingItemNames.map(item => item.item_name) } },
                            { cityid: x.cityid }
                        ]
                    }
                },
                {
                    $group: {
                        _id: {
                            item_name: "$item_name",
                            cityid: x.cityid // Use result[0].location directly, assuming it's a valid cityid
                        },
                        totalAmount: { $sum: 0 },
                        totalWeight: { $sum: 0 },
                        orderedItems: {
                            $first: {
                                _id: "$$ROOT._id",
                                item_name: "$$ROOT.item_name",
                                price: "$$ROOT.price",
                                dealer_price: "$$ROOT.dealer_price",
                                description: "$$ROOT.description",
                                isActive: "$$ROOT.isActive",
                                isDelete: "$$ROOT.isDelete",
                                createdAt: "$$ROOT.createdAt",
                                updatedAt: "$$ROOT.updatedAt",
                                __v: "$$ROOT.__v",
                                unit: "$$ROOT.unit",
                                category: "$$ROOT.category",
                                city: "$$ROOT.city",
                                cityid: "$$ROOT.cityid",
                                imageURL: {
                                    $cond: [
                                        { $isArray: "$$ROOT.imageURL" },
                                        { $arrayElemAt: ["$$ROOT.imageURL", 0] },
                                        "$$ROOT.imageURL"
                                    ]
                                },
                                itemimages: {
                                    $cond: [
                                        { $isArray: "$$ROOT.imageURL" },
                                        "$$ROOT.imageURL",
                                        ["$$ROOT.imageURL"]
                                    ],
                                },                                
                                itemLang: "$$ROOT.itemLang"
                            }
                        },
                        orderedItemsitemLang: {
                            $first: {
                                $cond: {
                                    if: { $isArray: "$itemLang" },
                                    then: {
                                        $arrayElemAt: [
                                            {
                                                $filter: {
                                                    input: "$itemLang",
                                                    as: "langItem",
                                                    cond: { $eq: ["$$langItem.language", req.params.language]}
                                                }
                                            },
                                            0
                                        ]
                                    },
                                    else: null
                                }
                            }
                        },
                        city: { $first: "$$ROOT.cityid" }
                    }
                },
                {
                    $sort: {
                        "_id.item_name": 1
                    }
                }
            ];
            
            
        
            const missingItems = await Items.aggregate(missingItemsPipeline);
        
            // Concatenate the results
            const finalResult = result.concat(missingItems);
        
            // Sort the final result by totalAmount in descending order
            const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);   
        
        
        
            if (result) {
                res.json({
                    isSuccess: true,
                    message: "Items sort by estimate purchase.",
                    result: sortedResult,
                });
            } else {
                res.json({
                    isSuccess: false,
                    message: "Something went wrong.",
                });
            }
        } )

        
    }
    
}

export const getAllItemsPurbyCityidByLang=async(req,res)=>{
    try {
        const { cityid, language } = req.params;
        console.log("language is",language)

        // Find the location based on cityid
        const location = await Location.findOne({ PIN: cityid });

        // Validate if location exists
        if (!location) {
            return res.status(200).json({
                isSuccess: false,
                message: "Items sort by estimate purchase.",
                result: [],
            });
        }

        // Aggregation pipeline for missing items
        const missingItemsPipeline = [
            {
                $match: { cityid: location.cityid }
            },
            {
                $group: {
                    _id: {
                        item_name: "$item_name",
                        cityid: location.cityid
                    },
                    totalAmount: { $sum: 0 },
                    totalWeight: { $sum: 0 },
                    orderedItems: {
                        $first: {
                            _id: "$$ROOT._id",
                            item_name: "$$ROOT.item_name",
                            price: "$$ROOT.price",
                            dealer_price: "$$ROOT.dealer_price",
                            description: "$$ROOT.description",
                            isActive: "$$ROOT.isActive",
                            isDelete: "$$ROOT.isDelete",
                            createdAt: "$$ROOT.createdAt",
                            updatedAt: "$$ROOT.updatedAt",
                            __v: "$$ROOT.__v",
                            unit: "$$ROOT.unit",
                            category: "$$ROOT.category",
                            city: "$$ROOT.city",
                            cityid: "$$ROOT.cityid",
                            imageURL: {
                                $cond: [
                                    { $isArray: "$$ROOT.imageURL" },
                                    { $arrayElemAt: ["$$ROOT.imageURL", 0] },
                                    "$$ROOT.imageURL"
                                ]
                            },
                            itemimages: {
                                $cond: [
                                    { $isArray: "$$ROOT.imageURL" },
                                    "$$ROOT.imageURL",
                                    ["$$ROOT.imageURL"]
                                ],
                            },
                            itemLang: "$$ROOT.itemLang"
                        }
                    },
                    showLang: {
                        $first: {
                            $cond: {
                                if: { $isArray: "$itemLang" },
                                then: {
                                    $arrayElemAt: [
                                        {
                                            $filter: {
                                                input: "$itemLang",
                                                as: "langItem",
                                                cond: { $eq: ["$$langItem.language", language] }
                                            }
                                        },
                                        0
                                    ]
                                },
                                else: null
                            }
                        }
                    },
                    city: { $first: "$$ROOT.cityid" }
                }
            },
            {
                $addFields: {
                    "orderedItems.orderedItemsitemLang": {
        $cond: {
          if: { $ne: ["$showLang", null] },
          then: ["$showLang"], // Wrap itemLang inside an array
          else: [], // If no itemLang found, return an empty array
        },
      },
                  customSort: {
                    $cond: [
                      { $in: ["$_id.item_name", ["କାର୍ଡବୋର୍ଡ", "Cardboard", "गत्ता"]] }, 
                      0, // These items should come first
                      1  // All other items should follow
                    ]
                  }
                }
              },
            {
                $sort: {
                    customSort: 1,  // Sort by customSort first
                    "_id.item_name": 1  // Then sort alphabetically by item name
                  }
            }
        ];

        // Execute the aggregation
        const missingItems = await Items.aggregate(missingItemsPipeline);

        // Send the response
        res.json({
            isSuccess: true,
            message: "Items sorted by estimated purchase.",
            result: missingItems,
        });

    } catch (error) {
        console.error("Error fetching items:", error);
        res.status(500).json({
            isSuccess: false,
            message: "An error occurred while fetching items.",
        });
    }
}







export const searchItemByZIP = async (req, res) => {

    const item_name = req.query.item_name;
    const searchItem = item_name.trim();

    const pipeline = [
        {
            $match: {
                $and: [
                    // { cityid: new ObjectId(req.params.cityid) },
                    {$or: [
                        { status: "processed" },
                        { status: "cash" }
                    ]}
                ]
            }
        },
        {
            $unwind: "$item"
        },
        {
            $lookup: {
                from: "items",
                localField: "item.itemid",
                foreignField: "_id",
                as: "orderedItems"
            }
        },
        {
            $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
        },
        {
            $lookup: {
                from: "locations",
                localField: "orderedItems.cityid",
                foreignField: "cityid",
                as: "locationItems"
            }
        },
        {
            $unwind: { path: "$locationItems", preserveNullAndEmptyArrays: true }
        },
        {
            $match: {
                "locationItems.PIN": req.query.ZIP
            }
        },
        {
            $group: {
                _id: {
                    item_name: "$orderedItems.item_name",
                    // location: "$locationItems.PIN",
                    // locationId: "$locationItems._id"
                },
                totalAmount: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalAmount to 0
                            {
                                $multiply: [
                                    { $toDouble: "$item.price" },
                                    { $toDouble: "$item.weight" }
                                ]
                            }
                        ]
                    }
                },
                totalWeight: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
                            0, // If missing, set totalWeight to 0
                            { $toDouble: "$item.weight" } // Otherwise, use the weight value
                        ]
                    }
                },
                orderedItems: { $first: "$orderedItems" }, // Select the first orderedItems document
                location: { $first: "$orderedItems.cityid" }
            }
        },
        {
            $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
        }
    ];
    
    // const pipeline = [
    //     {
    //         $match: {
    //             $or: [
    //                 { status: "processed" },
    //                 { status: "cash" }
    //             ]
    //         }
    //     },
    //     {
    //         $unwind: "$item"
    //     },
    //     {
    //         $lookup: {
    //             from: "items",
    //             localField: "item.itemid",
    //             foreignField: "_id",
    //             as: "orderedItems"
    //         }
    //     },
    //     {
    //         $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
    //     },
    //     {
    //         $group: {
    //             _id: "$orderedItems.item_name",
    //             totalAmount: {
    //                 $sum: {
    //                     $cond: [
    //                         { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
    //                         0, // If missing, set totalAmount to 0
    //                         {
    //                             $multiply: [
    //                                 { $toDouble: "$item.price" },
    //                                 { $toDouble: "$item.weight" }
    //                             ]
    //                         }
    //                     ]
    //                 }
    //             },
    //             totalWeight: {
    //                 $sum: {
    //                     $cond: [
    //                         { $eq: [{ $type: "$item.itemid" }, "missing"] }, // Check if itemid is missing
    //                         0, // If missing, set totalWeight to 0
    //                         { $toDouble: "$item.weight" } // Otherwise, use the weight value
    //                     ]
    //                 }
    //             },
    //             orderedItems: { $first: "$orderedItems" } // Select the first orderedItems document
    //         }
    //     },
    //     {
    //         $sort: { totalAmount: -1 } // Sort by totalAmount in descending order
    //     }
    // ];

    const result = await Orders.aggregate(pipeline);

    const existingItemNames = result.map((item) => item._id);

    // Find the missing items from the "items" collection
    // const missingItemsPipeline = [
    //     {
    //         $match: {
    //             $and: [
    //                 { item_name: { $nin: existingItemNames } },
    //                 { item_name: { $regex: new RegExp('^' + searchItem, 'i') } } // Match item_name with regex
    //             ]
    //         }
    //     },
    //     {
    //         $group: {
    //             _id: "$item_name",
    //             totalAmount: { $sum: 0 }, // Set totalAmount to 0 for missing items
    //             totalWeight: { $sum: 0 }, // Set totalWeight to 0 for missing items
    //             orderedItems: { $first: "$$ROOT" } // Select the first document for missing items
    //         }
    //     }
    // ];

    console.log(existingItemNames);

    if (existingItemNames.length != 0) {
        const missingItemsPipeline = [
            {
                $match: {
                    $and: [
                        { item_name: { $nin: existingItemNames.map(item => item.item_name) } },
                        { cityid: new ObjectId(result[0].location) },
                        { item_name: { $regex: new RegExp('^' + searchItem, 'i') } },
                    ]
                }
            },
            {
                $group: {
                    _id: {
                        item_name: "$item_name",
                        cityid: result[0].location // Use result[0].location directly, assuming it's a valid cityid
                    },
                    // item_name: "$item_name",
                    totalAmount: { $sum: 0 },
                    totalWeight: { $sum: 0 },
                    orderedItems: { $first: "$$ROOT" },
                    // city: { $first: "$$ROOT.cityid" }
                }
            },
            {
                $sort: {
                    "_id.item_name": 1
                }
            }
        ];
    
        console.log(missingItemsPipeline);
    
        const missingItems = await Items.aggregate(missingItemsPipeline);
    
        const finalResult = result.concat(missingItems);
    
        const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);
    
        const filteredResult = sortedResult.filter(item => {
            const itemName = item._id ? item._id.item_name : null;
            console.log(itemName);
            return itemName && itemName.match(new RegExp('^' + searchItem, 'i'));
        });
          
          console.log('filteredResult', filteredResult);
        // const existingItemNames = result.map((item) => item._id);
        // console.log(result, existingItemNames[0].location);
          
        if (result) {
            res.json({
                isSuccess: true,
                message: "Item found.",
                result: filteredResult,
            });
        } else {
            res.json({
                isSuccess: false,
                result: [],
                message: "Item not found.",
            });
        }
    } else {
        await Location.findOne({ PIN: req.query.ZIP }).then(async (x) => {
            console.log("1234567890x", x);
        const missingItemsPipeline = [
            {
                $match: {
                    $and: [
                        { item_name: { $nin: existingItemNames.map(item => item.item_name) } },
                        { cityid: new ObjectId(x.cityid) },
                        { item_name: { $regex: new RegExp('^' + searchItem, 'i') } },
                    ]
                }
            },
            {
                $group: {
                    _id: {
                        item_name: "$item_name",
                        cityid: x.cityid // Use result[0].location directly, assuming it's a valid cityid
                    },
                    // item_name: "$item_name",
                    totalAmount: { $sum: 0 },
                    totalWeight: { $sum: 0 },
                    orderedItems: { $first: "$$ROOT" },
                    // city: { $first: "$$ROOT.cityid" }
                }
            },
            {
                $sort: {
                    "_id.item_name": 1
                }
            }
        ];
    
        console.log(missingItemsPipeline);
    
        const missingItems = await Items.aggregate(missingItemsPipeline);
    
        const finalResult = result.concat(missingItems);
    
        const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);
    
        const filteredResult = sortedResult.filter(item => {
            const itemName = item._id ? item._id.item_name : null;
            console.log(itemName);
            return itemName && itemName.match(new RegExp('^' + searchItem, 'i'));
        });
          
          console.log('filteredResult', filteredResult);
        // const existingItemNames = result.map((item) => item._id);
        // console.log(result, existingItemNames[0].location);
          
        if (result) {
            res.json({
                isSuccess: true,
                message: "Item found.",
                result: filteredResult,
            });
        } else {
            res.json({
                isSuccess: false,
                result: [],
                message: "Item not found.",
            });
        }
    })
    }
    
}





export const searchItemByZIPByLang = async (req, res) => {

    const item_name = req.query.item_name;
    const searchItem = item_name.trim();

    const pipeline = [
        {
            $match: {
                $and: [
                    {$or: [
                        { status: "processed" },
                        { status: "cash" }
                    ]}
                ]
            }
        },
        {
            $unwind: "$item"
        },
        {
            $lookup: {
                from: "items",
                localField: "item.itemid",
                foreignField: "_id",
                as: "orderedItems"
            }
        },
        {
            $unwind: { path: "$orderedItems", preserveNullAndEmptyArrays: true }
        },
        {
            $lookup: {
                from: "locations",
                localField: "orderedItems.cityid",
                foreignField: "cityid",
                as: "locationItems"
            }
        },
        {
            $unwind: { path: "$locationItems", preserveNullAndEmptyArrays: true }
        },
        {
            $match: {
                "locationItems.PIN": req.query.ZIP
            }
        },
        {
            $addFields: {
                "orderedItems.itemLang": {
                    $filter: {
                        input: "$orderedItems.itemLang",
                        as: "item",
                        cond: { $eq: ["$$item.language", req.params.language] }
                    }
                }
            }
        },
        {
            $group: {
                _id: {
                    item_name: "$orderedItems.item_name",
                },
                totalAmount: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] },
                            0,
                            {
                                $multiply: [
                                    { $toDouble: "$item.price" },
                                    { $toDouble: "$item.weight" }
                                ]
                            }
                        ]
                    }
                },
                totalWeight: {
                    $sum: {
                        $cond: [
                            { $eq: [{ $type: "$item.itemid" }, "missing"] },
                            0,
                            { $toDouble: "$item.weight" }
                        ]
                    }
                },
                orderedItems: {
                    $first: {
                        _id: "$orderedItems._id",
                        item_name: "$orderedItems.item_name",
                        price: "$orderedItems.price",
                        dealer_price: "$orderedItems.dealer_price",
                        description: "$orderedItems.description",
                        isActive: "$orderedItems.isActive",
                        isDelete: "$orderedItems.isDelete",
                        createdAt: "$orderedItems.createdAt",
                        updatedAt: "$orderedItems.updatedAt",
                        __v: "$orderedItems.__v",
                        unit: "$orderedItems.unit",
                        category: "$orderedItems.category",
                        city: "$orderedItems.city",
                        cityid: "$orderedItems.cityid",
                        imageURL: {
                            $cond: [
                                { $isArray: "$orderedItems.imageURL" },
                                { $arrayElemAt: ["$orderedItems.imageURL", 0] },
                                "$orderedItems.imageURL"
                            ]
                        },
                        itemimages: {
                            $cond: [
                                { $isArray: "$orderedItems.imageURL" },
                                "$orderedItems.imageURL",
                                ["$orderedItems.imageURL"]
                            ]
                        },
                        itemLang: "$orderedItems.itemLang",
                        orderedItemsitemLang: [{ $first: "$orderedItems.itemLang" }],
                    }
                },
                orderedItemsitemLang: { $first: "$orderedItems.itemLang" },
                location: { $first: "$orderedItems.cityid" }
            }
        },
        {
            $sort: { totalAmount: -1 }
        }
    ];

    const result = await Orders.aggregate(pipeline);
    const existingItemNames = result.map((item) => item._id);

    // Find the missing items from the "items" collection
    // const missingItemsPipeline = [
    //     {
    //         $match: {
    //             $and: [
    //                 { item_name: { $nin: existingItemNames } },
    //                 { item_name: { $regex: new RegExp('^' + searchItem, 'i') } } // Match item_name with regex
    //             ]
    //         }
    //     },
    //     {
    //         $group: {
    //             _id: "$item_name",
    //             totalAmount: { $sum: 0 }, // Set totalAmount to 0 for missing items
    //             totalWeight: { $sum: 0 }, // Set totalWeight to 0 for missing items
    //             orderedItems: { $first: "$$ROOT" } // Select the first document for missing items
    //         }
    //     }
    // ];

    console.log(existingItemNames);

    if (existingItemNames.length != 0) {
        const missingItemsPipeline = [
            {
                $match: {
                    $and: [
                        { item_name: { $nin: existingItemNames.map(item => item.item_name) } },
                        { cityid: new ObjectId(result[0].location) },
                        { item_name: { $regex: new RegExp('^' + searchItem, 'i') } },
                    ]
                }
            },
            {
                $addFields: {
                    orderedItemsitemLang: [{
                        $arrayElemAt: [
                            {
                                $filter: {
                                    input: "$itemLang",
                                    cond: { $eq: ["$$this.language", req.params.language] }
                                }
                            },
                            0 // Index of the first matched item (in case multiple items match)
                        ]
                    }]
                }
            },
            {
                $group: {
                    _id: {
                        item_name: "$item_name",
                        cityid: result[0].location // Use result[0].location directly, assuming it's a valid cityid
                    },
                    totalAmount: { $sum: 0 },
                    totalWeight: { $sum: 0 },
                    orderedItems: {
                        $first: {
                            _id: "$$ROOT._id",
                            item_name: "$$ROOT.item_name",
                            price: "$$ROOT.price",
                            dealer_price: "$$ROOT.dealer_price",
                            unit: "$$ROOT.unit",
                            city: "$$ROOT.city",
                            cityid: "$$ROOT.cityid",
                            description: "$$ROOT.description",
                            isActive: "$$ROOT.isActive",
                            isDelete: "$$ROOT.isDelete",
                            category: "$$ROOT.category",
                            createdAt: "$$ROOT.createdAt",
                            updatedAt: "$$ROOT.updatedAt",
                            __v: "$$ROOT.__v",
                            imageURL: {
                                $cond: [
                                    { $isArray: "$$ROOT.imageURL" },
                                    { $arrayElemAt: ["$$ROOT.imageURL", 0] },
                                    "$$ROOT.imageURL"
                                ]
                            },
                            itemimages: {
                                $cond: [
                                    { $isArray: "$$ROOT.imageURL" },
                                    "$$ROOT.imageURL",
                                    ["$$ROOT.imageURL"]
                                ]
                            },
                            itemLang: "$$ROOT.itemLang",
                            orderedItemsitemLang: [{
                                $cond: {
                                    if: { $isArray: "$itemLang" },
                                    then: {
                                        $arrayElemAt: [
                                            {
                                                $filter: {
                                                    input: "$itemLang",
                                                    as: "langItem",
                                                    cond: { $eq: ["$$langItem.language", req.params.language]}
                                                }
                                            },
                                            0
                                        ]
                                    },
                                    else: null
                                }
                            }]
                        }
                    }
                }
            },
            {
                $sort: {
                    "_id.item_name": 1
                }
            }
        ];
        
    
        console.log(missingItemsPipeline);
    
        const missingItems = await Items.aggregate(missingItemsPipeline);
    
        const finalResult = result.concat(missingItems);
    
        const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);
    
        const filteredResult = sortedResult.filter(item => {
            const itemName = item._id ? item._id.item_name : null;
            console.log(itemName);
            return itemName && itemName.match(new RegExp('^' + searchItem, 'i'));
        });
          
          console.log('filteredResult', filteredResult);
        // const existingItemNames = result.map((item) => item._id);
        // console.log(result, existingItemNames[0].location);
          
        if (result) {
            res.json({
                isSuccess: true,
                message: "Item found.",
                result: filteredResult,
            });
        } else {
            res.json({
                isSuccess: false,
                result: [],
                message: "Item not found.",
            });
        }
    } else {
        await Location.findOne({ PIN: req.query.ZIP }).then(async (x) => {
            console.log("1234567890x", x);
            const missingItemsPipeline = [
                {
                    $match: {
                        $and: [
                            { item_name: { $nin: existingItemNames.map(item => item.item_name) } },
                            { cityid: new ObjectId(x.cityid) },
                            { item_name: { $regex: new RegExp('^' + searchItem, 'i') } },
                        ]
                    }
                },
                {
                    $group: {
                        _id: {
                            item_name: "$item_name",
                            cityid: x.cityid // Use result[0].location directly, assuming it's a valid cityid
                        },
                        totalAmount: { $sum: 0 },
                        totalWeight: { $sum: 0 },
                        orderedItems: {
                            $first: {
                                _id: "$$ROOT._id",
                                item_name: "$$ROOT.item_name",
                                price: "$$ROOT.price",
                                dealer_price: "$$ROOT.dealer_price",
                                unit: "$$ROOT.unit",
                                description: "$$ROOT.description",
                                isActive: "$$ROOT.isActive",
                                isDelete: "$$ROOT.isDelete",
                                category: "$$ROOT.category",
                                createdAt: "$$ROOT.createdAt",
                                updatedAt: "$$ROOT.updatedAt",
                                __v: "$$ROOT.__v",
                                city: "$$ROOT.city",
                                cityid: "$$ROOT.cityid",
                                imageURL: {
                                    $cond: [
                                        { $isArray: "$$ROOT.imageURL" },
                                        { $arrayElemAt: ["$$ROOT.imageURL", 0] },
                                        "$$ROOT.imageURL"
                                    ]
                                },
                                itemimages: {
                                    $cond: [
                                        { $isArray: "$$ROOT.imageURL" },
                                        "$$ROOT.imageURL",
                                        ["$$ROOT.imageURL"]
                                    ]
                                },
                                itemLang: "$$ROOT.itemLang",
                                orderedItemsitemLang: [{
                                    $cond: {
                                        if: { $isArray: "$itemLang" },
                                        then: {
                                            $arrayElemAt: [
                                                {
                                                    $filter: {
                                                        input: "$itemLang",
                                                        as: "langItem",
                                                        cond: { $eq: ["$$langItem.language", req.params.language]}
                                                    }
                                                },
                                                0
                                            ]
                                        },
                                        else: null
                                    }
                                }]
                            }
                        }
                    }
                },
                {
                    $sort: {
                        "_id.item_name": 1
                    }
                }
            ];
        
            const missingItems = await Items.aggregate(missingItemsPipeline);
        
            const finalResult = result.concat(missingItems);
        
            const sortedResult = finalResult.sort((a, b) => b.totalAmount - a.totalAmount);
        
            const filteredResult = sortedResult.filter(item => {
                const itemName = item._id ? item._id.item_name : null;
                return itemName && itemName.match(new RegExp('^' + searchItem, 'i'));
            });
        
            if (filteredResult.length > 0) {
                res.json({
                    isSuccess: true,
                    message: "Item found.",
                    result: filteredResult,
                });
            } else {
                res.json({
                    isSuccess: false,
                    result: [],
                    message: "Item not found.",
                });
            }
        });
        
        
        

    }
}


// Get all items by cityid for Admin
export const getCitywiseItems = async (req, res) => {
    let filteredItems = [];
    try {
        let cityid = "all";
        
        // Check if cityid is provided in params and replace the initial value
        if (req.params.cityid) {
            cityid = req.params.cityid;
        }

        console.log("cityid", req.params.cityid);
        
        if (!cityid || cityid === "all") {
            return res.json({error: false, msg: "City id not provided", filteredItems});
        }
        
        filteredItems = await Items.find(
            { cityid: cityid === "all" ? "all" : new ObjectId(cityid), isActive: true, isDelete: false },
            { _id: 1, item_name: 1, city: 1, cityid: 1, isActive: 1, isDelete: 1 }
        );

        if (filteredItems.length > 0) {
            res.json({ error: false, filteredItems });
        } else {
            res.json({ error: false, msg: 'Items not found', filteredItems });
        }
    } catch (error) {
        console.error('Error fetching items:', error);
        res.status(500).json({ error: 'Internal server error', filteredItems });
    }
};
// Get particular Item details by ID & language
// export const getItemsByIdByLang = async (req, res) => {
//     const items = await Items.find({ _id: new ObjectId(req.params.id), isDelete: false })
//         .populate('category', 'item_name');      // Populate the category details
//     res.send(items)
// }


export const getItemsByIdByLang = async (req, res) => {
    try {
        const itemId = new ObjectId(req.params.id);
        const language = req.params.language;

        const item = await Items.findOne({ _id: itemId, isDelete: false })
            .populate('category', 'item_name');  // Populate the category details

        if (!item) {
            return res.status(404).send({
                isSuccess: false,
                msg: 'Item not found'
            });
        }

        // Filter itemLang array based on the provided language
        const itemLang = item.itemLang.find(lang => lang.language === language);

        if (!itemLang) {
            return res.status(404).send({
                isSuccess: false,
                msg: 'Item language not found'
            });
        }

        // Prepare the response object
        const responseItem = {
            _id: item._id,
            item_name: item.item_name,
            price: item.price,
            dealer_price: item.dealer_price,
            description: item.description,
            isActive: item.isActive,
            createdAt: item.createdAt,
            updatedAt: item.updatedAt,
            __v: item.__v,
            unit: item.unit,
            category: item.category,
            city: item.city,
            cityid: item.cityid,
            imageURL: item.imageURL[0],
            itemimages: item.imageURL,
            itemLang: item.itemLang,
            orderedItemsitemLang: itemLang
        };

        res.send({
            isSuccess: true,
            msg: 'Item found successfully',
            data: responseItem
        });

    } catch (error) {
        res.status(500).send({ message: 'Error retrieving item details', error });
    }
}


//api for getting scan result by om
export const getItemByZip = async (req, res) => {
    try {
        // Find the location by PIN
        const location = await Location.findOne({ PIN: req.params.PIN });

        if (!location) {
            return res.json({
                msg: "Currently we aren't available in this location.",
                items: []
            });
        }
        let cityId;
        if (location.isActive) {
            cityId = location.cityid;
        } else {
            cityId = "65d5772df1cccd1633087da7"; // Default city ID
        }

        const itemNames = req.body.items;
        const language = req.body.language; 

        if (!Array.isArray(itemNames) || itemNames.length === 0) {
            return res.json({
                msg: "No items provided in the request.",
                items: []
            });
        }
        const foundItems = await Items.find({
            cityid: cityId,
            item_name: { $in: itemNames }  
        });

        const activeItems = foundItems.filter(item => item.isActive);

        // If no active items are found, return a message and an empty array
        if (activeItems.length > 0) {
            const filteredItems = activeItems.map(item => {
                if (language) {
                    const filteredItemLang = item.itemLang.filter(lang => lang.language === language);
                    return {
                        ...item._doc,
                        itemLang: filteredItemLang 
                    };
                }
                return item; 
            });
            return res.json({
                msg: "Items found in this location.",
                items: filteredItems
            });
        } else {
            return res.json({
                msg: "Items don't exist in this location.",
                items: []
            });
        }
    } catch (error) {
        console.error(error);
        return res.status(500).json({ msg: "Server error", error });
    }
};

//get api for getting all collected items of this month
// export const getAllCollectedItems = async (req, res) => {
//     try {
//       const name = req.query.name || '';
//       const itemcategoryname = req.query.itemcategoryname || '';
//       const unit = req.query.unit || '';
//       const asc = req.query.asc === "true";
//       const desc = req.query.desc === "true";
//       const limit = parseInt(req.query.limit) || 10;
//       const offset = parseInt(req.query.offset) || 1;
//       const skip = (offset - 1) * limit;
//       const stdate = req.query.stdate;
//       const enddate = req.query.enddate;
//       const startDate = new Date(stdate.split('-').reverse().join('-')); // "24-09-2024" -> "2024-09-24"
//       const endDate = new Date(enddate.split('-').reverse().join('-'));
  
//       const validStatuses = ["processed", "processing", "Pickup Completed", "reversed"];
//       const matchCondition = {
//         status: { $in: validStatuses },
//         $expr: {
//           $and: [
//             { 
//               $gte: [
//                 { 
//                   $dateFromString: { 
//                     dateString: "$schedule_date", 
//                     format: "%d-%m-%Y" 
//                   } 
//                 }, 
//                 startDate 
//               ] 
//             },
//             { 
//               $lte: [
//                 { 
//                   $dateFromString: { 
//                     dateString: "$schedule_date", 
//                     format: "%d-%m-%Y" 
//                   } 
//                 }, 
//                 endDate 
//               ] 
//             }
//           ]
//         }
//       };
//       const totalCount = await Orders.aggregate([
//         { $match: matchCondition },
//         { $unwind: "$item" },
//         {
//           $lookup: {
//             from: "items",
//             localField: "item.itemid",
//             foreignField: "_id",
//             as: "itemDetails"
//           }
//         },
//         { $unwind: "$itemDetails" },
//         {
//           $lookup: {
//             from: "itemcategories",
//             localField: "itemDetails.category",
//             foreignField: "_id",
//             as: "categoryDetails"
//           }
//         },
//         { $unwind: "$categoryDetails" },
//         {
//           $group: {
//             _id: {
//               item_name: { $trim: { input: "$itemDetails.item_name" } },
//               unit: "$itemDetails.unit",
//               category_name: { $trim: { input: "$categoryDetails.item_name" } }
//             },
//             totalQuantity: { $sum: "$item.weight" }
//           }
//         },
//         {
//           $match: {
//             totalQuantity: { $gt: 0 },
//             "_id.item_name": new RegExp(name, 'i'),
//             "_id.unit": new RegExp(unit, 'i'),
//             "_id.category_name": new RegExp(itemcategoryname, 'i')
//           }
//         }
//       ]).count("totalCount");
  
//       const results = await Orders.aggregate([
//         { $match: matchCondition },
//         { $unwind: "$item" },
//         {
//           $lookup: {
//             from: "items",
//             localField: "item.itemid",
//             foreignField: "_id",
//             as: "itemDetails"
//           }
//         },
//         { $unwind: "$itemDetails" },
//         {
//           $lookup: {
//             from: "itemcategories",
//             localField: "itemDetails.category",
//             foreignField: "_id",
//             as: "categoryDetails"
//           }
//         },
//         { $unwind: "$categoryDetails" },
//         {
//           $group: {
//             _id: {
//               item_name: { $trim: { input: "$itemDetails.item_name" } },
//               unit: "$itemDetails.unit",
//               category_name: { $trim: { input: "$categoryDetails.item_name" } }
//             },
//             totalQuantity: { $sum: "$item.weight" }
//           }
//         },
//         {
//           $match: {
//             totalQuantity: { $gt: 0 },
//             "_id.item_name": new RegExp(name, 'i'),
//             "_id.unit": new RegExp(unit, 'i'),
//             "_id.category_name": new RegExp(itemcategoryname, 'i')
//           }
//         },
//         {
//           $project: {
//             _id: 0,
//             Item_Name: "$_id.item_name",
//             Category_Name: "$_id.category_name",
//             Quantity: {
//               $cond: {
//                 if: { $eq: ["$_id.unit", "Kg"] },
//                 then: { $round: ["$totalQuantity", 3] },
//                 else: "$totalQuantity"
//               }
//             },
//             Unit: "$_id.unit",
//           }
//         },
//         asc && !desc ? { $sort: { Quantity: 1 } } :
//         !asc && desc ? { $sort: { Quantity: -1 } } :
//         { $sort: { Item_Name: 1 } },
//         { $skip: skip },
//         { $limit: limit }
//       ]);

//       const totalOffset = Math.ceil(totalCount[0]?.totalCount / limit);
//       res.status(200).json({
//         results,
//         totalCount: totalCount[0]?.totalCount || 0,
//         totalOffset: totalOffset,
//         currentPage: offset
//       });
  
//     } catch (error) {
//       console.error("Error in getAllCollectedItems:", error);
//       res.status(500).json({ message: "Error fetching collected items" });
//     }
//   };  

export const getAllCollectedItems = async (req, res) => {
    try {
      const name = req.query.name || '';
      const itemcategoryname = req.query.itemcategoryname || '';
      const unit = req.query.unit || '';
      const asc = req.query.asc === 'true';
      const desc = req.query.desc === 'true';
      const limit = parseInt(req.query.limit) || 10;
      const offset = parseInt(req.query.offset) || 1;
      const skip = (offset - 1) * limit;
  
      const stdate = req.query.stdate;
      const enddate = req.query.enddate;
      const startDate = stdate ? new Date(stdate.split('-').reverse().join('-')) : null;
      const endDate = enddate ? new Date(enddate.split('-').reverse().join('-')) : null;
  
      const validStatuses = ['processed', 'processing', 'Pickup Completed', 'reversed'];
      const matchCondition = { status: { $in: validStatuses } };
      if (startDate || endDate) {
        matchCondition.formatDate = {}; // Ensure we are not overwriting the object
    
        if (startDate !== "") {
            matchCondition.formatDate.$gte = new Date(startDate); // Add the $gte condition
        }
    
        if (endDate !== "") {
            matchCondition.formatDate.$lte = new Date(endDate); // Add the $lte condition
        }
    }
    
    console.log("Date fitred data is",matchCondition)
  
      // Aggregation for total count
      const totalCount = await Orders.aggregate([
        {
            $addFields: {
              formatDate: {
                $dateFromString: {
                  dateString: {
                    $concat: [
                      {
                        $arrayElemAt: [
                          {
                            $split: [
                              "$schedule_date",
                              "-"
                            ]
                          },
                          2
                        ]
                      },
                      // Year
                      "-",
                      {
                        $arrayElemAt: [
                          {
                            $split: [
                              "$schedule_date",
                              "-"
                            ]
                          },
                          1
                        ]
                      },
                      // Month
                      "-",
                      {
                        $arrayElemAt: [
                          {
                            $split: [
                              "$schedule_date",
                              "-"
                            ]
                          },
                          0
                        ]
                      }
                    ]
                  }
                }
              }
            }
          },
        { $match: matchCondition },
        { $unwind: '$item' },
        {
          $lookup: {
            from: 'items',
            localField: 'item.itemid',
            foreignField: '_id',
            as: 'itemDetails',
          },
        },
        { $unwind: '$itemDetails' },
        {
          $lookup: {
            from: 'itemcategories',
            localField: 'itemDetails.category',
            foreignField: '_id',
            as: 'categoryDetails',
          },
        },
        { $unwind: '$categoryDetails' },
        {
          $group: {
            _id: {
              item_name: { $trim: { input: '$itemDetails.item_name' } },
              unit: '$itemDetails.unit',
              category_name: { $trim: { input: '$categoryDetails.item_name' } },
            },
            totalQuantity: { $sum: '$item.weight' },
          },
        },
        {
          $match: {
            totalQuantity: { $gt: 0 },
            '_id.item_name': new RegExp(name, 'i'),
            '_id.unit': new RegExp(unit, 'i'),
            '_id.category_name': new RegExp(itemcategoryname, 'i'),
          },
        },
      ]).count('totalCount');
  
      // Aggregation for results with pagination
      const results = await Orders.aggregate([
        {
            $addFields: {
              formatDate: {
                $dateFromString: {
                  dateString: {
                    $concat: [
                      {
                        $arrayElemAt: [
                          {
                            $split: [
                              "$schedule_date",
                              "-"
                            ]
                          },
                          2
                        ]
                      },
                      // Year
                      "-",
                      {
                        $arrayElemAt: [
                          {
                            $split: [
                              "$schedule_date",
                              "-"
                            ]
                          },
                          1
                        ]
                      },
                      // Month
                      "-",
                      {
                        $arrayElemAt: [
                          {
                            $split: [
                              "$schedule_date",
                              "-"
                            ]
                          },
                          0
                        ]
                      }
                    ]
                  }
                }
              }
            }
          },
        { $match: matchCondition },
        { $unwind: '$item' },
        {
          $lookup: {
            from: 'items',
            localField: 'item.itemid',
            foreignField: '_id',
            as: 'itemDetails',
          },
        },
        { $unwind: '$itemDetails' },
        {
          $lookup: {
            from: 'itemcategories',
            localField: 'itemDetails.category',
            foreignField: '_id',
            as: 'categoryDetails',
          },
        },
        { $unwind: '$categoryDetails' },
        {
          $group: {
            _id: {
              item_name: { $trim: { input: '$itemDetails.item_name' } },
              unit: '$itemDetails.unit',
              category_name: { $trim: { input: '$categoryDetails.item_name' } },
            },
            totalQuantity: { $sum: '$item.weight' },
          },
        },
        {
          $match: {
            totalQuantity: { $gt: 0 },
            '_id.item_name': new RegExp(name, 'i'),
            '_id.unit': new RegExp(unit, 'i'),
            '_id.category_name': new RegExp(itemcategoryname, 'i'),
          },
        },
        {
          $project: {
            _id: 0,
            Item_Name: '$_id.item_name',
            Category_Name: '$_id.category_name',
            Quantity: {
              $cond: {
                if: { $eq: ['$_id.unit', 'Kg'] },
                then: { $round: ['$totalQuantity', 3] },
                else: '$totalQuantity',
              },
            },
            Unit: '$_id.unit',
          },
        },
        asc && !desc ? { $sort: { Quantity: 1 } } :
        !asc && desc ? { $sort: { Quantity: -1 } } :
        { $sort: { Item_Name: 1 } },
        { $skip: skip },
        { $limit: limit },
      ]);
  
      const totalOffset = Math.ceil((totalCount[0]?.totalCount || 0) / limit);
  
      res.status(200).json({
        results,
        totalCount: totalCount[0]?.totalCount || 0,
        totalOffset,
        currentPage: offset,
      });
    } catch (error) {
      console.error('Error in getAllCollectedItems:', error);
      res.status(500).json({ message: 'Error fetching collected items' });
    }
  };


  export const NewsearchItemByZIPByLang = async (req, res) => {

    const item_name = req.query.item_name;
    const searchItem = item_name.trim();

   
        await Location.findOne({ PIN: req.query.ZIP }).then(async (x) => {
            console.log("1234567890x", x);
            const missingItemsPipeline = [
                {
                    $match: {
                        $and: [
                            { cityid: new ObjectId(x.cityid) },
                            { item_name: { $regex: new RegExp('^' + searchItem, 'i') } },
                        ]
                    }
                },
                {
                    $group: {
                        _id: {
                            item_name: "$item_name",
                            cityid: x.cityid // Use result[0].location directly, assuming it's a valid cityid
                        },
                        totalAmount: { $sum: 0 },
                        totalWeight: { $sum: 0 },
                        orderedItems: {
                            $first: {
                                _id: "$$ROOT._id",
                                item_name: "$$ROOT.item_name",
                                price: "$$ROOT.price",
                                dealer_price: "$$ROOT.dealer_price",
                                unit: "$$ROOT.unit",
                                description: "$$ROOT.description",
                                isActive: "$$ROOT.isActive",
                                isDelete: "$$ROOT.isDelete",
                                category: "$$ROOT.category",
                                createdAt: "$$ROOT.createdAt",
                                updatedAt: "$$ROOT.updatedAt",
                                __v: "$$ROOT.__v",
                                city: "$$ROOT.city",
                                cityid: "$$ROOT.cityid",
                                imageURL: {
                                    $cond: [
                                        { $isArray: "$$ROOT.imageURL" },
                                        { $arrayElemAt: ["$$ROOT.imageURL", 0] },
                                        "$$ROOT.imageURL"
                                    ]
                                },
                                itemimages: {
                                    $cond: [
                                        { $isArray: "$$ROOT.imageURL" },
                                        "$$ROOT.imageURL",
                                        ["$$ROOT.imageURL"]
                                    ]
                                },
                                itemLang: "$$ROOT.itemLang",
                                orderedItemsitemLang: [{
                                    $cond: {
                                        if: { $isArray: "$itemLang" },
                                        then: {
                                            $arrayElemAt: [
                                                {
                                                    $filter: {
                                                        input: "$itemLang",
                                                        as: "langItem",
                                                        cond: { $eq: ["$$langItem.language", req.params.language]}
                                                    }
                                                },
                                                0
                                            ]
                                        },
                                        else: null
                                    }
                                }]
                            }
                        }
                    }
                },
                {
                    $sort: {
                        "_id.item_name": 1
                    }
                }
            ];
        
            const missingItems = await Items.aggregate(missingItemsPipeline);
        
            
        
            if (missingItems.length > 0) {
                res.json({
                    isSuccess: true,
                    message: "Item found.",
                    result: missingItems,
                });
            } else {
                res.json({
                    isSuccess: false,
                    result: [],
                    message: "Item not found.",
                });
            }
        });
        
        
        

}