import mongoose from "mongoose";
import Location from "../model/location.model.js";
import Address from "../model/address.model.js";
import CityPercent from "../model/city_percent.model.js";
import axios from "axios";
import City from "../model/city_percent.model.js";

const ObjectId = mongoose.Types.ObjectId;


// Add New Location
export const addLocation = async (req, res) => {
    const cityid = new mongoose.Types.ObjectId(req.body.cityid);
    const { location, city, state, PIN } = req.body;

    // Check if a location with the same name already exists
    const existingLocation = await Location.findOne({ location: { $regex: new RegExp(`^${location}$`, 'i') }, PIN });
    const existingLocationCount = await Location.findOne({ PIN, count: { $gte: 1 } });

    console.log('existingLocationCount', existingLocationCount);

    if (existingLocation) {
        return res.send({ "uploaded": false, "message": "Location already exist." });
    } else if (existingLocationCount) {
    // const existingLocation = await Location.findOne({ PIN, isActive: false, isDelete: false, cityId: { $exists: false } });

        const x = await Location.findOneAndUpdate({ _id: existingLocationCount._id, cityId: { $exists: false } }, { location: location,
            city: city,
            state: state,
            PIN: PIN,
            cityid: cityid, isActive: true, isDelete: false },
            { new: true })

        // if (location) {
            const address = await Address.updateMany({ PIN: PIN}, { isActive: true, isDelete: false }, { new: true });
            console.log(address);
        // }
        if (x) {
            return res.send({ "uploaded": false, "message": "Update location.", "location": x });
        } else {
            return res.send({ "Error": "Error" });
        }
    } else {
        const newLocation = await Location.create({
            location: location,
            city: city,
            state: state,
            PIN: PIN,
            cityid: cityid
        });
      
        // Input validation for city & state
        try {
            validateInput(req.body.city, "city");
            validateInput(req.body.state, "state");
        } catch (error) {
            return res.send({
                error: true,
                message: error.message
            });
        }
    
        // Check if PIN is a valid 6-digit number
        if (!isValidPIN(req.body.PIN)) {
            return res.send({
                error: true,
                message: "Invalid PIN. PIN must be a 6-digit number."
            });
        }
      
        if (newLocation) {
            res.send(
                {
                    error: false,
                    message: "saved successfully"
                }
            );
        }
    }
  
};


// Get all Locations
export const getAllLocation = async (req, res) => {
    const location = await Location.find({ isDelete:false })
    .sort({ createdAt: -1 }).exec();
    res.send(location)
}


// Get particular Location by ID
export const getLocationById = async (req, res) => {
    const location = await Location.find({ _id: new ObjectId(req.params.id), isDelete:false });
    res.send(location)
}


// Activate a location
export const activateLocationById = async (req, res) => {
    try {
      const location = await Location.findByIdAndUpdate({ _id: new ObjectId(req.params.id)}, { isActive: true }, { new: true });
      console.log(location);
      if (location) {
        const address = await Address.updateMany({ PIN: location.PIN}, { isActive: true, isDelete: false }, { new: true });
        console.log(address);
      }
      res.send(location);
    } catch (error) {
      console.error(error);
      res.send({ error: 'Internal Server Error' });
    }
  };
  
  // Deactivate a location
  export const deactivateLocationById = async (req, res) => {
    try {
      const location = await Location.findByIdAndUpdate({ _id: new ObjectId(req.params.id)}, { isActive: false }, { new: true });
      res.send(location);
    } catch (error) {
      console.error(error);
      res.send({ error: 'Internal Server Error' });
    }
  };



// Update Location
export const updateLocation = async (req, res) => {
    const updateLocation = await Location.updateOne({ _id: new ObjectId(req.body._id) }, { $set: { "location": req.body.location, "city": req.body.city, "state": req.body.state } });
  
    // Input validation for city & state
    try {
        validateInput(req.body.city, "city");
        validateInput(req.body.state, "state");
    } catch (error) {
        return res.send({
            error: true,
            message: error.message
        });
    }

    // Check if PIN is a valid 6-digit number
    // if (!isValidPIN(req.body.PIN)) {
    //     return res.send({
    //         error: true,
    //         message: "Invalid PIN. PIN must be a 6-digit number."
    //     });
    // }

    if (updateLocation) {
        console.log(updateLocation)
        res.send("update success")
    } else {
        res.send("update failed")
    }
};



// Delete Location by ID
export const deleteLocation = async (req, res) => {
    const location = await Location.updateOne({ _id: new ObjectId(req.params.id) }, { $set: { "isActive": false, "isDelete": true } });
    if (location) {
        res.send("deleted successfully")
    } else {
        res.send("deletion failed")
    }
}


// Validation of PIN code
export const validatePIN = async (req, res) => {
    const api = `https://api.postalpincode.in/pincode/${req.params.PIN}`
    axios.get(api).then(resp => {
        // console.log(resp.data);
        if (resp.data[0].Status != "Error") {
            res.send("valid pin")
        } else {
            res.send("invalid pin")
        }
    });
}

// Compare Location
export const compareLocation = async (req, res) => {
    const location = await Location.findOne({$and: [{ PIN: req.params.PIN },{isDelete: false}]});

    if (location) {
        return res.send({
            error: false,
            message: "service is available on the given pincode"
        });
    } else {
        res.send(
            {
                error: true,
                message: "service is not available on the given pincode"
            }
        );
    }
}

// Add Locations by importing Excelsheet
export const addLocationBulkExcel = async (req, res) => {
    try {
        const locations = req.body;

        // Keep track of errors encountered while processing locations
        const errors = [];

        // Iterate through each location object in the array
        for (const locationData of locations) {
            const { location, city, state, PIN, cityid } = locationData;

            // Check if a location with the same name already exists
            const existingLocation = await Location.findOne({ location: { $regex: new RegExp(`^${location}$`, 'i') }, PIN });

            if (existingLocation) {
                errors.push({ "uploaded": false, "message": "The location already exists." });
                continue;
            }

            // Create a new location entry
            const newLocation = await Location.create({
                location: location,
                city: city,
                state: state,
                PIN: PIN,
                cityid: cityid
            });

            // Input validation for city & state
            validateInput(city, "city");
            validateInput(state, "state");

            // Check if PIN is a valid 6-digit number
            if (!isValidPIN(PIN)) {
                errors.push({ "error": true, "message": "Invalid PIN. PIN must be a 6-digit number." });
                continue;
            }

            if (newLocation) {
                console.log(`Location ${location} created successfully`);
            }
        }

        // After processing all locations, send the response
        if (errors.length > 0) {
            res.status(400).json(errors);
        } else {
            res.json({ "message": "Locations saved successfully" });
        }
    } catch (error) {
        console.error(error);
        res.status(500).json({ "error": true, "message": "An error occurred while saving locations" });
    }
};

 
// FUNCTIONS

// Input validation for city & state
function validateInput(input, fieldName) {
    if (!/^[A-Za-z\s]+$/.test(input)) {
        throw new Error(`Invalid characters in ${fieldName}. Only alphabets and spaces allowed.`);
    }
}

// Check if PIN is a valid 6-digit number
function isValidPIN(PIN) {
    const pinRegex = /^\d{6}$/;
    return pinRegex.test(PIN);
}





export const findCitybyPIN = async (req, res) => {
    await Location.findOne({ PIN: req.params.PIN, isActive: true }).then((location) => {
        console.log(location);
        if (location) {
            return res.send({
                isSuccess: true,
                city: location.city,
                cityid: location.cityid,
            })
        } else {
            return res.send({
                isSuccess: false,
                city: '',
                cityid: '',
            })
        }
    })
}




