import mongoose from "mongoose";
import NGO from '../model/ngo.model.js';
import multer from "multer";
import axios from 'axios';
import dotenv from 'dotenv';
import Paymentdetails from "../model/paymentdetails.model.js";
const ObjectId = mongoose.Types.ObjectId;

dotenv.config();
const apiKey = process.env.APIKEY;
// const apiKey = 'rzp_live_c8RouP3eKq6D1C';
const apiSecret = process.env.APISECRET;
// const apiSecret = '63WBO3yFwX6gHymOiypMjXm7';

// const Storage = multer.diskStorage({
//     destination: 'uploads',
//     filename: (req, res, cb) => {
//         cb(null, Date.now() + "_" + (res.originalname).trim());
//     }
// });

const Storage = multer.diskStorage({
    destination: 'uploads',
    filename: (req, file, cb) => {
        cb(null, Date.now() + "_" + file.originalname.trim());
    }
});

const upload = multer({
    storage: Storage,
}).fields([
    { name: 'pan_fileURL', maxCount: 1 },
    { name: 'logoURL', maxCount: 1 }
]);

// Adding NGO details
export const createNGO = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
            return res.send({
                error: true,
                message: "File upload failed"
            });
        }

        const { name, email, PAN, mobile, account_type, ngo_bank_account, purpose, bankName, bankBranch, bankAddress } = req.body;

        try {
            // Check if NGO data already exists
            const existingNGO = await NGO.findOne({ name, email, PAN, isDelete: false });

            if (existingNGO) {
                return res.send({
                    isSuccess: false,
                    message: "The NGO already exists."
                });
            }

            let ngo_data = await NGO.create({
                name,
                email,
                PAN,
                mobile,
                account_type,
                ngo_bank_account,
                purpose,
                bankName,
                bankBranch,
                bankAddress
            });

            if (req.file) {
                ngo_data.pan_fileURL = 'uploads/' + req.file.filename;
            }

            if (ngo_data) {
                const auth = {
                    username: apiKey,
                    password: apiSecret,
                }

                const requestData = {
                    name: req.body.name,
                    email: req.body.email,
                    contact: req.body.mobile,
                };

                console.log(requestData);

                if (requestData) {
                    try {
                        const response = await axios.post(`https://api.razorpay.com/v1/contacts`, requestData, {
                            headers: {
                                'Content-Type': 'application/json',
                            },
                            auth: auth,
                        });

                        console.log('API Response:', response.data);

                        await NGO.findOneAndUpdate({ _id: ngo_data._id }, {
                            $set: {
                                ci: response.data.id
                            }
                        });

                        let fa_id = null;

                        // If account_type is "ngo_bank_account", generate FA
                        if (account_type == "bank_account") {
                            const faRequestData = {
                                contact_id: response.data.id,
                                account_type: "bank_account",
                                bank_account: ngo_bank_account,
                            };
                            console.log("faRequestData", faRequestData);
                            try {
                                const faResponse = await axios.post(`https://api.razorpay.com/v1/fund_accounts`, faRequestData, {
                                    headers: {
                                        'Content-Type': 'application/json',
                                    },
                                    auth: auth,
                                });

                                console.log('FA API Response:', faResponse.data);

                                fa_id = faResponse.data.id;

                                await Paymentdetails.create({
                                    customer_id: ngo_data._id,
                                    ci: response.data.id,
                                    fa: fa_id,
                                    account_type: account_type,
                                    bank_account: ngo_bank_account,
                                    mode: "Donation",
                                });

                                await NGO.findOneAndUpdate({ _id: ngo_data._id }, {
                                    $set: {
                                        fa: fa_id,
                                        mode: "Donation",
                                    }
                                });
                            } catch (error) {
                                console.error(error);
                                return res.status(500).send({
                                    error: true,
                                    message: 'Failed to generate FA for the NGO',
                                    details: error.response.data, // Include details from Razorpay response for debugging
                                });
                            }
                        }

                        return res.send({
                            isSuccess: true,
                            message: "NGO data saved successfully.",
                            data: ngo_data,
                            ci: response.data.id,
                            fa: fa_id,
                        });
                    } catch (error) {
                        console.error(error);
                        return res.status(500).send({
                            error: true,
                            message: 'Failed to generate Ci for the NGO',
                            details: error.response.data,
                        });
                    }
                } else {
                    return res.status(500).send({
                        error: true,
                        message: 'Invalid request data',
                    });
                }
            } else {
                return res.status(500).send({
                    error: true,
                    message: 'An error occurred while saving the NGO data',
                });
            }
        } catch (error) {
            console.error(error);
            return res.status(500).send({
                error: true,
                message: 'An error occurred while completing the profile',
            });
        }
    });
}


// Upload Logo or PAN in NGO
export const uploadLogoOrPAN = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
            return res.status(500).send({
                error: true,
                message: "File upload failed"
            });
        }
        
        try {
            const ngo = await NGO.findOne({ _id: req.params.id, isDelete: false });

            if (!ngo) {
                return res.status(404).json({ message: "NGO not found" });
            }

            const updateData = {};

            if (req.files) {
                if (req.files.pan_fileURL) {
                    updateData.pan_fileURL = 'uploads/' + req.files.pan_fileURL[0].filename;
                }
                if (req.files.logoURL) {
                    updateData.logoURL = 'uploads/' + req.files.logoURL[0].filename;
                }
            }

            const updatedNGO = await NGO.findByIdAndUpdate(req.params.id, { $set: updateData }, { new: true });

            if (updatedNGO) {
                return res.status(200).json({ message: "NGO data updated successfully", ngo: updatedNGO });
            } else {
                return res.status(500).json({ message: "Update failed" });
            }
        } catch (error) {
            console.error(error);
            return res.status(500).send({
                error: true,
                message: "Internal server error"
            });
        }
    });
};

// Get all NGO details
export const getAllNGO = async (req, res) => {
    try {
        const ngo = await NGO.find({ isDelete: false })
            .collation({ locale: 'en', strength: 2 })
            .sort({ name: 1 }).exec();

        if (ngo) {
            res.send({
                isSuccess: true,
                data: ngo
            });
        } else {
            res.status(500).json({ error: 'NGO not found' });
        }
    } catch (error) {
        console.error(error);
        return res.send({
            isSuccess: false,
            message: "Something went wrong."
        });
    }
}


// Get NGO details by ID
export const getNGOById = async (req, res) => {
    try {
        const ngo = await NGO.find({ _id: (req.params.id), isDelete: false });

        if (ngo.length > 0) {
            res.send({
                isSuccess: true,
                "NGO Details": ngo
            });
        } else {
            res.send({ msg: 'The NGO details not found' });
        }
    } catch (error) {
        console.error(error);
        return res.send({
            isSuccess: false,
            message: "Something went wrong."
        });
    }
}
