import mongoose from "mongoose";
import NotifyCondition from "../model/notifyCondition.model.js";

const ObjectId = mongoose.Types.ObjectId;


// Add new Condition for notification
export const addCondition = async (req, res) => {
    const condition = req.body.condition_name;
console.log(condition);
    try {
        // Check if an item with the same name already exists
        const existingCondition = await NotifyCondition.findOne({ condition_name: { $regex: new RegExp(`^${condition}$`, 'i') } });
console.log(existingCondition);
        if (existingCondition) {
            return res.status(400).json({ "uploaded": false, "message": "Condition already exists." });
        }

        const newCondition = new NotifyCondition({
            condition_name: condition
        });
console.log("newCondition", newCondition);
        await newCondition.save();
        res.status(201).json({ "uploaded": true, data: newCondition });
    } catch (error) {
        console.error(error);
        res.status(500).json({ "uploaded": false, "error": "Internal server error" });
    }
};

// Get all Conditions for notfication
export const getAllConditions = async (req, res) => {
    const allNotifyCondition = await NotifyCondition.find({ isDelete: false })
        .sort({ createdAt: -1 }).exec();
    res.send({
        error: false,
        allNotifyCondition
    })
}


// Get particular condition by ID
export const getConditionById = async (req, res) => {
    const condition = await NotifyCondition.findOne({ _id: new ObjectId(req.params.id), isDelete: false });
    if (condition && condition.length != 0) {
        res.send({
            error: false,
            condition
        })
    } else {
        res.send({
            error: false,
            message: "condition not found !!"
        })
    }
}


// Edit custom condition
export const editCondition = async (req, res) => {
    const { _id, condition_name } = req.body;

    // Check if the Condition with the same name already exists
    let existingCondition = await NotifyCondition.findOne({ condition_name: condition_name });

    if (existingCondition) {
        if (existingCondition.isDelete) {
            // If existing Condition is marked as deleted, update fields
            existingCondition.isDelete = false;
            existingCondition.isActive = true;
            await existingCondition.save();

            return res.send({ "uploaded": true, "message": "Update successful." });
        } else {
            return res.send({ "uploaded": false, "message": "The Condition already exists." });
        }
    } else {
        const updateCondition = await NotifyCondition.updateOne(
            { _id: new ObjectId(_id) },
            {
                $set: {
                    "condition_name": condition_name,
                }
            }
        );

        if (updateCondition) {
            return res.status(200).json({ status: 200, message: "Condition updated successfully." });
        } else {
            return res.status(500).json({ status: 500, message: "Update failed or no changes made." });
        }
    }

};