import Paymentdetails from "../model/paymentdetails.model.js";
import axios from "axios";
import dotenv from 'dotenv';
import Customers from "../model/customer.model.js";
import mongoose from "mongoose";
const ObjectId = mongoose.Types.ObjectId;


dotenv.config();
const apiKey = process.env.APIKEY;
// const apiKey = 'rzp_live_c8RouP3eKq6D1C';
const apiSecret = process.env.APISECRET;
// const apiSecret = '63WBO3yFwX6gHymOiypMjXm7';


// Create FA for Razorpay
export const creatFa = async (req, res) => {
    try {
        const auth = {
            username: apiKey,
            password: apiSecret,
        };
        
        let requestData = {};
        let payment = {};
        let ciResponse;  
        let customer = await Customers.findOne({ _id: new mongoose.Types.ObjectId(req.body.customer_id), isDelete: false });
        if (!customer) {
            return res.status(404).json({
                isSuccess: false,
                message: "Customer not found."
            });
        }
        if (!customer.ci) {
            console.log("CI not found, validating data and generating new CI...");
            const { name, email, mobile } = customer;
            if (!name || name.trim().length < 3) {
                return res.status(400).json({
                    isSuccess: false,
                    message: "Customer name must be at least 3 characters long and should not have trailing spaces.",
                });
            }
            const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
            if (!email || !emailRegex.test(email)) {
                return res.status(400).json({
                    isSuccess: false,
                    message: "Invalid email address found for the customer.",
                });
            }
            const mobileRegex = /^[0-9]{10}$/;
            if (!mobile || !mobileRegex.test(mobile)) {
                return res.status(400).json({
                    isSuccess: false,
                    message: "Invalid mobile number found for the customer. It must be 10 digits.",
                });
            }
            const requestDataForCi = {
                name: name.trim(),
                email: email.trim(),
                contact: mobile,
            };  
            try{
                ciResponse = await axios.post(`https://api.razorpay.com/v1/contacts`, requestDataForCi, {
                    headers: {
                        'Content-Type': 'application/json',
                    },
                    auth: auth,
                });
                console.log('CI Generation Response:', ciResponse.data);
                customer = await Customers.findOneAndUpdate(
                    { _id:new mongoose.Types.ObjectId(req.body.customer_id) },
                    { $set: { ci: ciResponse.data.id } },
                    { new: true }  // Return updated document
                );

            }
            catch(ciError){
                return res.status(500).json({
                    isSuccess: false,
                    message: "Failed to generate CI.",
                    error: ciError.response?.data?.error?.description || ciError.message,
                });
            }


        }

        if (req.body.bank_account && req.body.account_type === "bank_account") {
            requestData = {
                contact_id: customer.ci,
                account_type: req.body.account_type,
                bank_account: req.body.bank_account,
            };
        } else if (req.body.vpa && req.body.account_type === "vpa") {
            const existingPayment = await Paymentdetails.findOne({ customer_id: req.body.customer_id, vpa: req.body.vpa });

            if (existingPayment) {
                return res.json({
                    isSuccess: true,
                    msg: "UPI already exists.",
                });
            }

            requestData = {
                contact_id: customer.ci,
                account_type: req.body.account_type,
                vpa: req.body.vpa,
            };
        }

        if (Object.keys(requestData).length !== 0) {
            const response = await axios.post(`https://api.razorpay.com/v1/fund_accounts`, requestData, {
                headers: {
                    'Content-Type': 'application/json',
                },
                auth: auth,
            });

            if (response) {
                if (req.body.bank_account && req.body.account_type === "bank_account") {
                    payment = await Paymentdetails.create({
                        customer_id: req.body.customer_id,
                        ci: customer.ci,
                        fa: response.data.id,
                        account_type: req.body.account_type,
                        bank_account: req.body.bank_account,
                        mode: "IMPS",
                        default: 1,
                    });
                } else if (req.body.vpa && req.body.account_type === "vpa") {
                    const countPayment = await Paymentdetails.countDocuments({ customer_id: req.body.customer_id, mode: "UPI" });
                    console.log("countPayment", countPayment);
                    if (countPayment == 0) {
                        payment = await Paymentdetails.create({
                            customer_id: req.body.customer_id,
                            ci: customer.ci,
                            fa: response.data.id,
                            account_type: req.body.account_type,
                            vpa: req.body.vpa,
                            mode: "UPI",
                            default: 1,
                        });
                    } else {
                        payment = await Paymentdetails.create({
                            customer_id: req.body.customer_id,
                            ci: customer.ci,
                            fa: response.data.id,
                            account_type: req.body.account_type,
                            vpa: req.body.vpa,
                            mode: "UPI",
                            default: 0,
                        });
                    }
                    
                }

                if (payment) {
                    return res.json({
                        isSuccess: true,
                        message: `Payment ${req.body.account_type} details added.`,
                        fa: payment.fa,
                    });
                } else {
                    return res.json({
                        isSuccess: false,
                        message: "Failed to add payment details.",
                    });
                }
            }
        } else {
            return res.json({
                isSuccess: false,
                message: "No valid data provided.",
            });
        }
    } catch (error) {
        console.error(error);
        return res.json({
            isSuccess: false,
            message: 'An error occurred during the API request.',
            error: error.response.data.error.description
        });
    }
}





// Give request to default UPI
export const defReq = async (req, res) => {
    await Paymentdetails.findOneAndUpdate({ _id: req.params.id, mode: "UPI", reqdefaultcount: 0 }, { $set: { reqdefault: 1 } }).then( async (x) => {
        if (x) {
            await Paymentdetails.findOneAndUpdate({ _id: req.params.id, mode: "UPI" }, { $set: { reqdefaultcount: 1 } }).then ((y) => {
                return res.json({
                    isSuccess: true,
                    msg: "Request sent to Admin."
                })
            })
            
        } else {
            return res.json({
                isSuccess: false,
                msg: "Can't send Request to Admin."
            })
        }
    })
}




// Request get by Admin
export const getAllReq = async (req, res) => {
    const pipeline = [
        {
            $match: {
                reqdefault: 1 
            }
        },
        {
            $lookup: {
                from: "customers",
                localField: "customer_id",
                foreignField: "_id",
                as: "customerInfo"
            }
        },
        {
            $unwind: "$customerInfo" // Unwind to work with customer information
        },
        {
            $group: {
                _id: "$_id",
                customer_id: { $first: "$customer_id" },
                vpa: { $first: "$vpa.address" },
                ci: { $first: "$ci" },
                fa: { $first: "$fa" },
                account_type: { $first: "$account_type" },
                mode: { $first: "$mode" },
                default: { $first: "$default" },
                reqdefault: { $first: "$reqdefault" },
                reqdefaultcount: { $first: "$reqdefaultcount" },
                customers: { $first: "$customerInfo.name" },
                email: { $first: "$customerInfo.email" },
                mobile: { $first: "$customerInfo.mobile" },
            }
        },
        {
            $sort: {
                createdAt: -1 // 1 for ascending order, -1 for descending order
            }
        }
    ];
    

    const result = await Paymentdetails.aggregate(pipeline);
    console.log(result);
    // await Paymentdetails.find({ reqdefault: 1 }).then ((x) => {
        if (result) {
            return res.json({
                isSuccess: true,
                msg: "Pending Requests.",
                data: result,
            })
        } else {
            return res.json({
                isSuccess: true,
                msg: "No Requests.",
                data: [],
            })
        }
    // })
}




// Request get individual view by ID (** ADMIN)
export const getAllReqId = async (req, res) => {
    const pipeline = [
        {
            $match: {
                reqdefault: 1,
                _id: new mongoose.Types.ObjectId(req.params.id),
            }
        },
        {
            $lookup: {
                from: "customers",
                localField: "customer_id",
                foreignField: "_id",
                as: "customerInfo"
            }
        },
        {
            $unwind: "$customerInfo" // Unwind to work with customer information
        },
        {
            $group: {
                _id: "$_id",
                customer_id: { $first: "$customer_id" },
                vpa: { $first: "$vpa.address" },
                ci: { $first: "$ci" },
                fa: { $first: "$fa" },
                account_type: { $first: "$account_type" },
                mode: { $first: "$mode" },
                default: { $first: "$default" },
                reqdefault: { $first: "$reqdefault" },
                reqdefaultcount: { $first: "$reqdefaultcount" },
                customers: { $first: "$customerInfo.name" },
                email: { $first: "$customerInfo.email" },
                mobile: { $first: "$customerInfo.mobile" },
            }
        },
        {
            $sort: {
                createdAt: -1 // 1 for ascending order, -1 for descending order
            }
        }
    ];
    

    const result = await Paymentdetails.aggregate(pipeline);
    console.log(result);
        if (result) {
            return res.json({
                isSuccess: true,
                msg: "Individual Pending Requests.",
                data: result,
            })
        } else {
            return res.json({
                isSuccess: true,
                msg: "No Requests.",
                data: [],
            })
        }
}




// Approve Request by Admin
export const aprReq = async (req, res) => {
    await Paymentdetails.updateMany({ customer_id: req.body.customer_id, mode: "UPI" }, { $set: { reqdefault: 0, default: 0 } }).then(async (x) => {
        console.log(x);
        if (x) {
            await Paymentdetails.findOneAndUpdate({ _id: req.body.id, mode: "UPI" }, { $set: { reqdefault: 0, default: 1 } }).then( async (y) => {
                if (y) {
                    return res.json({
                        isSuccess: true,
                        msg: "Make as default.",
                        data: x,
                    })
                } else {
                    return res.json({
                        isSuccess: true,
                        msg: "Something went wrong.",
                        data: [],
                    })
                }
            })
        }
    })
}





// Multiple UPI add by admin
export const updateUpi = async (req, res) => {
    try {
        const auth = {
            username: apiKey,
            password: apiSecret,
        }
        console.log(req.body);
        let requestData = {}
        let payment = {}
        await Customers.findOne({ mobile: req.body.mobile, isDelete: false }).then( async (x) => {
            if (req.body.vpa && req.body.account_type == "vpa") {
                requestData = {
                    // customer_id: req.body.customer_id,
                    contact_id: x.ci,
                    account_type: req.body.account_type,
                    vpa: req.body.vpa,
                }
            }
        
            console.log('requestData', requestData);
            if (requestData) {
                const response = await axios.post(`https://api.razorpay.com/v1/fund_accounts`, requestData, {
                    headers: {
                        'Content-Type': 'application/json',
                    },
                    auth: auth,
                })
                console.log('response', response);
                if (response) {
                    console.log('API Response:', response.data);
                    if (response) {
                        if (req.body.vpa && req.body.account_type == "vpa") {
                            payment = await Paymentdetails.create({
                                customer_id: x.customer_id,
                                ci: x.ci,
                                fa: response.data.id,
                                account_type: req.body.account_type,
                                vpa: req.body.vpa,
                                mode: "UPI",
                            });
                        }
                        console.log(payment);
                        if (payment) {
                            return res.send({
                                isSuccess: true,
                                message: `Payment ${req.body.account_type} details added.`,
                            });
                        } else {
                            return res.send({
                                data: response.data
                            })
                        }
                    } else {
                        return res.send({
                            data: response.data
                        })
                    }
        
                }
            } else {
                return res.send({
                    data: response.data
                })
            }
        })
       
    } catch (error) {
    
        return res.send({
            isSuccess: false,
            message: 'An error occurred during the API request.',
            error: error.response.data.error.description
        });
    }
}




// Get UPI or Bank details by customer_id
export const payDetails = async (req, res) => {
    await Paymentdetails.find({ customer_id: req.params.custid, mode: req.query.mode, default: 1 }).then( async (paymentDetails) => {
        console.log(paymentDetails.length == 0);
        if (paymentDetails != 0) {
            await Paymentdetails.find({ customer_id: req.params.custid, mode: req.query.mode }).then((paymentMultiDetails) => {
                const paymentDetailsObj = paymentDetails.reduce((acc, curr) => {
                    acc = curr;
                    return acc;
                }, {});
                return res.send({
                    isSuccess: true,
                    data: paymentDetailsObj,
                    multidata: paymentMultiDetails,
                });
            })
            
        } else {
            return res.send({
                isSuccess: false,
                data: 'Something went wrong.',
                multidata: [],
            });
        }
    })
}


