import mongoose from "mongoose";
import Users from "../model/users.model.js";
import CityPercent from "../model/city_percent.model.js";
import multer from 'multer';
import bcrypt from 'bcrypt';
import Verifyotp from "../model/verifyotp.model.js";
import axios from "axios";
import fs from 'fs';
import path from 'path';
import SibApiV3Sdk from 'sib-api-v3-sdk';
import dotenv from 'dotenv';
import customers from "../model/customer.model.js";
dotenv.config();

const ObjectId = mongoose.Types.ObjectId;
import Routeslot from "../model/routeslot.model.js";
import jwt from 'jsonwebtoken';

const Storage = multer.diskStorage({
    destination: 'uploads',
    filename: (req, file, cb) => {
        cb(null, Date.now() + "_" + (file.originalname).trim());
    }
});
// const upload = multer({
//     storage: Storage,
// }).single('image');


// Configure multer storage
// const storage = multer.diskStorage({
//     destination: function (req, file, cb) {
//         cb(null, 'uploads/'); // Adjust the destination path as needed
//     },
//     filename: function (req, file, cb) {
//         cb(null, Date.now() + '-' + file.originalname);
//     }
// });

// Initialize upload with storage settings
const upload = multer({ storage: Storage }).fields([
    { name: 'image', maxCount: 1 },
    { name: 'aadhar_imgURL', maxCount: 1 },
    { name: 'vehicle_registrationURL', maxCount: 1 },
    { name: 'vehicle_insuranceURL', maxCount: 1 },
    { name: 'dl_imgURL', maxCount: 1 }
]);

// Internal Login
// export const usersLogin = async (req, res) => {
//     const { email, password } = req.body;
//     const users = await Users.findOne({ email });

//     if (users && (await users.comparePassword(password))) {
//         return res.json({
//             _id: users._id,
//             name: users.name,
//             email: users.email,
//             mobile: users.mobile,
//             usertype: users.usertype,
//             imageURL: users.imageURL
//         });
//     } else {
//         res.send(
//             {
//                 error: true,
//                 message: "User doesn't exist"
//             }
//         );
//     }
// }

export const usersLogin = async (req, res) => {
    try {
        const { email, password } = req.body;
        const user = await Users.findOne({ email });

        if (!user) {
            return res.send({
                error: true,
                message: "Invalid email or password"
            });
        }

        // Compare the password
        const isMatch = await bcrypt.compare(password, user.password);
        if (!isMatch) {
            return res.send({
                error: true,
                message: "Invalid email or password"
            });
        }
        const token = jwt.sign(
            {
                _id: user._id,
                name: user.name,
                email: user.email,
                usertype: user.usertype,
                mobile: user.mobile,
            },
            "AjatusAgent",
        );

        return res.send({
            _id: user._id,
            name: user.name,
            email: user.email,
            mobile: user.mobile,
            cityid: user.cityid,
            usertype: user.usertype,
            imageURL: user.imageURL,
            userToken: token
        });

    } catch (error) {
        console.error(error);
        return res.send({
            error: true,
            message: "Server Error"
        });
    }
};


// Create Users
// export const createUsers = async (req, res) => {
//     upload(req, res, async (err) => {
//         if (err) {
//             console.log(err)
//         } else {
//             try {
//                 const { name, email, password, usertype, mobile, cityid } = req.body;

//                 // Check if the user with the same mobile already exists
//                 const existingMobile = await Users.findOne({ mobile: req.body.mobile });
//                 if (existingMobile) {
//                     return res.send({ "uploaded": false, "message": "This mobile number already exists." });
//                 }

//                 // Check if the email already exists in the database
//                 const existingEmail = await Users.findOne({ email: req.body.email });
//                 if (existingEmail) {
//                     return res.send({ "uploaded": false, message: 'Oops! This email address is already in use with another account.' });
//                 }

//                 const users = Users.create({
//                     name: name,
//                     email: email,
//                     password: password,
//                     usertype: usertype,
//                     mobile: mobile,
//                     cityid: cityid,
//                     image: {
//                         contentType: 'image',
//                         data: req.files.image[0].filename
//                     },
//                     imageURL: 'uploads/' + req.files.image[0].filename
//                 });
//                 if (users) {
//                     res.send(
//                         {
//                             error: false,
//                             message: "saved successfully"
//                         }
//                     );
//                 }
//             } catch (error) {
//                 console.error(error);
//                 return res.status(500).json({ "uploaded": false, message: "Server Error" });
//             }
//         }
//     });
// };

export const createUsers = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
        } else {
            try {
                const { name, email, password, usertype, mobile, cityid } = req.body;

                // Check if the user with the same mobile already exists
                const existingMobile = await Users.findOne({ mobile: req.body.mobile });
                if (existingMobile) {
                    return res.send({ "uploaded": false, "message": "This mobile number already exists." });
                }

                // Check if the email already exists in the database
                const existingEmail = await Users.findOne({ email: req.body.email });
                if (existingEmail) {
                    return res.send({ "uploaded": false, message: 'Oops! This email address is already in use with another account.' });
                }

                // Hash the password before saving the user
                const hashedPassword = await bcrypt.hash(password, 8);

                const newUser = new Users({
                    name: name,
                    email: email,
                    password: hashedPassword,
                    usertype: usertype,
                    mobile: mobile,
                    cityid: cityid,
                    image: {
                        contentType: 'image',
                        data: req.files.image[0].filename
                    },
                    imageURL: 'uploads/' + req.files.image[0].filename
                });

                // Save the new user
                const users = await newUser.save();

                if (users) {
                    res.send({
                        error: false,
                        message: "saved successfully",
                        data: users
                    });
                }
            } catch (error) {
                console.error(error);
                return res.status(500).json({ "uploaded": false, message: "Server Error" });
            }
        }
    });
};


// Get Users by ID
export const getUsersById = async (req, res) => {
    const users = await Users.find({ _id: new ObjectId(req.params.id), isDelete: false });

    res.send(users)
};

// export const getAgent = async (req, res) => {
//     console.log(req.params.id)
//     //const users = await Users.find({ _id: new ObjectId(req.params.id), isDelete: false });
//     const users = await Users.findById(req.params.id);
//     console.log(users);
//     console.log("the users city id is",users.cityid);
//     const city=await CityPercent.find({_id:new ObjectId(users.cityid)})
//     console.log("the city is",city);
//     res.send(users)
// };
// Ensure you import ObjectId correctly

export const getAgent = async (req, res) => {
  try {
    console.log(req.params.id);

    // Fetch the user by ID
    const user = await Users.findById(req.params.id);
    console.log("User details:", user);

    if (user && user.cityid) {
      // Fetch the city details using cityid
      const city = await CityPercent.findById(user.cityid);
      console.log("City details:", city);

      // Prepare the response as an array
      const responseData = [
        { ...user._doc ,
         cityName: city ? city.city : null }, 
      ];
      console.log(responseData);

      res.status(200).json(responseData);
    } else {
      res.status(404).json({ message: "User or city not found" });
    }
  } catch (error) {
    console.error("Error fetching user or city:", error);
    res.status(500).json({ message: "Internal server error" });
  }
};



// Get all Users (excluding Admin) by usertype
export const getAllUsers = async (req, res) => {
    let query = { 'usertype': { $ne: "admin" }, isDelete: false }
    if (req.query.utype) {
        query = { 'usertype': req.query.utype, isDelete: false }
    }
    const users = await Users.find(query)
        .sort({ createdAt: -1 }).exec();
    res.send(users)
};




// get all agents, using date and zip code
export const getAllAgent = async (req, res) => {
    const pipeline = [
        {
            $match: {
                date: req.params.date
            },
        },
        {
            $group: {
                _id: {
                    date: "$date"
                },
                times: { $push: { time: "$time", zip: "$zip", agent_id: "$agent_id" } }
            }
        },
        {
            $sort: {
                "updatedAt": 1
            }
        },
        {
            $addFields: {
                filteredDates: {
                    $map: {
                        input: "$times",
                        as: "timeArray",
                        in: {
                            time: {
                                $filter: {
                                    input: "$$timeArray.time",
                                    as: "t",
                                    cond: { $eq: ["$$t.zip", req.params.zip] }
                                }
                            },
                            agent_id: "$$timeArray.agent_id"  // Include agent_id in the output
                        }
                    }
                }
            }
        },
        {
            $match: {
                "filteredDates": { $not: { $size: 0 } } // Filter out empty arrays
            }
        },
        {
            $project: {
                _id: 0,
                date: "$_id.date",
                filteredDates: 1
            }
        },
        {
            $unwind: "$filteredDates"  // Unwind to work with each agent_id individually
        },
        {
            $lookup: {
                from: "users",  // Assuming the collection name is "users"
                localField: "filteredDates.agent_id",
                foreignField: "_id",  // Assuming _id in "users" collection
                as: "agent_details"
            }
        },
        {
            $unwind: "$agent_details"  // Unwind to flatten the agent_details array
        },
        {
            $group: {
                _id: null,
                agent_details: { $push: "$agent_details" }  // Push agent details into one array
            }
        },
        {
            $project: {
                _id: 0,
                agent_details: 1
            }
        }
    ];
    
    
    
    
    
    const slot = await Routeslot.aggregate(pipeline);

    console.log(slot);
    res.json(slot);
    
};





// Activate a user
export const activateUserById = async (req, res) => {
    try {
        //   const userId = req.params.id;
        const user = await Users.findByIdAndUpdate({ _id: new ObjectId(req.params.id) }, { isActive: true }, { new: true });
        res.send(user);
    } catch (error) {
        console.error(error);
        res.send({ error: 'Internal Server Error' });
    }
};

// Deactivate a user
export const deactivateUserById = async (req, res) => {
    try {
        //   const userId = req.params.id;
        const user = await Users.findByIdAndUpdate({ _id: new ObjectId(req.params.id) }, { isActive: false }, { new: true });
        res.send(user);
    } catch (error) {
        console.error(error);
        res.send({ error: 'Internal Server Error' });
    }
};


// Update Users by ID
export const updateUsers = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
            return res.send({
                error: true,
                message: "File upload failed"
            });
        }

        // Initialize updateData with non-password fields
        const updateData = {
            $set: {
                "name": req.body.name,
                "email": req.body.email,
                "mobile": req.body.mobile,
                "cityid": req.body.cityid,
                "usertype": req.body.usertype
            }
        };

        // Check if password is provided in the request body
        if (req.body.password) {
            // Hash the provided password
            try {
                const hashedPassword = await bcrypt.hash(req.body.password, 8);
                updateData.$set.password = hashedPassword; // Set the hashed password in updateData
            } catch (hashError) {
                console.error('Error hashing password:', hashError);
                return res.status(500).json({ message: "Password hashing failed" });
            }
        }

        // Add image data to updateData if a file is uploaded
        if (req.file) {
            updateData.$set.image = {
                contentType: 'image',
                data: req.file.filename
            };
            updateData.$set.imageURL = 'uploads/' + req.file.filename;
        }

        try {
            const resp = await Users.updateOne({ _id: req.body._id }, updateData);

            if (resp) {
                res.status(200).json({ message: "update success" });
            } else {
                res.status(500).json({ message: "update failed" });
            }
        } catch (error) {
            console.error(error);
            return res.send({
                error: true,
                message: "Internal server error"
            });
        }
    });
};


// Check email duplicacy
export const duplicateEmail = async (req, res) => {
    const existingEmail = await Users.findOne({ email: req.params.email });
    if (existingEmail) {
        return res.send({ "uploaded": false, message: 'Oops! This email address is already in use with another account.' });
    } else {
        return res.send({ message: 'Unique email address' });
    }
}


// Delete users by ID
export const deleteusersById = async (req, res) => {
    const users = await Users.updateOne({ _id: new ObjectId(req.params.id) }, { $set: { "isActive": false, "isDelete": true } });
    if (users) {
        res.send(
            {
                error: false,
                message: "deleted successfully"
            }
        );
    } else {
        res.send(
            {
                error: true,
                message: "deletion failed"
            }
        );
    }
}


// Agent login through mobile and OTP
export const otpLogin = async (req, res) => {
    const { mobile, device_id } = req.body;

    try {
        const user = await Users.findOne({ mobile });
        if (user) {
            if (user.usertype === "agent" || user.usertype === "partner") {
                if (user.isLogin) {
                    console.log("Agent is already logged in");
                    const existingDevice = await Verifyotp.findOne({ mobile });
                    if (existingDevice && existingDevice.device_id !== device_id) {
                        return res.send({
                            isSuccess: false,
                            message: "device_id is different"
                        });
                    }
                }
            } else {
                return res.send({
                    isSuccess: false,
                    message: "User is not an agent"
                });
            }
            if (!user.isActive) {
                return res.send({
                    isSuccess: false,
                    message: "User not active yet"
                });
            }

        }

        // Send OTP
        const otp = Math.floor(1000 + Math.random() * 9000);
        const headers = {
            'Authorization': 'Basic Q2VpYmE6Wjd5S3V4Qno=',
        };
        const auth = {
            username: 'Ceiba',
            password: 'Z7yKuxBz',
        };

        const response = await axios.post(
            `https://app.crunchysms.com:3001/rest/send_sms?senderId=CEIBA&msg=Use OTP ${otp} to log into your Recycle Pay account. Do not share the OTP or your number with anyone. CEIBA GREEN&destAddr=91${mobile}&command=submit&flash=0&dataCoding=0&PE_ID=1701169770111580785&TEMP_ID=1707169787605858070`,
            null,
            { headers, auth }
        );

        if (response) {
            // const user = await Users.findOne({ mobile, isDelete: false });
            const existingOTP = await Verifyotp.findOne({ mobile });

            if (existingOTP) {
                await Verifyotp.findOneAndUpdate({ mobile }, { $set: { OTP: otp } });
            } else {
                await Verifyotp.create({ mobile, OTP: otp });
            }



            return res.send({
                isSuccess: true,
                message: "OTP sent successfully"
            });
        } else {
            return res.send({
                isSuccess: false,
                message: "Failed to send OTP"
            });
        }
    } catch (error) {
        console.error(error);
        return res.send({
            isSuccess: false,
            message: "Internal server error"
        });
    }
};


// Verify OTP to Login
export const verifyOTP = async (req, res) => {
    const { mobile, OTP, device_id } = req.body;

    if (!mobile || !OTP || !device_id) {
        return res.send({
            error: true,
            message: "Mobile, OTP, and device_id are required fields."
        });
    }
    try {
        const user = await Users.findOne({ mobile, isDelete: false });
        const otpEntry = await Verifyotp.findOne({ mobile });

        if (otpEntry && otpEntry.OTP == OTP) {
            if (!otpEntry.device_id) {
                await Verifyotp.updateOne({ mobile }, { $set: { device_id: device_id } });
                console.log("device_id", device_id);
            };

            if (otpEntry.device_id !== device_id && otpEntry.device_id == "") {
                if (otpEntry.device_id == "") {
                    await Verifyotp.updateOne({ mobile }, { $set: { device_id: device_id } });
                    console.log("device_id", device_id);
                } else if (otpEntry.device_id !== device_id) {
                    return res.send({
                        isSuccess: false,
                        message: "Device ID does not match."
                    });
                }
            }

            if (otpEntry.device_id == "") {
                await Verifyotp.updateOne({ mobile }, { $set: { device_id: device_id } });
                console.log("device_id", device_id);
            };

            console.log(user);

            if (user) {
                await Users.updateOne({ mobile }, { $set: { isLogin: true } });
                console.log(user);
                const token = jwt.sign(
                    {
                        _id: user._id,
                        name: user.name,
                        email: user.email,
                        usertype: user.usertype,
                        mobile: user.mobile,
                    },
                    "AjatusAgent",
                );
                return res.send({
                    _id: user._id,
                    name: user.name,
                    email: user.email,
                    mobile: user.mobile,
                    usertype: user.usertype,
                    imageURL: user.imageURL,
                    userToken: token
                });
            } else {
                return res.send({
                    error: true,
                    message: "User doesn't exist"
                });
            }
        } else {
            return res.send({
                isSuccess: false,
                message: "Wrong OTP entered."
            });
        }
    } catch (error) {
        // console.error(error);
        return res.send({
            error: true,
            message: "Internal server error"
        });
    }
};


// API to handle login from a different device ID
export const loginFromDifferentDevice = async (req, res) => {
    const { mobile, device_id } = req.body;

    if (!mobile || !device_id) {
        return res.send({
            error: true,
            message: "Mobile and device_id are required fields."
        });
    }

    try {
        const user = await Users.findOne({ mobile, isDelete: false });
        const otpEntry = await Verifyotp.findOne({ mobile });

        if (!user) {
            return res.send({
                error: true,
                message: "User doesn't exist"
            });
        }

        if (otpEntry) {
            // Delete existing device ID and set isLogin to false

            await Verifyotp.findOneAndUpdate({ mobile }, { $set: { device_id: "" } });
            await Users.updateOne({ mobile }, { $set: { isLogin: false } });

            // Send new OTP
            const otp = Math.floor(1000 + Math.random() * 9000);
            const headers = {
                'Authorization': 'Basic Q2VpYmE6Wjd5S3V4Qno=',
            };
            const auth = {
                username: 'Ceiba',
                password: 'Z7yKuxBz',
            };

            const response = await axios.post(
                `https://app.crunchysms.com:3001/rest/send_sms?senderId=CEIBA&msg=Use OTP ${otp} to log into your Recycle Pay account. Do not share the OTP or your number with anyone. CEIBA GREEN&destAddr=91${mobile}&command=submit&flash=0&dataCoding=0&PE_ID=1701169770111580785&TEMP_ID=1707169787605858070`,
                null,
                { headers, auth }
            );

            if (response) {
                // await Verifyotp.create({ mobile, OTP: otp, device_id });
                await Verifyotp.findOneAndUpdate({ mobile }, { $set: { OTP: otp } });

                return res.send({
                    isSuccess: true,
                    message: "OTP sent successfully"
                });
            } else {
                return res.send({
                    isSuccess: false,
                    message: "Failed to send OTP"
                });
            }
        } else {
            return res.send({
                isSuccess: false,
                message: "no mobile found in verifyOTP."
            });
        }
    } catch (error) {
        console.log(error);
        return res.send({
            error: true,
            message: "Internal server error"
        });
    }
};


// Agent self register through APP
export const selfRegister = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
            return res.send({ isSuccess: false, uploaded: false, message: "File upload error" });
        } else {
            try {
                console.log('req.files:', req.files);
                // const { name, email, mobile, cityid, aadhar_no, vehicle_type } = req.body;
                const { name, email, mobile, cityid, aadhar_no, vehicle_type, user_type } = req.body;

                // Check if the user with the same mobile already exists
                const existingMobile = await Users.findOne({ mobile: mobile });
                if (existingMobile) {
                    return res.send({ uploaded: false, message: "This mobile number already exists." });
                }

                // Check if the email already exists in the database
                const existingEmail = await Users.findOne({ email: email });
                if (existingEmail) {
                    return res.send({ uploaded: false, message: "Oops! This email address is already in use with another account." });
                }

                const readFileSync = (filepath) => {
                    return fs.readFileSync(filepath);
                };

                console.log("creating new user");
                const newUser = {
                    name,
                    email,
                    usertype: user_type,
                    mobile,
                    cityid,
                    aadhar_no,
                    vehicle_type,
                    // aadhar_imgURL: 'uploads/' + req.files.aadhar_imgURL[0].filename,
                    // vehicle_registrationURL: 'uploads/' + req.files.vehicle_registrationURL[0].filename,
                    // vehicle_insuranceURL: 'uploads/' + req.files.vehicle_insuranceURL[0].filename
                    aadhar_imgURL: null,
                    vehicle_registrationURL: null,
                    vehicle_insuranceURL: null,

                };

                if (req.files.image) {
                    // newUser.image = {
                    //     contentType: req.files.image[0].mimetype,
                    //     data: readFileSync(path.join('uploads', req.files.image[0].filename))
                    // };
                    newUser.imageURL = 'uploads/' + req.files.image[0].filename;
                }

                const createdUser = new Users(newUser);
                const savedUser = await createdUser.save();
                console.log("saved user", savedUser);

                if (savedUser) {
                    const userToken = jwt.sign(
                        {
                            _id: savedUser._id,
                            name: savedUser.name,
                            email: savedUser.email,
                            usertype: savedUser.usertype,
                            mobile: savedUser.mobile,
                        },
                        "AjatusAgent",// Token expires in 1 hour
                    );
                    console.log("new user is saved", savedUser);
                    res.send({
                        error: false,
                        message: "saved successfully",
                        user: {
                            ...savedUser._doc, // Spread the saved user object
                            userToken,         // Add the generated token
                        }
                    });
                } else {
                    console.log("Failed to save new user");
                    res.send({
                        error: true,
                        message: "Failed to save user"
                    });
                }
            } catch (error) {
                console.error(error);
                return res.status(500).json({ uploaded: false, message: "Server Error" });
            }
        }
    });
};
//api for register add agent through admin
 // Assuming you have a Users model

export const registerAgentByAdmin = async (req, res) => {
    upload(req, res, async (err) => {
        if (err) {
            console.log(err);
            return res.send({ isSuccess: false, uploaded: false, message: "File upload error" });
        } else {
            try {
                console.log('req.files:', req.files);
                const { name, email, mobile, cityid, aadhar_no, vehicle_type } = req.body;
                console.log(name);
                console.log(mobile);
                console.log(email);

                // Check if the user with the same mobile already exists
                const existingMobile = await Users.findOne({ mobile: mobile });
                console.log("the existing mobile is",existingMobile);
                if (existingMobile) {
                     res.send({ uploaded: false, message: "mobile number exists." });
                     return;
                }
                // Check if the email already exists in the database
                const existingEmail = await Users.findOne({ email: email });
                console.log("the existing email is",existingEmail);
                if (existingEmail) {
                     res.send({ uploaded: false, message: "mail exist" });
                     return;
                }

                console.log("creating new user");
                const newUser = {
                    name,
                    email,
                    mobile,
                    cityid,
                    usertype :"agent",
                    aadhar_no,
                    isApproved:"true",
                    vehicle_type,
                    // Check if files exist, otherwise set them as null
                    aadhar_imgURL: req.files.aadhar_imgURL ? 'uploads/' + req.files.aadhar_imgURL[0].filename : null,
                    vehicle_registrationURL: req.files.vehicle_registrationURL ? 'uploads/' + req.files.vehicle_registrationURL[0].filename : null,
                    vehicle_insuranceURL: req.files.vehicle_insuranceURL ? 'uploads/' + req.files.vehicle_insuranceURL[0].filename : null,
                };

                if (req.files.image) {
                    newUser.imageURL = 'uploads/' + req.files.image[0].filename;
                }

                const createdUser = new Users(newUser);
                const savedUser = await createdUser.save();
                console.log("saved user", savedUser);

                if (savedUser) {
                    // const userToken = jwt.sign(
                    //     {
                    //         _id: savedUser._id,
                    //         name: savedUser.name,
                    //         email: savedUser.email,
                    //         usertype: savedUser.usertype,
                    //         mobile: savedUser.mobile,
                    //     },
                    //     "AjatusAgent"
                    // );
                    console.log("new user is saved", savedUser);
                    res.send({
                        error: false,
                        message: "saved successfully",
                        user: {
                            ...savedUser._doc // Spread the saved user object
                            // userToken,         // Add the generated token
                        }
                    });
                } else {
                    console.log("Failed to save new user");
                    res.send({
                        error: true,
                        message: "Failed to save user"
                    });
                }
            } catch (error) {
                console.error(error);
                return res.status(500).json({ uploaded: false, message: "Server Error" });
            }
        }
    });
};



// Logout Agent
export const logoutAgent = async (req, res) => {
    const { mobile } = req.params;

    if (!mobile) {
        return res.send({
            error: true,
            message: "Mobile number required."
        });
    }

    try {
        const user = await Users.findOne({ mobile, isDelete: false });
        if (!user) {
            return res.send({
                error: true,
                message: "User doesn't exist"
            });
        }

        const otpEntry = await Verifyotp.findOne({ mobile });
        if (otpEntry) {
            await Users.findOneAndUpdate({ mobile }, { $set: { isLogin: false } });
            await Verifyotp.findOneAndUpdate({ mobile }, { $set: { device_id: "" } });
        }

        return res.send({
            error: false,
            message: "logged out successfully"
        });

    } catch (error) {
        console.error(error);
        return res.send({
            error: true,
            message: "Internal server error"
        });
    }
};


// Get Device ID of an agent
export const getDeviceId = async (req, res) => {
    try {
        const data = await Verifyotp.findOne({ mobile: req.params.mobile });
        if (data) {
            if (data.device_id) {
                return res.send({ isSuccess: true, message: 'Device ID found', data: data.device_id });
            } else {
                return res.send({ isSuccess: false, message: 'Device ID not found' });
            }
        } else {
            return res.send({ isSuccess: false, message: 'Mobile not found' });
        }

    } catch (error) {
        console.error(error);
        return res.send({
            error: true,
            message: "Internal server error"
        });
    }
};

// Update isApprove Status of a user
export const approveUserById = async (req, res) => {
    try {
        const user = await Users.findByIdAndUpdate({ _id: new ObjectId(req.params.userid) }, { isApproved: true }, { new: true });
        console.log("status updated");
        res.send(
            {
                error: false,
                message: "isApproved status updated successfully",
                data: user
            }
        );
    } catch (error) {
        console.error(error);
        res.send({ error: 'Internal Server Error' });
    }
};

// Get all Approved agents only
export const approvedUserslist = async (req, res) => {
    try {
        const query = { usertype: 'agent', isDelete: false, isApproved: true };

        const users = await Users.find(query)
            .sort({ createdAt: -1 })
            .exec();

        res.send(users);
    } catch (error) {
        console.error(error);
        res.status(500).send({ error: 'Internal Server Error' });
    }
};

//controller for delete agent or partner(chandan)
export const deleteAgentOrPartner = async (req, res) => {
    const id = req.params.id;
    if (!id) {
        return res.json({ isSuccess: false, message: "id is required" });
    }
    const updatedUser = await Users.findByIdAndUpdate({ _id: new ObjectId(id) }, {
        isLogin: false,
        isActive: false,
        isDelete: true
    }, { new: true });

    if (!updatedUser) {
        return res.status(404).json({ isSuccess: false, message: 'User not found' });
    }
    res.json({ isSuccess: true, message: 'User deleted successfully' });

}
export const getUserDetails = async (req, res) => {
    const { mobile } = req.params
    try {
        if (!mobile) {

            return res.status(404).json({ isSuccess: false, message: "Mobile number is required" });
        }
        const user = await Users.findOne({ mobile: mobile });
        if (!user) {
            return res.status(404).json({ isSuccess: false, message: "User not found" });
        }
        const userDetails = {
            name: user.name,
            email: user.email,
            usertype: user.usertype,
            mobile: user.mobile,
            cityid: user.cityid,
            isLogin: user.isLogin,
            aadhar_imgURL: user.aadhar_imgURL,
            vehicle_registrationURL: user.vehicle_registrationURL,
            vehicle_insuranceURL: user.vehicle_insuranceURL,
            isApproved: user.isApproved,
            dl_imgURL: user.dl_imgURL || null,
            isActive: user.isActive,
            isDelete: user.isDelete,
        };
        res.status(200).json({ isSuccess: true, userDetails });
    }
    catch (err) {
        console.log("Print error", err)
        res.send({ isSuccess: false, message: 'Internal Server Error' });
    }
}
export const updateUserImages = async (req, res) => {
    const { mobile } = req.params
    try {
        if (!mobile) {
            return res.status(404).json({ isSuccess: false, message: "Mobile number is required" });
        }
        const user = await Users.findOne({ mobile: mobile });
        if (!user) {
            return res.status(404).json({ isSuccess: false, message: "User not found" });
        }
        upload(req, res, async (err) => {
            if (err) {
                console.error(err);
                return res.send({ isSuccess: false, message: "File upload error", error: err });
            }

            // Update the user's image URLs if files are uploaded
            if (req.files.aadhar_imgURL) {
                user.aadhar_imgURL = 'uploads/' + req.files.aadhar_imgURL[0].filename;
            }
            if (req.files.vehicle_registrationURL) {
                user.vehicle_registrationURL = 'uploads/' + req.files.vehicle_registrationURL[0].filename;
            }
            if (req.files.vehicle_insuranceURL) {
                user.vehicle_insuranceURL = 'uploads/' + req.files.vehicle_insuranceURL[0].filename;
            }
            if (req.files.image) {
                user.imageURL = 'uploads/' + req.files.image[0].filename;
            }
            if (req.files.DL) {
                user.dl_imgURL = 'uploads/' + req.files.DL[0].filename;
            }

            // Save the updated user document
            const updatedUser = await user.save();
            res.send({ isSuccess: true, message: "User images updated successfully", user: updatedUser });
        });

    } catch (error) {
        console.error(error);
        res.send({ error: 'Internal Server Error' });
    }
};
export const updateAgentByAdmin=async(req,res)=>{
    upload(req,res,async(err)=>{
       if(err){
        console.log("Error in update agent");
        console.log(err);
        return res.send({isSuccess:false,message:"Error in update Agent",error:err});
       }
       else{
        try{
         const {name,email,cityid,aadhar_no,mobile}=req.body;
        //  console.log(name);
        //  console.log(email);
        //  console.log(cityid);
        //  console.log(aadhar_no);
        //  console.log(mobile);
        //  const img=req.files;
        //  console.log(img);
      
        const updateData={
            $set:{
                name:name,
                email:email,
                cityid:cityid,
                aadhar_no:aadhar_no,
                mobile:mobile,
                aadhar_imgURL: req.files.aadhar_imgURL ? 'uploads/' + req.files.aadhar_imgURL[0].filename : null,
                imageURL: req.files.image ? 'uploads/' + req.files.image[0].filename : null,
                vehicle_registrationURL: req.files.vehicle_registrationURL ? 'uploads/' + req.files.vehicle_registrationURL[0].filename : null,
                vehicle_insuranceURL: req.files.vehicle_insuranceURL ? 'uploads/' + req.files.vehicle_insuranceURL[0].filename : null
            }
           
        }
        console.log(updateData);
          const response=await Users.updateOne({ _id:req.body._id},updateData);
          if(response){
            res.status(200).json({message:"Agent data updated successfully"})
          }
          else
          res.status(500).json({message:"Error in update Admin server error"})
          
        }catch(error){
            console.log(error,"error in update user");
        }
       }
    })
}






// Payment mode active from admin-side
export const paymentAct = async (req, res) => {
    await Users.find({ _id: new ObjectId(req.params.adm), isActive: true, isDelete: false }).then(async (getadmin) => {
        console.log("getadmin", getadmin);        
        if (getadmin[0].paymentStatus == true) {
            let trueStatus = await Users.findByIdAndUpdate({ _id: new ObjectId(getadmin[0]._id) }, { $set: { paymentStatus: false } })
            if (trueStatus) {
                return res.status(200).json({ isSuccess: true, paymentStatus: false, data: trueStatus }); 
            }
        } else if (getadmin[0].paymentStatus == false) {
            let trueStatus = await Users.findByIdAndUpdate({ _id: new ObjectId(getadmin[0]._id) }, { $set: { paymentStatus: true } })
            if (trueStatus) {
                return res.status(200).json({ isSuccess: true, paymentStatus: true, data: trueStatus }); 
            }
        }
    })
}





// get Payment mode active -> use by admin
export const getPaymentAct = async (req, res) => {
    await Users.find({ _id: new ObjectId(req.params.adm), isActive: true, isDelete: false }).then(async (getadmin) => {
        return res.status(200).json({ isSuccess: true, data: getadmin[0] }); 
    })
}











//code by om
//make today slot available for customer 
//here the sameday slot will set to true or false
export const samedayslotshow = async (req, res) => {
    await Users.find({ _id: new ObjectId(req.params.adm), isActive: true, isDelete: false }).then(async (getadmin) => {
        console.log("getadmin", getadmin);        
        if (getadmin[0].samedayslotStatus == true) {
            let trueStatus = await Users.findByIdAndUpdate({ _id: new ObjectId(getadmin[0]._id) }, { $set: { samedayslotStatus: false } })
            if (trueStatus) {
                return res.status(200).json({ isSuccess: true, samedayslotStatus: false }); 
            }
        } else if (getadmin[0].samedayslotStatus == false) {
            let trueStatus = await Users.findByIdAndUpdate({ _id: new ObjectId(getadmin[0]._id) }, { $set: { samedayslotStatus: true } })
            if (trueStatus) {
                return res.status(200).json({ isSuccess: true, samedayslotStatus: true }); 
            }
        }
    })
}
//checking same day slot status
export const samedayslotstatus = async (req, res) => {
    await Users.find({ _id: new ObjectId(req.params.adm), isActive: true, isDelete: false }).then(async (getadmin) => {
        return res.status(200).json({ isSuccess: true, data: getadmin[0] }); 
    })
}
export const updateAgentByAdmin=async(req,res)=>{
    upload(req,res,async(err)=>{
       if(err){
        console.log("Error in update agent");
        console.log(err);
        return res.send({isSuccess:false,message:"Error in update Agent",error:err});
       }
       else{
        try{
         const {name,email,cityid,aadhar_no,mobile}=req.body;
        //  console.log(name);
        //  console.log(email);
        //  console.log(cityid);
        //  console.log(aadhar_no);
        //  console.log(mobile);
        //  const img=req.files;
        //  console.log(img);
      
        const updateData={
            $set:{
                name:name,
                email:email,
                cityid:cityid,
                aadhar_no:aadhar_no,
                mobile:mobile,
                aadhar_imgURL: req.files.aadhar_imgURL ? 'uploads/' + req.files.aadhar_imgURL[0].filename : null,
                imageURL: req.files.image ? 'uploads/' + req.files.image[0].filename : null,
                vehicle_registrationURL: req.files.vehicle_registrationURL ? 'uploads/' + req.files.vehicle_registrationURL[0].filename : null,
                vehicle_insuranceURL: req.files.vehicle_insuranceURL ? 'uploads/' + req.files.vehicle_insuranceURL[0].filename : null
            }
           
        }
        console.log(updateData);
          const response=await Users.updateOne({ _id:req.body._id},updateData);
          if(response){
            res.status(200).json({message:"Agent data updated successfully"})
          }
          else
          res.status(500).json({message:"Error in update Admin server error"})
          
        }catch(error){
            console.log(error,"error in update user");
        }
       }
    })
}
//function to genearate forgot password otp and send to the user through mail
async function sendOtpViaMail(userEmail,otpp){
    var defaultClient = SibApiV3Sdk.ApiClient.instance;
    var apiKey = defaultClient.authentications['api-key'];
    console.log("the api key is", apiKey)
    apiKey.apiKey = process.env.MAILAPIKEY;
    var apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
    var adminMail = new SibApiV3Sdk.SendSmtpEmail();

    //here we will write the logic for generate random password
    

    //to convert the time into indian time
    const getIndianTime = () => {
        return new Date().toLocaleTimeString('en-IN', { 
            timeZone: 'Asia/Kolkata', 
            hour: '2-digit', 
            minute: '2-digit', 
            second: '2-digit', 
            hour12: false 
        });
    };
    try {

        adminMail = {
            to: [
                {
                    email:userEmail 
                }
            ],
            from: {
                email: 'support@ceibagreen.com'
            },
            templateId: 28,
            params: {
                parameter: 'recycle_pay',
                subject: `Forgot password otp ${otpp}`,
                date: new Date().toLocaleDateString(),
                time: getIndianTime(),
                otp: otpp,
                email: userEmail

            },
            subject: `One Time Password (OTP) for change Password`,
            headers: {
                'X-Mailin-custom': 'custom_header_1:custom_value_1|custom_header_2:custom_value_2'
            },
        }
        const data = await apiInstance.sendTransacEmail(adminMail);
        console.log("mail send successfully", data.messageId);
    } catch (error) {
        console.log(error);
    }
}
//api for send the mmail which contains forgot password otp
export const sendOtpByMail = async (req, res) => {
    const email = req.body.email;
    // generate a random OTP
    const otpp = Math.floor(1000 + Math.random() * 9000);
    console.log("The entered email is", email);

    try {
        const mailExist = await customers.findOne({ email });
        if (!mailExist) {
            return res.json({ message: "This email is not registered with us" });
        } 

        const result = await Verifyotp.findOneAndUpdate(
            { email },
            { OTP: otpp },
            { new: true, upsert: true }
        );
       // await sendOtpViaMail(email, otpp);
        if (result) {
             await sendOtpViaMail(email, otpp);
            console.log("OTP sent successfully");
            return res.status(200).json({ message: "OTP sent to your email", otp: otpp });
        }
        
    } catch (error) {
        console.error("Error in OTP sending:", error);
        return res.status(500).json({ message: "Internal server error in OTP sending" });
    }
}
//api for verify the otp in admin side
export const sendOtpByMailToAdmin = async (req, res) => {
    const email = req.body.email;
    // generate a random OTP
    const otpp = Math.floor(1000 + Math.random() * 9000);
    console.log("The entered email is", email);

    try {
        const mailExist = await Users.findOne({ email });
        if (!mailExist) {
            return res.json({ message: "This email is not registered with us" });
        } 

        const result = await Verifyotp.findOneAndUpdate(
            { email },
            { OTP: otpp },
            { new: true, upsert: true }
        );
       // await sendOtpViaMail(email, otpp);
        if (result) {
             await sendOtpViaMail(email, otpp);
            console.log("OTP sent successfully");
            return res.status(200).json({ message: "OTP sent to your email", otp: otpp });
        }
        
    } catch (error) {
        console.error("Error in OTP sending:", error);
        return res.status(500).json({ message: "Internal server error in OTP sending" });
    }
}
//api for verify the otp 
export const verifyOtpPassword = async (req,res) => {
    const otp=req.body.otp;
    const email=req.body.email
    console.log("the otp is",otp);
    console.log(email);
   try{
    const previousOtp = await Verifyotp.findOne({email},
        { OTP: 1, _id: 0 } 
    );
    console.log(previousOtp.OTP);
       if(otp==previousOtp.OTP){
         res.status(200).json({message:"verification successfull"});
       }
       else
         res.json({message:"Incorrect otp verification failed"})

     }catch(error){
     console.log(error);
   }
   }

//api for update the password in the customer db for the new updated password
 export const  updatePassword = async(req,res)=>{
    try{
    const email=req.body.email;
    const password=req.body.password;
    console.log("the updated email",email);
    console.log("the updated password is",password);
            const hashedPassword=await bcrypt.hash(password,8);
            console.log(hashedPassword);
            //here we will update the password in the table
            const result=await customers.updateOne({email},{$set:{password:hashedPassword}});
            if(result){
                console.log("password updated ");
                return  res.status(200).json({message:"password updated successfully"});         
    }
    else 
    return res.json({message:"Password updation failed"});
    }catch(error){
   console.log(error);
   res.status(500).send("failed in password updation ")
  }
  }
  //api for update the password in the user table 
  export const  updatePasswordByAdmin = async(req,res)=>{
    try{
    const email=req.body.email;
    const password=req.body.password;
    console.log("the updated email",email);
    console.log("the updated password is",password);
            const hashedPassword=await bcrypt.hash(password,8);
            console.log(hashedPassword);
            //here we will update the password in the table
            const result=await Users.updateOne({email},{$set:{password:hashedPassword}});
            if(result){
                console.log("password updated ");
                return  res.status(200).json({message:"password updated successfully"});         
    }
    else 
    return res.json({message:"Password updation failed"});
    }catch(error){
   console.log(error);
   res.status(500).send("failed in password updation ")
  }
  }

